@@ -54,6 +54,11 @@
 #include "float_cast.h"
 #include <stdarg.h>
 
+#define LPC_ORDER 24
+#if !defined(FIXED_POINT) || defined(NEW_PLC)
+#include "plc.c"
+#endif
+
 static const celt_word16 preemph = QCONST16(0.8f,15);
 
 #ifdef FIXED_POINT
@@ -644,11 +649,12 @@ int celt_encode_float(CELTEncoder * restrict st, const celt_sig * pcm, celt_sig
             && norm_rate < 50;
    if (has_pitch)
    {
-      /* FIXME: Should probably do a stack save/pop here */
       VARDECL(celt_word16, x_lp);
+      SAVE_STACK;
       ALLOC(x_lp, (2*N-2*N4)>>1, celt_word16);
       pitch_downsample(in, x_lp, 2*N-2*N4, N, C, &st->xmem, &st->pitch_buf[MAX_PERIOD>>1]);
       pitch_search(st->mode, x_lp, st->pitch_buf, 2*N-2*N4, MAX_PERIOD-(2*N-2*N4), &pitch_index, &st->xmem);
+      RESTORE_STACK;
    }
 
    /* Deferred allocation after find_spectral_pitch() to reduce 
@@ -1089,6 +1095,10 @@ struct CELTDecoder {
 
    celt_word16 *oldBandE;
    
+#ifdef NEW_PLC
+   float *lpc;
+#endif
+
    int last_pitch_index;
    int loss_count;
 };
@@ -1163,9 +1173,16 @@ CELTDecoder *celt_decoder_create(const CELTMode *mode, int channels, int *error)
    
    st->preemph_memD = (celt_sig*)celt_alloc(C*sizeof(celt_sig));
 
+#ifdef NEW_PLC
+   st->lpc = (float*)celt_alloc(C*LPC_ORDER*sizeof(float));
+#endif
+
    st->loss_count = 0;
 
    if ((st->decode_mem!=NULL) && (st->out_mem!=NULL) && (st->oldBandE!=NULL) &&
+#ifdef NEW_PLC
+         (st->lpc!=NULL) &&
+#endif
        (st->preemph_memD!=NULL))
    {
       if (error)
@@ -1208,22 +1225,21 @@ void celt_decoder_destroy(CELTDecoder *st)
    celt_free(st->decode_mem);
    celt_free(st->oldBandE);
    celt_free(st->preemph_memD);
+
+#ifdef NEW_PLC
+   celt_free(st->lpc);
+#endif
    
    st->marker = DECODERFREED;
    
    celt_free(st);
 }
 
-/** Handles lost packets by just copying past data with the same
-    offset as the last
-    pitch period */
-#ifdef NEW_PLC
-#include "plc.c"
-#else
 static void celt_decode_lost(CELTDecoder * restrict st, celt_word16 * restrict pcm)
 {
    int c, N;
    int pitch_index;
+   int overlap = st->mode->overlap;
    celt_word16 fade = Q15ONE;
    int i, len;
    VARDECL(celt_sig, freq);
@@ -1231,7 +1247,6 @@ static void celt_decode_lost(CELTDecoder * restrict st, celt_word16 * restrict p
    int offset;
    SAVE_STACK;
    N = st->block_size;
-   ALLOC(freq,C*N, celt_sig); /**< Interleaved signal MDCTs */
    
    len = N+st->mode->overlap;
    
@@ -1241,10 +1256,10 @@ static void celt_decode_lost(CELTDecoder * restrict st, celt_word16 * restrict p
       celt_word32 tmp=0;
       celt_word32 mem0[2]={0,0};
       celt_word16 mem1[2]={0,0};
-      /*find_spectral_pitch(st->mode, st->mode->fft, &st->mode->psy, st->out_mem+MAX_PERIOD-len, st->out_mem, st->mode->window, NULL, len, MAX_PERIOD-len-100, &pitch_index, C);*/
-      /* FIXME: Should do a bit of interpolation while decimating */
-      pitch_downsample(st->out_mem, pitch_buf, MAX_PERIOD, MAX_PERIOD, C, mem0, mem1);
-      pitch_search(st->mode, pitch_buf+((MAX_PERIOD-len)>>1), pitch_buf, len, MAX_PERIOD-len-100, &pitch_index, &tmp);
+      pitch_downsample(st->out_mem, pitch_buf, MAX_PERIOD, MAX_PERIOD,
+                       C, mem0, mem1);
+      pitch_search(st->mode, pitch_buf+((MAX_PERIOD-len)>>1), pitch_buf, len,
+                   MAX_PERIOD-len-100, &pitch_index, &tmp);
       pitch_index = MAX_PERIOD-len-pitch_index;
       st->last_pitch_index = pitch_index;
    } else {
@@ -1255,7 +1270,9 @@ static void celt_decode_lost(CELTDecoder * restrict st, celt_word16 * restrict p
          fade = 0;
    }
 
+#ifndef NEW_PLC
    offset = MAX_PERIOD-pitch_index;
+   ALLOC(freq,C*N, celt_sig); /**< Interleaved signal MDCTs */
    while (offset+len >= MAX_PERIOD)
       offset -= pitch_index;
    compute_mdcts(st->mode, 0, st->out_mem+offset*C, freq, C);
@@ -1265,24 +1282,117 @@ static void celt_decode_lost(CELTDecoder * restrict st, celt_word16 * restrict p
    CELT_MOVE(st->out_mem, st->out_mem+C*N, C*(MAX_PERIOD+st->mode->overlap-N));
    /* Compute inverse MDCTs */
    compute_inv_mdcts(st->mode, 0, freq, -1, 0, st->out_mem, C);
-
+#else
    for (c=0;c<C;c++)
    {
-      int j;
-      for (j=0;j<N;j++)
+      celt_word32 e[MAX_PERIOD];
+      float exc[MAX_PERIOD];
+      float ac[LPC_ORDER+1];
+      float decay = 1;
+      float S1=0;
+      float mem[LPC_ORDER]={0};
+
+      offset = MAX_PERIOD-pitch_index;
+      for (i=0;i<MAX_PERIOD;i++)
+         exc[i] = st->out_mem[i*C+c];
+
+      if (st->loss_count == 0)
       {
-         celt_sig tmp = MAC16_32_Q15(st->out_mem[C*(MAX_PERIOD-N)+C*j+c],
-                                preemph,st->preemph_memD[c]);
-         st->preemph_memD[c] = tmp;
-         pcm[C*j+c] = SCALEOUT(SIG2WORD16(tmp));
+         _celt_autocorr(exc, ac, st->mode->window, st->mode->overlap,
+                        LPC_ORDER, MAX_PERIOD);
+
+         /* Noise floor -50 dB */
+         ac[0] *= 1.00001;
+         /* Lag windowing */
+         for (i=1;i<=LPC_ORDER;i++)
+         {
+            /*ac[i] *= exp(-.5*(2*M_PI*.002*i)*(2*M_PI*.002*i));*/
+            ac[i] -= ac[i]*(.008*i)*(.008*i);
+         }
+
+         _celt_lpc(st->lpc, ac, LPC_ORDER);
       }
+      fir(exc, st->lpc, exc, MAX_PERIOD, LPC_ORDER, mem);
+
+      /* Check if the waveform is decaying (and if so how fast) */
+      {
+         float E1=0, E2=0;
+         int period;
+         if (pitch_index <= MAX_PERIOD/2)
+            period = pitch_index;
+         else
+            period = MAX_PERIOD/2;
+         for (i=0;i<period;i++)
+         {
+            E1 += exc[MAX_PERIOD-period+i]*exc[MAX_PERIOD-period+i];
+            E2 += exc[MAX_PERIOD-2*period+i]*exc[MAX_PERIOD-2*period+i];
+         }
+         decay = sqrt((E1+1)/(E2+1));
+         if (decay > 1)
+            decay = 1;
+      }
+
+      /* Copy excitation, taking decay into account */
+      for (i=0;i<len+st->mode->overlap;i++)
+      {
+         if (offset+i >= MAX_PERIOD)
+         {
+            offset -= pitch_index;
+            decay *= decay;
+         }
+         e[i] = decay*exc[offset+i];
+         S1 += st->out_mem[offset+i]*1.*st->out_mem[offset+i];
+      }
+
+      iir(e, st->lpc, e, len+st->mode->overlap, LPC_ORDER, mem);
+
+      {
+         float S2=0;
+         for (i=0;i<len+overlap;i++)
+            S2 += e[i]*1.*e[i];
+         /* This checks for an "explosion" in the synthesis (including NaNs) */
+         if (!(S1 > 0.2f*S2))
+         {
+            for (i=0;i<len+overlap;i++)
+               e[i] = 0;
+         } else if (S1 < S2)
+         {
+            float ratio = sqrt((S1+1)/(S2+1));
+            for (i=0;i<len+overlap;i++)
+               e[i] *= ratio;
+         }
+      }
+
+      for (i=0;i<MAX_PERIOD+st->mode->overlap-N;i++)
+         st->out_mem[C*i+c] = st->out_mem[C*(N+i)+c];
+
+      /* Apply TDAC to the concealed audio so that it blends with the
+         previous and next frames */
+      for (i=0;i<overlap/2;i++)
+      {
+         celt_word32 tmp1, tmp2;
+         tmp1 = MULT16_32_Q15(st->mode->window[i          ], e[i          ]) -
+                MULT16_32_Q15(st->mode->window[overlap-i-1], e[overlap-i-1]);
+         tmp2 = MULT16_32_Q15(st->mode->window[i],           e[N+overlap-1-i]) +
+                MULT16_32_Q15(st->mode->window[overlap-i-1], e[N+i          ]);
+         tmp1 = MULT16_32_Q15(fade, tmp1);
+         tmp2 = MULT16_32_Q15(fade, tmp2);
+         st->out_mem[C*(MAX_PERIOD+i)+c] = MULT16_32_Q15(st->mode->window[overlap-i-1], tmp2);
+         st->out_mem[C*(MAX_PERIOD+overlap-i-1)+c] = MULT16_32_Q15(st->mode->window[i], tmp2);
+         st->out_mem[C*(MAX_PERIOD-N+i)+c] += MULT16_32_Q15(st->mode->window[i], tmp1);
+         st->out_mem[C*(MAX_PERIOD-N+overlap-i-1)+c] -= MULT16_32_Q15(st->mode->window[overlap-i-1], tmp1);
+      }
+      for (i=0;i<N-overlap;i++)
+         st->out_mem[C*(MAX_PERIOD-N+overlap+i)+c] = MULT16_32_Q15(fade, e[overlap+i]);
    }
+#endif
+
+   deemphasis(st->out_mem, pcm, N, C, preemph, st->preemph_memD);
    
    st->loss_count++;
 
    RESTORE_STACK;
 }
-#endif
 
 #ifdef FIXED_POINT
 int celt_decode(CELTDecoder * restrict st, const unsigned char *data, int len, celt_int16 * restrict pcm)
@@ -1337,8 +1447,6 @@ int celt_decode_float(CELTDecoder * restrict st, const unsigned char *data, int
       celt_decode_lost(st, pcm);
       RESTORE_STACK;
       return 0;
-   } else {
-      st->loss_count = 0;
    }
    if (len<0) {
      RESTORE_STACK;
@@ -1427,7 +1535,7 @@ int celt_decode_float(CELTDecoder * restrict st, const unsigned char *data, int
    compute_inv_mdcts(st->mode, shortBlocks, freq, transient_time, transient_shift, st->out_mem, C);
 
    deemphasis(st->out_mem, pcm, N, C, preemph, st->preemph_memD);
-
+   st->loss_count = 0;
    RESTORE_STACK;
    return 0;
 }
