@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -34,13 +34,14 @@ static HANDLE hMapObject = NULL;
 static HANDLE hHookMutex = NULL;
 static HHOOK hhookWnd = 0;
 
-SharedMem *sm;
+SharedMem *sm = NULL;
 HANDLE hSharedMutex = NULL;
 HMODULE hSelf = NULL;
 static BOOL bMumble = FALSE;
 static BOOL bDebug = FALSE;
 
 static HardHook hhLoad;
+static HardHook hhLoadW;
 void *HardHook::pCode = NULL;
 unsigned int HardHook::uiCode = 0;
 
@@ -124,6 +125,9 @@ void *HardHook::cloneCode(void **porig) {
 			case 0x5e:
 			case 0x5f:
 				break;
+			case 0x68:
+				extra = 4;
+				break;
 			case 0x81: // CMP immediate
 				extra = modrmbytes(a,b) + 5;
 				break;
@@ -134,7 +138,7 @@ void *HardHook::cloneCode(void **porig) {
 				extra = modrmbytes(a,b) + 1;
 				break;
 			default:
-				fods("Unknown opcode %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x", o[0], o[1], o[2], o[3], o[4], o[5], o[6], o[7], o[8], o[9], o[10], o[11]);
+				fods("Unknown opcode at %d: %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x %2x", idx-1, o[0], o[1], o[2], o[3], o[4], o[5], o[6], o[7], o[8], o[9], o[10], o[11]);
 				return NULL;
 				break;
 		}
@@ -399,6 +403,7 @@ static const char *blacklist[] = {
 	"vlc.exe",
 	"dbgview.exe",
 	"opera.exe",
+	"explorer.exe",
 	NULL
 };
 
@@ -412,6 +417,7 @@ static HMODULE WINAPI MyLoadLibrary(const char *lpFileName) {
 
 	if (! bMumble) {
 		checkD3D9Hook();
+		checkDXGIHook();
 		checkOpenGLHook();
 	}
 
@@ -419,6 +425,24 @@ static HMODULE WINAPI MyLoadLibrary(const char *lpFileName) {
 	return h;
 }
 
+typedef HMODULE(__stdcall *LoadLibraryWType)(const wchar_t *);
+static HMODULE WINAPI MyLoadLibraryW(const wchar_t *lpFileName) {
+	LoadLibraryWType oLoadLibrary = (LoadLibraryWType) hhLoadW.call;
+	hhLoadW.restore();
+
+	HMODULE h = oLoadLibrary(lpFileName);
+	ods("Library %ls wloaded to %p", lpFileName, h);
+
+	if (! bMumble) {
+		checkD3D9Hook();
+		checkDXGIHook();
+		checkOpenGLHook();
+	}
+
+	hhLoadW.inject();
+	return h;
+}
+
 static LRESULT CALLBACK CallWndProc(int nCode, WPARAM wParam, LPARAM lParam) {
 	return CallNextHookEx(hhookWnd, nCode, wParam, lParam);
 }
@@ -458,6 +482,10 @@ extern "C" __declspec(dllexport) void __cdecl InstallHooks() {
 	}
 }
 
+extern "C" __declspec(dllexport) unsigned int __cdecl GetOverlayMagicVersion() {
+	return OVERLAY_MAGIC_NUMBER;
+}
+
 extern "C" __declspec(dllexport) SharedMem * __cdecl GetSharedMemory() {
 	return sm;
 }
@@ -474,7 +502,7 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 				char *p = strrchr(procname, '\\');
 
 				if (p) {
-					if (_stricmp(p+1, "mumble.exe")==0)
+					if ((_stricmp(p+1, "mumble.exe")==0) || (_stricmp(p+1, "mumble11x.exe")==0))
 						bMumble = TRUE;
 					int i =0;
 					while (blacklist[i]) {
@@ -516,7 +544,9 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 					return TRUE;
 				}
 
-				hMapObject = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, sizeof(SharedMem) + sizeof(Direct3D9Data), "MumbleSharedMemory");
+				DWORD dwSharedSize = sizeof(SharedMem) + sizeof(Direct3D9Data) + sizeof(DXGIData);
+
+				hMapObject = CreateFileMapping(INVALID_HANDLE_VALUE, NULL, PAGE_READWRITE, 0, dwSharedSize, "MumbleSharedMemory");
 				if (hMapObject == NULL) {
 					ods("Lib: CreateFileMapping failed");
 					ReleaseMutex(hSharedMutex);
@@ -525,18 +555,20 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 
 				bool bInit = (GetLastError() != ERROR_ALREADY_EXISTS);
 
-				sm = (SharedMem *) MapViewOfFile(hMapObject, FILE_MAP_ALL_ACCESS, 0, 0, 0);
-
-				unsigned char *raw = (unsigned char *) sm;
-				d3dd = (Direct3D9Data *)(raw + sizeof(SharedMem));
+				sm = (SharedMem *) MapViewOfFile(hMapObject, FILE_MAP_ALL_ACCESS, 0, 0, dwSharedSize);
 
 				if (sm == NULL) {
 					ods("MapViewOfFile Failed");
 					ReleaseMutex(hSharedMutex);
 					return TRUE;
 				}
+
+				unsigned char *raw = (unsigned char *) sm;
+				d3dd = (Direct3D9Data *)(raw + sizeof(SharedMem));
+				dxgi = (DXGIData *)(raw + sizeof(SharedMem) + sizeof(Direct3D9Data));
+
 				if (bInit) {
-					memset(sm, 0, sizeof(SharedMem) + sizeof(Direct3D9Data));
+					memset(sm, 0, sizeof(SharedMem) + sizeof(Direct3D9Data) + sizeof(DXGIData));
 					sm->lastAppAlive = 0;
 					sm->bHooked = false;
 					sm->bDebug = false;
@@ -553,9 +585,11 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 
 				if (! bMumble) {
 					hhLoad.setup(reinterpret_cast<voidFunc>(LoadLibraryA), reinterpret_cast<voidFunc>(MyLoadLibrary));
+					hhLoadW.setup(reinterpret_cast<voidFunc>(LoadLibraryW), reinterpret_cast<voidFunc>(MyLoadLibraryW));
 
 					// Hm. Don't check D3D9 as apparantly it's creation causes problems in some applications.
 					checkD3D9Hook(true);
+					checkDXGIHook(true);
 					checkOpenGLHook();
 					ods("Injected");
 				}
@@ -564,6 +598,7 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 		case DLL_PROCESS_DETACH: {
 				ods("Lib: ProcDetach: %s", procname);
 				hhLoad.restore(true);
+				hhLoadW.restore(true);
 				if (sm)
 					UnmapViewOfFile(sm);
 				if (hMapObject)
@@ -576,9 +611,10 @@ extern "C" BOOL WINAPI DllMain(HINSTANCE, DWORD fdwReason, LPVOID) {
 			break;
 		case DLL_THREAD_ATTACH: {
 				static bool bTriedHook = false;
-				if (! bTriedHook && ! bMumble) {
+				if (sm && ! bTriedHook && ! bMumble) {
 					bTriedHook = true;
 					checkD3D9Hook();
+					checkDXGIHook();
 					checkOpenGLHook();
 				}
 			}
