@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -373,6 +373,9 @@ void checkOpenGLHook() {
 			fods("OpenGL: Unhooked OpenGL App %s", procname);
 			bHooked = true;
 
+			// Add a ref to ourselves; we do NOT want to get unloaded directly from this process.
+			GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS, reinterpret_cast<char *>(&checkOpenGLHook), &hSelf);
+
 			INJECT(wglSwapBuffers);
 			// INJECT(wglSwapLayerBuffers);
 
