@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2008, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -31,6 +31,8 @@
 #ifndef _INTERNAL_OVERLAY_H
 #define _INTERNAL_OVERLAY_H
 
+#define OVERLAY_MAGIC_NUMBER 0x00000005
+
 #define TEXT_WIDTH 600
 #define TEXT_HEIGHT 60
 #define NUM_TEXTS 32
@@ -39,13 +41,13 @@
 
 #define OVERLAY_VERSION_MAJ 1
 #define OVERLAY_VERSION_MIN 1
-#define OVERLAY_VERSION_PATCH 4
+#define OVERLAY_VERSION_PATCH 7
 #define OVERLAY_VERSION_SUB 0
 
 struct TextEntry {
 	unsigned int color;
 	wchar_t text[128];
-	bool bUpdated;
+	unsigned int uiCounter;
 	short width;
 	unsigned char texture[TEXTURE_SIZE];
 };
