@@ -1,3 +1,34 @@
+/* Copyright (C) 2009-2010, Stefan Hacker <dD0t@users.sourceforge.net>
+   Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+
+   All rights reserved.
+
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+
+   - Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
+   - Redistributions in binary form must reproduce the above copyright notice,
+     this list of conditions and the following disclaimer in the documentation
+     and/or other materials provided with the distribution.
+   - Neither the name of the Mumble Developers nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+
 #define _USE_MATH_DEFINES
 
 #include <stdio.h>
@@ -5,11 +36,15 @@
 #include <windows.h>
 #include <tlhelp32.h>
 #include <math.h>
+#include <string>
+#include <sstream>
 
 #include "../mumble_plugin.h"
 
 HANDLE h = NULL;
 
+using namespace std;
+
 static DWORD getProcess(const wchar_t *exename) {
 	PROCESSENTRY32 pe;
 	DWORD pid = 0;
@@ -38,7 +73,7 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 }
 
 static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from COD4 (v 1.7.568). IP:Port context without team discriminator.", L"Mumble COD4 Plugin", MB_OK);
+	::MessageBox(h, L"Reads audio position information from COD4 (v1.7.568). IP:Port context without team discriminator.", L"Mumble COD4 Plugin", MB_OK);
 }
 
 static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &context, std::wstring &identity) {
@@ -93,9 +128,13 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	    Get context string; in this plugin this will be an
 	    ip:port (char 256 bytes) string
 	*/
+
 	ccontext[127] = 0;
 	context = std::string(ccontext);
 
+	if (context.find(':')==string::npos)
+		context.append(":28960");
+
 	// Scale Coordinates
 	/*
 	   Z-Value is increasing when heading north
@@ -171,7 +210,7 @@ static void unlock() {
 }
 
 static const std::wstring longdesc() {
-	return std::wstring(L"Supports Call of Duty 4 v1.7.568 only. Only supports context.");
+	return std::wstring(L"Supports Call of Duty 4 v1.7.568 only. No identity support yet.");
 }
 
 static std::wstring description(L"Call of Duty 4 v1.7.568");
