@@ -17,7 +17,7 @@ BYTE *posptr;
 BYTE *rotptr;
 BYTE *stateptr;
 BYTE *hostptr;
-BYTE *teamptr;
+//BYTE *teamptr;
 
 static DWORD getProcess(const wchar_t *exename) {
 	PROCESSENTRY32 pe;
@@ -68,7 +68,7 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 }
 
 static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Counter-Strike: Source (Build 3945). IP:Port context with team discriminator.", L"Mumble CSS Plugin", MB_OK);
+	::MessageBox(h, L"Reads audio position information from Counter-Strike: Source (Build 3945). IP:Port context support.", L"Mumble CSS Plugin", MB_OK);
 }
 
 static bool calcout(float *pos, float *rot, float *opos, float *front, float *top) {
@@ -107,40 +107,47 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	bool ok;
 	char state;
 	char chHostStr[40];
-	BYTE bTeam;
-	string sTeam;
+	string sHost;
+	//BYTE bTeam;
+	//string sTeam;
 	wostringstream new_identity;
 	ostringstream new_context;
 
 	ok = peekProc(posptr, ipos, 12) &&
 	     peekProc(rotptr, rot, 12) &&
 	     peekProc(stateptr, &state, 1) &&
-	     peekProc(hostptr, chHostStr, 40) &&
-	     peekProc(teamptr, &bTeam, 1);
+	     peekProc(hostptr, chHostStr, 40); //&&
+	//peekProc(teamptr, &bTeam, 1);
 	if (!ok)
 		return false;
+
 	chHostStr[39] = 0;
 
+	sHost.assign(chHostStr);
+	if (sHost.find(':')==string::npos)
+		sHost.append(":27015");
 
-	switch (bTeam) {
-		case 60:
-			sTeam = "Terrorists";
-			break;
-		case 61:
-			sTeam = "Counter Terrorists";
-			break;
-		default:
-			sTeam = "Unknown";
-			break;
-	}
+	/*	switch (bTeam) {
+			case 60:
+				sTeam = "Terrorists";
+				break;
+			case 61:
+				sTeam = "Counter Terrorists";
+				break;
+			default:
+				sTeam = "Unknown";
+				break;
+		}
+	*/
 
 	new_context << "<context>"
 	<< "<game>css</game>"
-	<< "<hostport>" << chHostStr << "</hostport>"
-	<< "<team>" << sTeam << "</team>"
+	<< "<hostport>" << sHost << "</hostport>"
 	<< "</context>";
 	context = new_context.str();
 
+	//<< "<team>" << sTeam << "</team>"
+
 	/* TODO
 	new_identity << "<identity>"
 			<< "<name>" << "SAS" << "</name>"
@@ -177,7 +184,7 @@ static int trylock() {
 	BYTE *mod=getModuleAddr(pid, L"client.dll");
 	if (!mod)
 		return false;
-	BYTE *mod_engine = getModuleAddr(pid, L"engine.dll");
+	BYTE *mod_engine=getModuleAddr(pid, L"engine.dll");
 	if (!mod_engine)
 		return false;
 
@@ -199,7 +206,7 @@ static int trylock() {
 	rotptr = mod + 0x39d504;
 	stateptr = mod + 0x390070;
 	hostptr = mod_engine + 0x3909c4;
-	teamptr = mod + 0x3aa133;
+	//teamptr = mod + 0x3aa133;
 
 	//Gamecheck
 	char sMagic[16];
