@@ -2,16 +2,21 @@
 #include <stdlib.h>
 #include <windows.h>
 #include <tlhelp32.h>
+#include <string>
+#include <sstream>
 
 #define _USE_MATH_DEFINES
 #include <math.h>
 
 #include "../mumble_plugin.h"
 
+using namespace std;
+
 HANDLE h;
 BYTE *posptr;
 BYTE *rotptr;
 BYTE *stateptr;
+BYTE *hostptr;
 
 static DWORD getProcess(const wchar_t *exename) {
 	PROCESSENTRY32 pe;
@@ -62,7 +67,7 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 }
 
 static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Day of Defeat: Source (Build 3897)", L"Mumble DODS Plugin", MB_OK);
+	::MessageBox(h, L"Reads audio position information from Day of Defeat: Source (Build 4057). IP:Port context support.", L"Mumble DODS Plugin", MB_OK);
 }
 
 static bool calcout(float *pos, float *rot, float *opos, float *front, float *top) {
@@ -93,74 +98,43 @@ static bool calcout(float *pos, float *rot, float *opos, float *front, float *to
 	return true;
 }
 
-static int trylock() {
-	h = NULL;
-	posptr = rotptr = NULL;
-
-	DWORD pid=getProcess(L"hl2.exe");
-	if (!pid)
-		return false;
-	BYTE *mod=getModuleAddr(pid, L"client.dll");
-	if (!mod)
-		return false;
-	h=OpenProcess(PROCESS_VM_READ, false, pid);
-	if (!h)
-		return false;
-
-	// Check if we really have DODS running
-	/*
-		position tuple:		client.dll+0x454210  (x,y,z, float)
-		orientation tuple:	client.dll+0x442560  (v,h float)
-		ID string:			client.dll+0x3f4d19 = "DODSpectatorGUI@@" (17 characters, text)
-		spawn state:        client.dll+0x3e0b7c  (0 when at main menu, 2 when at team selection menu or when not spawned, and between 4 and 5 when spawned, byte)
-	*/
-	char sMagic[17];
-	if (!peekProc(mod + 0x3f4d19, sMagic, 17) || strncmp("DODSpectatorGUI@@", sMagic, 17)!=0)
-		return false;
-
-	// Remember addresses for later
-	posptr = mod + 0x454210;
-	rotptr = mod + 0x442560;
-	stateptr = mod + 0x3e0b7c;
-
-	float pos[3];
-	float rot[3];
-	float opos[3], top[3], front[3];
-
-	bool ok = peekProc(posptr, pos, 12) &&
-	          peekProc(rotptr, rot, 12);
-
-	if (ok)
-		return calcout(pos, rot, opos, top, front);
-	// If it failed clean up
-	CloseHandle(h);
-	h = NULL;
-	return false;
-}
-
-static void unlock() {
-	if (h) {
-		CloseHandle(h);
-		h = NULL;
-	}
-	return;
-}
-
-static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &context, std::wstring &identity) {
+static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, string &context, wstring &identity) {
 	for (int i=0;i<3;i++)
 		avatar_pos[i] = avatar_front[i] = avatar_top[i] = 0;
 
 	float ipos[3], rot[3];
 	bool ok;
 	char state;
+	char chHostStr[40];
+	string sHost;
+	wostringstream new_identity;
+	ostringstream new_context;
 
 	ok = peekProc(posptr, ipos, 12) &&
 	     peekProc(rotptr, rot, 12) &&
-	     peekProc(stateptr, &state, 1);
+	     peekProc(stateptr, &state, 1) &&
+	     peekProc(hostptr, chHostStr, 40);
 	if (!ok)
 		return false;
 
-	// Check to see if you are spawned
+	chHostStr[39] = 0;
+
+	sHost.assign(chHostStr);
+	if (sHost.find(':')==string::npos)
+
+		new_context << "<context>"
+		<< "<game>dods</game>"
+		<< "<hostport>" << sHost << "</hostport>"
+		<< "</context>";
+	context = new_context.str();
+
+	/* TODO
+	new_identity << "<identity>"
+			<< "<name>" << "SAS" << "</name>"
+		     << "</identity>";
+	identity = new_identity.str(); */
+
+	// Check to see if you are in a server
 	if (state == 0 || state == 2)
 		return true; // Deactivate plugin
 
@@ -179,11 +153,73 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	return false;
 }
 
-static const std::wstring longdesc() {
-	return std::wstring(L"Supports DODS build 3897. No identity or context support yet.");
+static int trylock() {
+
+	h = NULL;
+	posptr = rotptr = NULL;
+
+	DWORD pid=getProcess(L"hl2.exe");
+	if (!pid)
+		return false;
+	BYTE *mod=getModuleAddr(pid, L"client.dll");
+	if (!mod)
+		return false;
+	BYTE *mod_engine=getModuleAddr(pid, L"engine.dll");
+	if (!mod_engine)
+		return false;
+
+	h=OpenProcess(PROCESS_VM_READ, false, pid);
+	if (!h)
+		return false;
+
+	// Check if we really have DODS running
+	/*
+		position tuple:		client.dll+0x3f62a0  (x,y,z, float)
+		orientation tuple:	client.dll+0x3f6220  (v,h float)
+		ID string:			client.dll+0x3f6d91 = "DODSpectatorGUI@@" (17 characters, text)
+		spawn state:        client.dll+0x3e2b94  (0 when at main menu, 2 when at team selection menu or when not spawned, and between 5 and 6 when spawned, byte)
+		host string: 		client.dll+0x<address> (ip:port zero-terminated string)
+	*/
+
+	// Remember addresses for later
+	posptr = mod + 0x3FB358;
+	rotptr = mod + 0x3FB2CC;
+	stateptr = mod + 0x3E7BDC;
+	hostptr = mod_engine + 0x3C91A4;
+
+	// Gamecheck
+	char sMagic[17];
+	if (!peekProc(mod + 0x3FBE49, sMagic, 17) || strncmp("DODSpectatorGUI@@", sMagic, 17)!=0)
+		return false;
+
+	// Check if we can get meaningful data from it
+	float apos[3], afront[3], atop[3];
+	float cpos[3], cfront[3], ctop[3];
+	wstring sidentity;
+	string scontext;
+
+	if (fetch(apos, afront, atop, cpos, cfront, ctop, scontext, sidentity))
+		return true;
+
+	// If it failed clean up
+	CloseHandle(h);
+	h = NULL;
+	return false;
 }
 
-static std::wstring description(L"Day of Defeat: Source (Build 3897)");
+static void unlock() {
+	if (h) {
+		CloseHandle(h);
+		h = NULL;
+	}
+	return;
+}
+
+static const std::wstring longdesc() {
+	return std::wstring(L"Supports DODS build 4057. No identity support yet.");
+}
+
+static std::wstring description(L"Day of Defeat: Source (Build 4057)");
 static std::wstring shortname(L"Day of Defeat: Source");
 
 static MumblePlugin dodsplug = {
