@@ -67,7 +67,7 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 }
 
 static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Dystopia (Build 3945). IP:Port context without team descriminator.", L"Mumble Dystopia Plugin", MB_OK);
+	::MessageBox(h, L"Reads audio position information from Dystopia (Build 3945). IP:Port context support.", L"Mumble Dystopia Plugin", MB_OK);
 }
 
 static bool calcout(float *pos, float *rot, float *opos, float *front, float *top) {
@@ -106,6 +106,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	bool ok;
 	char state;
 	char chHostStr[40];
+	string sHost;
 	wostringstream new_identity;
 	ostringstream new_context;
 
@@ -115,11 +116,16 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	     peekProc(hostptr, chHostStr, 40);
 	if (!ok)
 		return false;
+
 	chHostStr[39] = 0;
 
+	sHost.assign(chHostStr);
+	if (sHost.find(':')==string::npos)
+		sHost.append(":27015");
+
 	new_context << "<context>"
 	<< "<game>dys</game>"
-	<< "<hostport>" << chHostStr << "</hostport>"
+	<< "<hostport>" << sHost << "</hostport>"
 	<< "</context>";
 	context = new_context.str();
 
@@ -211,7 +217,7 @@ static void unlock() {
 }
 
 static const std::wstring longdesc() {
-	return std::wstring(L"Supports Dystopia build 3945. No identity or context support yet.");
+	return std::wstring(L"Supports Dystopia build 3945. No identity support yet.");
 }
 
 static std::wstring description(L"Dystopia (Build 3945)");
