@@ -66,6 +66,10 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 	return (ok && (r == len));
 }
 
+static void about(HWND h) {
+	::MessageBox(h, L"Reads audio position information from Garry's Mod 11 (Build 3943). IP:Port context support.", L"Mumble Gmod Plugin", MB_OK);
+}
+
 static bool calcout(float *pos, float *rot, float *opos, float *front, float *top) {
 	float h = rot[0];
 	float v = rot[1];
@@ -102,6 +106,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	bool ok;
 	char state;
 	char chHostStr[40];
+	string sHost;
 	wostringstream new_identity;
 	ostringstream new_context;
 
@@ -111,11 +116,16 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	     peekProc(hostptr, chHostStr, 40);
 	if (!ok)
 		return false;
+
 	chHostStr[39] = 0;
 
+	sHost.assign(chHostStr);
+	if (sHost.find(':')==string::npos)
+		sHost.append(":27015");
+
 	new_context << "<context>"
 	<< "<game>hl2dm</game>"
-	<< "<hostport>" << chHostStr << "</hostport>"
+	<< "<hostport>" << sHost << "</hostport>"
 	<< "</context>";
 	context = new_context.str();
 
@@ -151,7 +161,7 @@ static int trylock() {
 	BYTE *mod=getModuleAddr(pid, L"client.dll");
 	if (!mod)
 		return false;
-	BYTE *mod_engine = getModuleAddr(pid, L"engine.dll");
+	BYTE *mod_engine=getModuleAddr(pid, L"engine.dll");
 	if (!mod_engine)
 		return false;
 	h=OpenProcess(PROCESS_VM_READ, false, pid);
@@ -162,7 +172,7 @@ static int trylock() {
 	/*
 		position tuple:		client.dll+0x3dcad4  (x,y,z, float)
 		orientation tuple:	client.dll+0x3dcae0  (v,h float)
-		ID string:		client.dll+0x3a5674 = "Dm/$" (4 characters, text)
+		ID string:			client.dll+0x3a5674 = "Dm/$" (4 characters, text)
 		spawn state:		client.dll+0x37e180  (0 when at main menu, 2 when not spawned, 7 when spawned, byte)
 		ip:port string		engine.dll+0x3909c4  (zero terminated ip:port, string)
 	*/
@@ -200,11 +210,6 @@ static void unlock() {
 	return;
 }
 
-
-static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Half-Life 2: Deathmatch (Build 3945)", L"Mumble HL2DM Plugin", MB_OK);
-}
-
 static const wstring longdesc() {
 	return wstring(L"Supports HL2DM build 3945. No identity support yet.");
 }
