@@ -1,3 +1,34 @@
+/* Copyright (C) 2009-2010, Snares <snares@users.sourceforge.net>
+   Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+
+   All rights reserved.
+
+   Redistribution and use in source and binary forms, with or without
+   modification, are permitted provided that the following conditions
+   are met:
+
+   - Redistributions of source code must retain the above copyright notice,
+     this list of conditions and the following disclaimer.
+   - Redistributions in binary form must reproduce the above copyright notice,
+     this list of conditions and the following disclaimer in the documentation
+     and/or other materials provided with the distribution.
+   - Neither the name of the Mumble Developers nor the names of its
+     contributors may be used to endorse or promote products derived from this
+     software without specific prior written permission.
+
+   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+   ``AS IS'' AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+   LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+   A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE FOUNDATION OR
+   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
+   EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
+   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+   LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+*/
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <windows.h>
@@ -67,7 +98,7 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 }
 
 static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Team Fortress 2 (Build 4013). IP:Port context without team discriminator.", L"Mumble TF2 Plugin", MB_OK);
+	::MessageBox(h, L"Reads audio position information from Team Fortress 2 (Build 4057). IP:Port context support.", L"Mumble TF2 Plugin", MB_OK);
 }
 
 static bool calcout(float *pos, float *rot, float *opos, float *front, float *top) {
@@ -106,6 +137,7 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 	bool ok;
 	char state;
 	char chHostStr[40];
+	string sHost;
 	wostringstream new_identity;
 	ostringstream new_context;
 
@@ -116,8 +148,13 @@ static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, floa
 
 	if (!ok)
 		return false;
+
 	chHostStr[39] = 0;
 
+	sHost.assign(chHostStr);
+	if (sHost.find(':')==string::npos)
+		sHost.append(":27015");
+
 	new_context << "<context>"
 	<< "<game>tf2</game>"
 	<< "<hostport>" << chHostStr << "</hostport>"
@@ -179,14 +216,14 @@ static int trylock() {
 	*/
 
 	// Remember addresses for later
-	posptr = mod + 0x52A6A4;
-	rotptr = mod + 0x4D6B78;
-	stateptr = mod + 0x4BDBF4;
+	posptr = mod_engine + 0x555BB4;
+	rotptr = mod_engine + 0x3CDC30;
+	stateptr = mod + 0x4EABCC;
 	hostptr = mod_engine + 0x3C91A4;
 
 	// Gamecheck
 	char sMagic[9];
-	if (!peekProc(mod + 0x4CE38B, sMagic, 9) || strncmp("teamJet@@", sMagic, 9)!=0)
+	if (!peekProc(mod + 0x4FB3E3, sMagic, 9) || strncmp("teamJet@@", sMagic, 9)!=0)
 		return false;
 
 	// Check if we can get meaningful data from it
@@ -213,10 +250,10 @@ static void unlock() {
 }
 
 static const std::wstring longdesc() {
-	return std::wstring(L"Supports TF2 build 4013. No identity or context support yet.");
+	return std::wstring(L"Supports TF2 build 4057. No identity support yet.");
 }
 
-static std::wstring description(L"Team Fortress 2 (Build 4013)");
+static std::wstring description(L"Team Fortress 2 (Build 4057)");
 static std::wstring shortname(L"Team Fortress 2");
 
 static MumblePlugin tf2plug = {
