@@ -38,16 +38,15 @@ static bool peekProc(VOID *base, VOID *dest, SIZE_T len) {
 }
 
 static void about(HWND h) {
-	::MessageBox(h, L"Reads audio position information from Wolfenstein: Enemy Territory (v 2.60b)", L"Mumble W:ET plugin", MB_OK);
+	::MessageBox(h, L"Reads audio position information from Wolfenstein: Enemy Territory (v2.60b)", L"Mumble W:ET plugin", MB_OK);
 }
 
-
-static int fetch(float *pos, float *front, float *top) {
+static int fetch(float *avatar_pos, float *avatar_front, float *avatar_top, float *camera_pos, float *camera_front, float *camera_top, std::string &context, std::wstring &identity) {
 	float viewHor, viewVer;
 	char state;
 
 	for (int i=0;i<3;i++)
-		pos[i]=front[i]=top[i]=0.0f;
+		avatar_pos[i]=avatar_front[i]=avatar_top[i]=0.0f;
 
 	bool ok;
 
@@ -70,15 +69,17 @@ static int fetch(float *pos, float *front, float *top) {
 	if (state != 32)
 		return true; // If this magic value is !=32 we are spectating, so switch off PA
 
-	ok = peekProc((BYTE *) 0x013F79CC, pos+1, 4) &&	//Y
-	     peekProc((BYTE *) 0x013E8CF4, pos, 4) &&	//X
-	     peekProc((BYTE *) 0x013E8CF8, pos+2, 4) && //Z
+	ok = peekProc((BYTE *) 0x013F79CC, avatar_pos+1, 4) &&	//Y
+	     peekProc((BYTE *) 0x013E8CF4, avatar_pos, 4) &&	//X
+	     peekProc((BYTE *) 0x013E8CF8, avatar_pos+2, 4) && //Z
 	     peekProc((BYTE *) 0x013F9E20, &viewHor, 4) && //Hor
 	     peekProc((BYTE *) 0x013F9E1C, &viewVer, 4); //Ver
 
 	if (! ok)
 		return false;
 
+	avatar_top[2] = -1; // Head movement is in front vector
+
 	/*
 	   Z-Value is increasing when heading north
 				  decreasing when heading south
@@ -105,9 +106,15 @@ static int fetch(float *pos, float *front, float *top) {
 	viewVer *= static_cast<float>(M_PI / 180.0f);
 	viewHor *= static_cast<float>(M_PI / 180.0f);
 
-	front[0] = cos(viewVer) * cos(viewHor);
-	front[1] = -sin(viewVer);
-	front[2] = cos(viewVer) * sin(viewHor);
+	avatar_front[0] = cos(viewVer) * cos(viewHor);
+	avatar_front[1] = -sin(viewVer);
+	avatar_front[2] = cos(viewVer) * sin(viewHor);
+
+	for (int i=0;i<3;i++) {
+		camera_pos[i] = avatar_pos[i];
+		camera_front[i] = avatar_front[i];
+		camera_top[i] = avatar_top[i];
+	}
 
 	return ok;
 }
@@ -122,8 +129,11 @@ static int trylock() {
 	if (!h)
 		return false;
 
-	float pos[3], front[3], top[3];
-	if (fetch(pos, front, top))
+	float apos[3], afront[3], atop[3], cpos[3], cfront[3], ctop[3];
+	std::string context;
+	std::wstring identity;
+
+	if (fetch(apos, afront, atop, cpos, cfront, ctop, context, identity))
 		return true;
 
 	CloseHandle(h);
@@ -138,14 +148,22 @@ static void unlock() {
 	}
 }
 
+static const std::wstring longdesc() {
+	return std::wstring(L"Supports Wolfenstien: Enemy Territory v2.60b. No context or identity support yet.");
+}
+
+static std::wstring description(L"Wolfenstein: Enemy Territory v2.60b");
+static std::wstring shortname(L"Wolfenstein: Enemy Territory");
+
 static MumblePlugin wolfetplug = {
 	MUMBLE_PLUGIN_MAGIC,
-	L"Wolfenstein: Enemy Territory 2.60b",
-	L"Wolfenstein: Enemy Territory",
+	description,
+	shortname,
 	about,
 	NULL,
 	trylock,
 	unlock,
+	longdesc,
 	fetch
 };
 
