@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -47,6 +47,9 @@ Channel::Channel(int id, const QString &name, QObject *p) : QObject(p) {
 	cParent = qobject_cast<Channel *>(p);
 	if (cParent)
 		cParent->addChannel(this);
+#ifdef MUMBLE
+	uiPermissions = 0;
+#endif
 }
 
 Channel::~Channel() {
@@ -94,7 +97,7 @@ bool Channel::lessThan(const Channel *first, const Channel *second) {
 	if (first->iPosition != second->iPosition)
 		return first->iPosition < second->iPosition;
 	else
-		return first->qsName < second->qsName;
+		return QString::localeAwareCompare(first->qsName, second->qsName) < 0;
 }
 
 bool Channel::isLinked(Channel *l) const {
