@@ -33,10 +33,12 @@
 
 #include "murmur_pch.h"
 
-class Player;
+class User;
 class Group;
 class ChanACL;
 
+class ClientUser;
+
 class Channel : public QObject {
 	private:
 		Q_OBJECT
@@ -45,45 +47,49 @@ class Channel : public QObject {
 		QSet<Channel *> qsUnseen;
 	public:
 		int iId;
+		int iPosition;
+		bool bTemporary;
 		Channel *cParent;
 		QString qsName;
 		QString qsDesc;
 		QList<Channel *> qlChannels;
-		QList<Player *> qlPlayers;
+		QList<User *> qlUsers;
 		QHash<QString, Group *> qhGroups;
 		QList<ChanACL *> qlACL;
 
 		QSet<Channel *> qsPermLinks;
 		QHash<Channel *, int> qhLinks;
 
-		typedef QPair<Player *, Channel *> qpPlayerLink;
-		QSet<qpPlayerLink> qsPlayerLinks;
-
 		bool bInheritACL;
 
+		Channel(int id, const QString &name, QObject *p = NULL);
+		~Channel();
+
+#ifdef MUMBLE
+		unsigned int uiPermissions;
+
 		static QHash<int, Channel *> c_qhChannels;
 		static QReadWriteLock c_qrwlChannels;
 
-		Channel(int id, const QString &name, const QString &desc, QObject *p = NULL);
-		~Channel();
-
 		static Channel *get(int);
-		static Channel *add(int, const QString &, const QString & = QString());
+		static Channel *add(int, const QString &);
 		static void remove(Channel *);
 
+		void addClientUser(ClientUser *p);
+#endif
+		static bool lessThan(const Channel *, const Channel *);
+
 		void addChannel(Channel *c);
 		void removeChannel(Channel *c);
-		void addPlayer(Player *p);
-		void removePlayer(Player *p);
+		void addUser(User *p);
+		void removeUser(User *p);
 
-		bool isLinked(Channel *c);
+		bool isLinked(Channel *c) const;
 		void link(Channel *c);
-		void unlink(Channel *c);
-
-		void playerLink(Channel *c, Player *p);
-		void playerUnlink(Channel *c, Player *p);
+		void unlink(Channel *c = NULL);
 
 		QSet<Channel *> allLinks();
+		QSet<Channel *> allChildren();
 
 		operator const QString() const;
 };
