@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -31,6 +31,19 @@
 #include "Message.h"
 #include "Connection.h"
 
+/*!
+  \fn void Connection::socketRead()
+  This function waits until a complete package is received and then emits it as a message.
+  It gets called everytime new data is available and interprets the message prefix header
+  to figure out the type and length. It then waits until the complete message is buffered
+  and emits it as a message so it can be handled by the corresponding message handler
+  routine.
+
+  \see QSslSocket::readyRead()
+  \see void ServerHandler::message(unsigned int msgType, const QByteArray &qbaMsg)
+  \see void Server::message(unsigned int uiType, const QByteArray &qbaMsg, ServerUser *u)
+*/
+
 #ifdef Q_OS_UNIX
 #include <sys/socket.h>
 #include <sys/types.h>
@@ -48,6 +61,12 @@ Connection::Connection(QObject *p, QSslSocket *qtsSock) : QObject(p) {
 	iPacketLength = -1;
 	bDisconnectedEmitted = false;
 
+	static bool bDeclared = false;
+	if (! bDeclared) {
+		bDeclared = true;
+		qRegisterMetaType<QAbstractSocket::SocketError>("QAbstractSocket::SocketError");
+	}
+
 	connect(qtsSocket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(socketError(QAbstractSocket::SocketError)));
 	connect(qtsSocket, SIGNAL(encrypted()), this, SIGNAL(encrypted()));
 	connect(qtsSocket, SIGNAL(readyRead()), this, SLOT(socketRead()));
@@ -57,6 +76,8 @@ Connection::Connection(QObject *p, QSslSocket *qtsSock) : QObject(p) {
 #ifdef Q_OS_WIN
 	dwFlow = 0;
 #endif
+
+
 }
 
 Connection::~Connection() {
@@ -83,6 +104,16 @@ void Connection::setToS() {
 		if (setsockopt(qtsSocket->socketDescriptor(), IPPROTO_IP, IP_TOS, &val, sizeof(val)))
 			qWarning("Connection: Failed to set TOS for TCP Socket");
 	}
+#if defined(SO_PRIORITY)
+	socklen_t optlen = sizeof(val);
+	if (getsockopt(qtsSocket->socketDescriptor(), SOL_SOCKET, SO_PRIORITY, &val, &optlen) == 0) {
+		if (val == 0) {
+			val = 6;
+			setsockopt(qtsSocket->socketDescriptor(), SOL_SOCKET, SO_PRIORITY, &val, sizeof(val));
+		}
+	}
+#endif
+
 #endif
 }
 
@@ -94,15 +125,15 @@ void Connection::socketRead() {
 	while (true) {
 		qint64 iAvailable = qtsSocket->bytesAvailable();
 		if (iPacketLength == -1) {
-			if (iAvailable < 4)
+			if (iAvailable < 6)
 				return;
 
-			unsigned char a_ucBuffer[4];
+			unsigned char a_ucBuffer[6];
 
-			qtsSocket->read(reinterpret_cast<char *>(a_ucBuffer), 4);
-			uiType = a_ucBuffer[0];
-			iPacketLength = ((a_ucBuffer[1] << 16) & 0xff0000) + ((a_ucBuffer[2] << 8) & 0xff00) + a_ucBuffer[3];
-			iAvailable -= 4;
+			qtsSocket->read(reinterpret_cast<char *>(a_ucBuffer), 6);
+			uiType = qFromBigEndian(* reinterpret_cast<quint16 *>(& a_ucBuffer[0]));
+			iPacketLength = qFromBigEndian(* reinterpret_cast<quint32 *>(& a_ucBuffer[2]));
+			iAvailable -= 6;
 		}
 
 		if ((iPacketLength == -1) || (iAvailable < iPacketLength))
@@ -117,10 +148,10 @@ void Connection::socketRead() {
 	}
 }
 
-void Connection::socketError(QAbstractSocket::SocketError) {
+void Connection::socketError(QAbstractSocket::SocketError err) {
 	if (! bDisconnectedEmitted) {
 		bDisconnectedEmitted = true;
-		emit connectionClosed(qtsSocket->errorString());
+		emit connectionClosed(err, qtsSocket->errorString());
 	}
 }
 
@@ -135,7 +166,7 @@ void Connection::proceedAnyway() {
 void Connection::socketDisconnected() {
 	if (! bDisconnectedEmitted) {
 		bDisconnectedEmitted = true;
-		emit connectionClosed(QString());
+		emit connectionClosed(QAbstractSocket::UnknownSocketError, QString());
 	}
 }
 
@@ -143,14 +174,12 @@ void Connection::messageToNetwork(const ::google::protobuf::Message &msg, unsign
 	int len = msg.ByteSize();
 	if (len > 0x7fffff)
 		return;
-	cache.resize(len + 4);
+	cache.resize(len + 6);
 	unsigned char *uc = reinterpret_cast<unsigned char *>(cache.data());
-	uc[0] = static_cast<unsigned char>(msgType);
-	uc[1] = static_cast<unsigned char>((len >> 16) & 0xFF);
-	uc[2] = static_cast<unsigned char>((len >> 8) & 0xFF);
-	uc[3] = static_cast<unsigned char>(len & 0xFF);
+	* reinterpret_cast<quint16 *>(& uc[0]) = qToBigEndian(static_cast<quint16>(msgType));
+	* reinterpret_cast<quint32 *>(& uc[2]) = qToBigEndian(static_cast<quint32>(len));
 
-	msg.SerializeToArray(uc + 4, len);
+	msg.SerializeToArray(uc + 6, len);
 }
 
 void Connection::sendMessage(const ::google::protobuf::Message &msg, unsigned int msgType, QByteArray &cache) {
