@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -30,16 +30,22 @@
 
 #include "Group.h"
 #include "Channel.h"
-#include "Player.h"
+#include "User.h"
+#ifdef MURMUR
+#include "Server.h"
+#endif
 
-Group::Group(Channel *assoc, QString name) {
+Group::Group(Channel *assoc, const QString &name) {
 	c = assoc;
 	bInherit = true;
 	bInheritable = true;
 	qsName = name;
-	c->qhGroups[name] = this;
+	if (c)
+		c->qhGroups[name] = this;
 }
 
+#ifdef MURMUR
+
 QSet<int> Group::members() {
 	QStack<Group *> s;
 	QSet<int> m;
@@ -116,34 +122,59 @@ QSet<QString> Group::groupNames(Channel *chan) {
 #define RET_FALSE (invert ? true : false)
 #define RET_TRUE (invert ? false : true)
 
-bool Group::isMember(Channel *curChan, Channel *aclChan, QString name, Player *pl) {
+bool Group::isMember(Channel *curChan, Channel *aclChan, QString name, ServerUser *pl) {
 	Channel *p;
 	Channel *c;
 	Group *g;
 
 	bool m = false;
 	bool invert = false;
+	bool token = false;
+	bool hash = false;
 	c = curChan;
 
-	if (name.isEmpty())
-		return false;
+	while (1) {
+		if (name.isEmpty())
+			return false;
 
-	if (name.startsWith(QChar::fromAscii('!'))) {
-		invert = true;
-		name = name.remove(0,1);
+		if (name.startsWith(QChar::fromAscii('!'))) {
+			invert = true;
+			name = name.remove(0,1);
+			continue;
+		}
+
+		if (name.startsWith(QChar::fromAscii('~'))) {
+			c = aclChan;
+			name = name.remove(0,1);
+			continue;
+		}
+
+		if (name.startsWith(QChar::fromAscii('#'))) {
+			token = true;
+			name = name.remove(0,1);
+			continue;
+		}
+		if (name.startsWith(QChar::fromAscii('$'))) {
+			hash = true;
+			name = name.remove(0,1);
+			continue;
+		}
+
+		break;
 	}
 
-	if (name.startsWith(QChar::fromAscii('~'))) {
-		c = aclChan;
-		name = name.remove(0,1);
-	}
-
-	if (name == QLatin1String("none"))
+	if (token)
+		m = pl->qslAccessTokens.contains(name, Qt::CaseInsensitive);
+	else if (hash)
+		m = pl->qsHash == name;
+	else if (name == QLatin1String("none"))
 		m = false;
 	else if (name == QLatin1String("all"))
 		m = true;
 	else if (name == QLatin1String("auth"))
 		m = (pl->iId >= 0);
+	else if (name == QLatin1String("strong"))
+		m = pl->bVerified;
 	else if (name == QLatin1String("in"))
 		m = (pl->cChannel == c);
 	else if (name == QLatin1String("out"))
@@ -225,7 +256,7 @@ bool Group::isMember(Channel *curChan, Channel *aclChan, QString name, Player *p
 
 		while (! s.isEmpty()) {
 			g = s.pop();
-			if (g->qsAdd.contains(pl->iId) || g->qsTemporary.contains(pl->iId))
+			if (g->qsAdd.contains(pl->iId) || g->qsTemporary.contains(pl->iId) || g->qsTemporary.contains(- static_cast<int>(pl->uiSession)))
 				m = true;
 			if (g->qsRemove.contains(pl->iId))
 				m = false;
@@ -233,3 +264,5 @@ bool Group::isMember(Channel *curChan, Channel *aclChan, QString name, Player *p
 	}
 	return invert ? !m : m;
 }
+
+#endif
