@@ -239,8 +239,8 @@ void AudioInputDialog::updateBitrate() {
 
 	audiorate = q;
 
-	// 50 packets, in bits, IP + UDP + Crypt + type + seq
-	overhead = 100 * 8 * (20 + 8 + 4 + 1 + 2);
+	// 50 packets, in bits, IP + UDP + Crypt + type + seq + frameheader
+	overhead = 100 * 8 * (20 + 8 + 4 + 1 + 2 + p);
 
 	// TCP is 12 more bytes than UDP
 	if (NetworkConfig::TcpModeEnabled())
@@ -266,7 +266,7 @@ void AudioInputDialog::updateBitrate() {
 
 	qlBitrate->setPalette(pal);
 
-	QString v = tr("%1kbit/s (Audio %2 %5, Position %4, Overhead %3)").arg(total / 1000.0, 0, 'f', 1).arg(audiorate / 1000.0, 0, 'f', 1).arg(overhead / 1000.0, 0, 'f', 1).arg(posrate / 1000.0, 0, 'f', 1).arg(::AudioInput::preferCELT(q,p) ? tr("CELT") : tr("Speex"));
+	QString v = tr("%1 kbit/s (Audio %2 %5, Position %4, Overhead %3)").arg(total / 1000.0, 0, 'f', 1).arg(audiorate / 1000.0, 0, 'f', 1).arg(overhead / 1000.0, 0, 'f', 1).arg(posrate / 1000.0, 0, 'f', 1).arg(::AudioInput::preferCELT(q,p) ? tr("CELT") : tr("Speex"));
 	qlBitrate->setText(v);
 
 	if (p == 1) {
@@ -420,7 +420,9 @@ void AudioOutputDialog::load(const Settings &r) {
 
 	loadSlider(qsDelay, r.iOutputDelay);
 	loadSlider(qsVolume, iroundf(r.fVolume * 100.0f));
-	loadSlider(qsOtherVolume, iroundf(r.fOtherVolume * 100.0f));
+	loadSlider(qsOtherVolume, iroundf((1.0f - r.fOtherVolume) * 100.0f));
+	loadCheckBox(qcbAttenuateOthersOnTalk, r.bAttenuateOthersOnTalk);
+	loadCheckBox(qcbAttenuateOthers, r.bAttenuateOthers);
 	loadSlider(qsJitter, r.iJitterBufferSize);
 	loadComboBox(qcbLoopback, r.lmLoopMode);
 	loadSlider(qsPacketDelay, static_cast<int>(r.dMaxPacketDelay));
@@ -431,12 +433,17 @@ void AudioOutputDialog::load(const Settings &r) {
 	loadSlider(qsBloom, iroundf(r.fAudioBloom * 100.0f));
 	loadCheckBox(qcbHeadphones, r.bPositionalHeadphone);
 	loadCheckBox(qcbPositional, r.bPositionalAudio);
+
+	qsOtherVolume->setEnabled(r.bAttenuateOthersOnTalk || r.bAttenuateOthers);
+	qlOtherVolume->setEnabled(r.bAttenuateOthersOnTalk || r.bAttenuateOthers);
 }
 
 void AudioOutputDialog::save() const {
 	s.iOutputDelay = qsDelay->value();
 	s.fVolume = static_cast<float>(qsVolume->value()) / 100.0f;
-	s.fOtherVolume = static_cast<float>(qsOtherVolume->value()) / 100.0f;
+	s.fOtherVolume = 1.0f - (static_cast<float>(qsOtherVolume->value()) / 100.0f);
+	s.bAttenuateOthersOnTalk = qcbAttenuateOthersOnTalk->isChecked();
+	s.bAttenuateOthers = qcbAttenuateOthers->isChecked();
 	s.iJitterBufferSize = qsJitter->value();
 	s.qsAudioOutput = qcbSystem->currentText();
 	s.lmLoopMode = static_cast<Settings::LoopMode>(qcbLoopback->currentIndex());
@@ -481,6 +488,8 @@ void AudioOutputDialog::on_qcbSystem_currentIndexChanged(int) {
 		}
 		bool canmute = aor->canMuteOthers();
 		qsOtherVolume->setEnabled(canmute);
+		qcbAttenuateOthersOnTalk->setEnabled(canmute);
+		qcbAttenuateOthers->setEnabled(canmute);
 		qlOtherVolume->setEnabled(canmute);
 
 		bool usesdelay = aor->usesOutputDelay();
@@ -503,11 +512,11 @@ void AudioOutputDialog::on_qsVolume_valueChanged(int v) {
 	}
 	qlVolume->setPalette(pal);
 
-	qlVolume->setText(tr("%1%").arg(v));
+	qlVolume->setText(tr("%1 %").arg(v));
 }
 
 void AudioOutputDialog::on_qsOtherVolume_valueChanged(int v) {
-	qlOtherVolume->setText(tr("%1%").arg(v));
+	qlOtherVolume->setText(tr("%1 %").arg(v));
 }
 
 void AudioOutputDialog::on_qsPacketDelay_valueChanged(int v) {
@@ -515,11 +524,11 @@ void AudioOutputDialog::on_qsPacketDelay_valueChanged(int v) {
 }
 
 void AudioOutputDialog::on_qsPacketLoss_valueChanged(int v) {
-	qlPacketLoss->setText(tr("%1%").arg(v));
+	qlPacketLoss->setText(tr("%1 %").arg(v));
 }
 
 void AudioOutputDialog::on_qsDelay_valueChanged(int v) {
-	qlDelay->setText(tr("%1ms").arg(v*10));
+	qlDelay->setText(tr("%1 ms").arg(v*10));
 }
 
 void AudioOutputDialog::on_qcbLoopback_currentIndexChanged(int v) {
@@ -534,25 +543,37 @@ void AudioOutputDialog::on_qcbLoopback_currentIndexChanged(int v) {
 }
 
 void AudioOutputDialog::on_qsMinDistance_valueChanged(int v) {
-	qlMinDistance->setText(tr("%1m").arg(v/10.0, 0, 'f', 1));
+	qlMinDistance->setText(tr("%1 m").arg(v/10.0, 0, 'f', 1));
 	if (qsMaxDistance->value() < v)
 		qsMaxDistance->setValue(v);
 }
 
 void AudioOutputDialog::on_qsMaxDistance_valueChanged(int v) {
-	qlMaxDistance->setText(tr("%1m").arg(v/10.0, 0, 'f', 1));
+	qlMaxDistance->setText(tr("%1 m").arg(v/10.0, 0, 'f', 1));
 	if (qsMinDistance->value() > v)
 		qsMinDistance->setValue(v);
 }
 
 void AudioOutputDialog::on_qsMaxDistVolume_valueChanged(int v) {
-	qlMaxDistVolume->setText(tr("%1%").arg(v));
+	qlMaxDistVolume->setText(tr("%1 %").arg(v));
 }
 
 void AudioOutputDialog::on_qsBloom_valueChanged(int v) {
-	qlBloom->setText(tr("%1%").arg(v+100));
+	qlBloom->setText(tr("%1 %").arg(v+100));
 }
 
 void AudioOutputDialog::on_qcbPositional_stateChanged(int v) {
 	qgbVolume->setEnabled(v);
 }
+
+void AudioOutputDialog::on_qcbAttenuateOthersOnTalk_clicked(bool checked) {
+	bool b = qcbAttenuateOthers->isChecked() || checked;
+	qsOtherVolume->setEnabled(b);
+	qlOtherVolume->setEnabled(b);
+}
+
+void AudioOutputDialog::on_qcbAttenuateOthers_clicked(bool checked) {
+	bool b = qcbAttenuateOthersOnTalk->isChecked() || checked;
+	qsOtherVolume->setEnabled(b);
+	qlOtherVolume->setEnabled(b);
+}
