@@ -45,7 +45,6 @@ class AudioInputDialog : public ConfigWidget, public Ui::AudioInput {
 		void hideEvent(QHideEvent *event);
 		void showEvent(QShowEvent *event);
 
-		QString browseForAudioFile();
 	public:
 		AudioInputDialog(Settings &st);
 		virtual QString title() const;
@@ -101,6 +100,8 @@ class AudioOutputDialog : public ConfigWidget, public Ui::AudioOutput {
 		void on_qsMaxDistVolume_valueChanged(int v);
 		void on_qcbSystem_currentIndexChanged(int);
 		void on_qcbPositional_stateChanged(int);
+		void on_qcbAttenuateOthersOnTalk_clicked(bool checked);
+		void on_qcbAttenuateOthers_clicked(bool checked);
 };
 
 #endif
