@@ -211,6 +211,9 @@ AudioInput::~AudioInput() {
 		delete [] pfOutput;
 }
 
+bool AudioInput::isTransmitting() const {
+	return bPreviousVoice;
+};
 
 #define IN_MIXER_FLOAT(channels) \
 static void inMixerFloat##channels ( float * RESTRICT buffer, const void * RESTRICT ipt, unsigned int nsamp, unsigned int N) { \
@@ -492,17 +495,17 @@ void AudioInput::adjustBandwidth(int bitspersec, int &bitrate, int &frames) {
 	if (bitspersec == -1) {
 		// No limit
 	} else {
-		if (getNetworkBandwidth(bitrate, frames) >= bitspersec) {
+		if (getNetworkBandwidth(bitrate, frames) > bitspersec) {
 			if ((frames <= 4) && (bitspersec <= 32000))
 				frames = 4;
 			else if ((frames == 1) && (bitspersec <= 64000))
 				frames = 2;
 			else if ((frames == 2) && (bitspersec <= 48000))
 				frames = 4;
-			if (getNetworkBandwidth(bitrate, frames) >= bitspersec) {
+			if (getNetworkBandwidth(bitrate, frames) > bitspersec) {
 				do {
 					bitrate -= 1000;
-				} while ((bitrate > 8000) && (getNetworkBandwidth(bitrate, frames) >= bitspersec));
+				} while ((bitrate > 8000) && (getNetworkBandwidth(bitrate, frames) > bitspersec));
 			}
 		}
 	}
@@ -541,7 +544,7 @@ void AudioInput::setMaxBandwidth(int bitspersec) {
 }
 
 int AudioInput::getNetworkBandwidth(int bitrate, int frames) {
-	int overhead = 20 + 8 + 4 + 1 + 2 + (g.s.bTransmitPosition ? 12 : 0) + (NetworkConfig::TcpModeEnabled() ? 12 : 0);
+	int overhead = 20 + 8 + 4 + 1 + 2 + (g.s.bTransmitPosition ? 12 : 0) + (NetworkConfig::TcpModeEnabled() ? 12 : 0) + frames;
 	overhead *= (800 / frames);
 	int bw = overhead + bitrate;
 
@@ -811,6 +814,8 @@ void AudioInput::encodeAudioFrame() {
 		if (! cCodec)
 			return;
 
+		cCodec->celt_encoder_ctl(ceEncoder, CELT_SET_PREDICTION(0));
+
 		cCodec->celt_encoder_ctl(ceEncoder,CELT_SET_VBR_RATE(iAudioQuality));
 		len = cCodec->celt_encode(ceEncoder, psSource, NULL, buffer, qMin(iAudioQuality / 800, 127));
 		iBitrate = len * 100 * 8;
@@ -845,6 +850,9 @@ void AudioInput::flushCheck(const QByteArray &frame, bool terminator) {
 		return;
 
 	int flags = g.iTarget;
+	if (terminator)
+		flags = g.iPrevTarget;
+
 	if (g.s.lmLoopMode == Settings::Server)
 		flags = 0x1f;
 
