@@ -34,6 +34,7 @@
 #include "Audio.h"
 #include "Settings.h"
 #include "Timer.h"
+#include "Message.h"
 #include "smallft.h"
 
 class AudioInput;
@@ -63,23 +64,29 @@ class AudioInput : public QThread {
 		friend class AudioEchoWidget;
 		friend class AudioStats;
 		friend class AudioInputDialog;
-		friend class FMODSystem;
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(AudioInput)
 	protected:
+		typedef enum { CodecCELT, CodecSpeex } CodecFormat;
 		typedef enum { SampleShort, SampleFloat } SampleFormat;
 		typedef void (*inMixerFunc)(float * RESTRICT, const void * RESTRICT, unsigned int, unsigned int);
 	private:
 		SpeexResamplerState *srsMic, *srsEcho;
-		JitterBuffer *jb;
+
+		QMutex qmEcho;
+		QList<short *> qlEchoFrames;
 		unsigned int iJitterSeq;
+		int iMinBuffered;
+
 		unsigned int iMicFilled, iEchoFilled;
 		inMixerFunc imfMic, imfEcho;
 		inMixerFunc chooseMixer(const unsigned int nchan, SampleFormat sf);
 	protected:
+		MessageHandler::UDPMessageType umtType;
 		SampleFormat eMicFormat, eEchoFormat;
 
+		unsigned int iSampleRate;
 		unsigned int iMicChannels, iEchoChannels;
 		unsigned int iMicFreq, iEchoFreq;
 		unsigned int iMicLength, iEchoLength;
@@ -93,9 +100,15 @@ class AudioInput : public QThread {
 		SpeexPreprocessState *sppPreprocess;
 		SpeexEchoState *sesEcho;
 
-		CELTMode *cmMode;
+		CELTCodec *cCodec;
 		CELTEncoder *ceEncoder;
 
+		SpeexBits sbBits;
+		void *esSpeex;
+
+		int iAudioQuality;
+		int iAudioFrames;
+
 		drft_lookup fftTable;
 
 		short *psMic;
@@ -111,7 +124,7 @@ class AudioInput : public QThread {
 		void addEcho(const void *data, unsigned int nsamp);
 
 		volatile bool bRunning;
-		bool bPreviousVoice;
+		volatile bool bPreviousVoice;
 
 		int iFrameCounter;
 		int iSilentFrames;
@@ -121,6 +134,9 @@ class AudioInput : public QThread {
 		void flushCheck(const QByteArray &, bool terminator);
 
 		void initializeMixer();
+
+		static bool preferCELT(int bitrate, int frames);
+		static void adjustBandwidth(int bitspersec, int &bitrate, int &frames);
 	signals:
 		void doMute();
 	public:
@@ -132,16 +148,13 @@ class AudioInput : public QThread {
 		float dPeakMic, dPeakSpeaker, dPeakSignal, dMaxMic;
 		float fSpeechProb;
 
-		bool isTransmitting() {
-			return bPreviousVoice;
-		};
-
-		static int getMaxBandwidth();
-		void setMaxBandwidth(int bytespersec);
+		static int getNetworkBandwidth(int bitrate, int frames);
+		static void setMaxBandwidth(int bitspersec);
 
 		AudioInput();
 		~AudioInput();
 		void run() = 0;
+		bool isTransmitting() const;
 };
 
 #else
