@@ -1,4 +1,5 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -236,7 +237,7 @@ AudioOutputSample::AudioOutputSample(const QString &name, SoundFile *psndfile, b
 	qWarning() << "Format: " << sfHandle->format() << endl; */
 
 	// If the frequencies don't match initialize the resampler
-	if (sfHandle->samplerate() != freq) {
+	if (sfHandle->samplerate() != static_cast<int>(freq)) {
 		srs = speex_resampler_init(1, sfHandle->samplerate(), iOutSampleRate, 3, &err);
 		if (err != RESAMPLER_ERR_SUCCESS) {
 			qWarning() << "Initialize " << sfHandle->samplerate() << " to " << iOutSampleRate << " resampler failed!";
@@ -250,9 +251,13 @@ AudioOutputSample::AudioOutputSample(const QString &name, SoundFile *psndfile, b
 
 	iLastConsume = iBufferFilled = 0;
 	bLoop = loop;
+	bEof = false;
 }
 
 AudioOutputSample::~AudioOutputSample() {
+	if (srs)
+		speex_resampler_destroy(srs);
+
 	if (sfHandle) {
 		delete sfHandle;
 		sfHandle = NULL;
@@ -285,6 +290,21 @@ SoundFile* AudioOutputSample::loadSndfile(const QString &filename) {
 	return sf;
 }
 
+QString AudioOutputSample::browseForSndfile() {
+	SoundFile *sf = NULL;
+	QString file = QFileDialog::getOpenFileName(NULL, tr("Choose sound file"), QString(), QLatin1String("*.wav *.ogg *.ogv *.oga *.flac"));
+	if (! file.isEmpty()) {
+		if ((sf = AudioOutputSample::loadSndfile(file)) == NULL) {
+			QMessageBox::critical(NULL,
+			                      tr("Invalid sound file"),
+			                      tr("The file '%1' cannot be used by Mumble. Please select a file with a compatible format and encoding.").arg(file));
+			return QString();
+		}
+		delete sf;
+	}
+	return file;
+}
+
 bool AudioOutputSample::needSamples(unsigned int snum) {
 	// Forward the buffer
 	for (unsigned int i=iLastConsume;i<iBufferFilled;++i)
@@ -335,30 +355,53 @@ bool AudioOutputSample::needSamples(unsigned int snum) {
 
 		}
 
-		spx_uint32_t inlen = read;
+		spx_uint32_t inlen = static_cast<unsigned int>(read);
 		spx_uint32_t outlen = snum;
 		if (srs) // If necessary resample
 			speex_resampler_process_float(srs, 0, pOut, &inlen, pfBuffer + iBufferFilled, &outlen);
 
 		iBufferFilled += outlen;
 	} while (iBufferFilled < snum);
+
+	if (eof && !bEof) {
+		emit playbackFinished();
+		bEof = true;
+	}
+
 	return !eof;
 }
 
-AudioOutputSpeech::AudioOutputSpeech(ClientUser *user, unsigned int freq) : AudioOutputUser(user->qsName) {
+AudioOutputSpeech::AudioOutputSpeech(ClientUser *user, unsigned int freq, MessageHandler::UDPMessageType type) : AudioOutputUser(user->qsName) {
 	int err;
 	p = user;
+	umtType = type;
 
-	iFrameSize = SAMPLE_RATE / 100;
-	cmMode = celt_mode_create(SAMPLE_RATE, 1, iFrameSize, NULL);
-	cdDecoder = celt_decoder_create(cmMode);
+	unsigned int srate = 0;
 
-	if (freq != SAMPLE_RATE)
-		srs = speex_resampler_init(1, SAMPLE_RATE, freq, 3, &err);
+	cCodec = NULL;
+	cdDecoder = NULL;
+
+	if (umtType != MessageHandler::UDPVoiceSpeex) {
+		srate = SAMPLE_RATE;
+		iFrameSize = srate / 100;
+
+		dsSpeex = NULL;
+	} else {
+		speex_bits_init(&sbBits);
+
+		dsSpeex = speex_decoder_init(speex_lib_get_mode(SPEEX_MODEID_UWB));
+		int iArg=1;
+		speex_decoder_ctl(dsSpeex, SPEEX_SET_ENH, &iArg);
+		speex_decoder_ctl(dsSpeex, SPEEX_GET_FRAME_SIZE, &iFrameSize);
+		speex_decoder_ctl(dsSpeex, SPEEX_GET_SAMPLING_RATE, &srate);
+	}
+
+	if (freq != srate)
+		srs = speex_resampler_init(1, srate, freq, 3, &err);
 	else
 		srs = NULL;
 
-	iOutputSize = iroundf(ceilf(static_cast<float>(iFrameSize * freq) / static_cast<float>(SAMPLE_RATE)));
+	iOutputSize = iroundf(ceilf(static_cast<float>(iFrameSize * freq) / static_cast<float>(srate)));
 
 	iBufferOffset = iBufferFilled = iLastConsume = 0;
 	bLastAlive = true;
@@ -366,6 +409,8 @@ AudioOutputSpeech::AudioOutputSpeech(ClientUser *user, unsigned int freq) : Audi
 	iMissCount = 0;
 	iMissedFrames = 0;
 
+	ucFlags = 0xFF;
+
 	jbJitter = jitter_buffer_init(iFrameSize);
 	int margin = g.s.iJitterBufferSize * iFrameSize;
 	jitter_buffer_ctl(jbJitter, JITTER_BUFFER_SET_MARGIN, &margin);
@@ -373,14 +418,21 @@ AudioOutputSpeech::AudioOutputSpeech(ClientUser *user, unsigned int freq) : Audi
 	fFadeIn = new float[iFrameSize];
 	fFadeOut = new float[iFrameSize];
 
-	float mul = M_PI / (2.0f * static_cast<float>(iFrameSize));
+	float mul = static_cast<float>(M_PI / (2.0 * static_cast<double>(iFrameSize)));
 	for (unsigned int i=0;i<iFrameSize;++i)
-		fFadeIn[i] = fFadeOut[iFrameSize-i-1] = sinf(i * mul);
+		fFadeIn[i] = fFadeOut[iFrameSize-i-1] = sinf(static_cast<float>(i) * mul);
 }
 
 AudioOutputSpeech::~AudioOutputSpeech() {
-	celt_decoder_destroy(cdDecoder);
-	celt_mode_destroy(cmMode);
+	if (cdDecoder) {
+		cCodec->celt_decoder_destroy(cdDecoder);
+	} else if (dsSpeex) {
+		speex_bits_destroy(&sbBits);
+		speex_decoder_destroy(dsSpeex);
+	}
+
+	if (srs)
+		speex_resampler_destroy(srs);
 
 	jitter_buffer_destroy(jbJitter);
 
@@ -401,7 +453,7 @@ void AudioOutputSpeech::addFrameToBuffer(const QByteArray &qbaPacket, unsigned i
 	int frames = 0;
 	unsigned int header = 0;
 	do {
-		header = pds.next();
+		header = static_cast<unsigned char>(pds.next());
 		frames++;
 		pds.skip(header & 0x7f);
 	} while ((header & 0x80) && pds.isValid());
@@ -413,6 +465,24 @@ void AudioOutputSpeech::addFrameToBuffer(const QByteArray &qbaPacket, unsigned i
 		jbp.span = iFrameSize * frames;
 		jbp.timestamp = iFrameSize * iSeq;
 
+		if (g.s.bUsage && (umtType != MessageHandler::UDPVoiceSpeex) && p && ! p->qsHash.isEmpty() && (p->qlTiming.count() < 3000)) {
+			QMutexLocker qml(& p->qmTiming);
+
+			ClientUser::JitterRecord jr;
+			jr.iSequence = iSeq;
+			jr.iFrames = frames;
+			jr.uiElapsed = p->tTiming.restart();
+
+			if (! p->qlTiming.isEmpty()) {
+				jr.iFrames -= p->iFrames;
+				jr.iSequence -= p->iSequence + p->iFrames;
+			}
+			p->iFrames = frames;
+			p->iSequence = iSeq;
+
+			p->qlTiming.append(jr);
+		}
+
 		jitter_buffer_put(jbJitter, &jbp);
 	}
 }
@@ -440,11 +510,24 @@ bool AudioOutputSpeech::needSamples(unsigned int snum) {
 		if (! bLastAlive) {
 			memset(pOut, 0, iFrameSize * sizeof(float));
 		} else {
-			int ts = jitter_buffer_get_pointer_timestamp(jbJitter);
-
 			if (p == &LoopUser::lpLoopy)
 				LoopUser::lpLoopy.fetchFrames();
 
+			int avail = 0;
+			int ts = jitter_buffer_get_pointer_timestamp(jbJitter);
+			jitter_buffer_ctl(jbJitter, JITTER_BUFFER_GET_AVAILABLE_COUNT, &avail);
+
+			if (p && (ts == 0)) {
+				int want = iroundf(p->fAverageAvailable);
+				if (avail < want) {
+					++iMissCount;
+					if (iMissCount < 20) {
+						memset(pOut, 0, iFrameSize * sizeof(float));
+						goto nextframe;
+					}
+				}
+			}
+
 			if (qlFrames.isEmpty()) {
 				QMutexLocker lock(&qmJitter);
 
@@ -459,12 +542,12 @@ bool AudioOutputSpeech::needSamples(unsigned int snum) {
 					PacketDataStream pds(jbp.data, jbp.len);
 
 					iMissCount = 0;
-					ucFlags = pds.next();
+					ucFlags = static_cast<unsigned char>(pds.next());
 					bHasTerminator = false;
 
 					unsigned int header = 0;
 					do {
-						header = pds.next();
+						header = static_cast<unsigned int>(pds.next());
 						if (header)
 							qlFrames << pds.dataBlock(header & 0x7f);
 						else
@@ -478,24 +561,97 @@ bool AudioOutputSpeech::needSamples(unsigned int snum) {
 					} else {
 						fPos[0] = fPos[1] = fPos[2] = 0.0f;
 					}
+
+					if (p) {
+						float a = static_cast<float>(avail);
+						if (avail >= p->fAverageAvailable)
+							p->fAverageAvailable = a;
+						else
+							p->fAverageAvailable *= 0.99f;
+					}
 				} else {
+					jitter_buffer_update_delay(jbJitter, &jbp, NULL);
+
 					iMissCount++;
 					if (iMissCount > 10)
 						nextalive = false;
 				}
-
-				jitter_buffer_update_delay(jbJitter, &jbp, NULL);
 			}
 
-			jitter_buffer_tick(jbJitter);
-
 			if (! qlFrames.isEmpty()) {
-				const QByteArray &qba = qlFrames.takeFirst();
-				celt_decode_float(cdDecoder, reinterpret_cast<const unsigned char *>(qba.data()), qba.size(), pOut);
+				QByteArray qba = qlFrames.takeFirst();
+
+				if (umtType != MessageHandler::UDPVoiceSpeex) {
+					int wantversion = (umtType == MessageHandler::UDPVoiceCELTAlpha) ? g.iCodecAlpha : g.iCodecBeta;
+					if ((p == &LoopUser::lpLoopy) && (! g.qmCodecs.isEmpty())) {
+						QMap<int, CELTCodec *>::const_iterator i = g.qmCodecs.constEnd();
+						--i;
+						wantversion = i.key();
+					}
+					if (cCodec && (cCodec->bitstreamVersion() != wantversion)) {
+						cCodec->celt_decoder_destroy(cdDecoder);
+						cdDecoder = NULL;
+					}
+					if (! cCodec) {
+						cCodec = g.qmCodecs.value(wantversion);
+						if (cCodec) {
+							cdDecoder = cCodec->decoderCreate();
+						}
+					}
+					if (cdDecoder)
+						cCodec->celt_decode_float(cdDecoder, qba.isEmpty() ? NULL : reinterpret_cast<const unsigned char *>(qba.constData()), qba.size(), pOut);
+					else
+						memset(pOut, 0, sizeof(float) * iFrameSize);
+				} else {
+					if (qba.isEmpty()) {
+						speex_decode(dsSpeex, NULL, pOut);
+					} else {
+						speex_bits_read_from(&sbBits, qba.data(), qba.size());
+						speex_decode(dsSpeex, &sbBits, pOut);
+					}
+					for (unsigned int i=0;i<iFrameSize;++i)
+						pOut[i] *= (1.0f / 32767.f);
+				}
+
+				bool update = true;
+				if (p) {
+					float &fPowerMax = p->fPowerMax;
+					float &fPowerMin = p->fPowerMin;
+
+					float pow = 0.0f;
+					for (unsigned int i=0;i<iFrameSize;++i)
+						pow += pOut[i] * pOut[i];
+					pow = sqrtf(pow / static_cast<float>(iFrameSize));
+
+					if (pow >= fPowerMax) {
+						fPowerMax = pow;
+					} else {
+						if (pow <= fPowerMin) {
+							fPowerMin = pow;
+						} else {
+							fPowerMax = 0.99f * fPowerMax;
+							fPowerMin += 0.0001f * pow;
+						}
+					}
+
+					update = (pow < (fPowerMin + 0.01f * (fPowerMax - fPowerMin)));
+				}
+				if (qlFrames.isEmpty() && update)
+					jitter_buffer_update_delay(jbJitter, NULL, NULL);
+
 				if (qlFrames.isEmpty() && bHasTerminator)
 					nextalive = false;
 			} else {
-				celt_decode_float(cdDecoder, NULL, 0, pOut);
+				if (umtType != MessageHandler::UDPVoiceSpeex) {
+					if (cdDecoder)
+						cCodec->celt_decode_float(cdDecoder, NULL, 0, pOut);
+					else
+						memset(pOut, 0, sizeof(float) * iFrameSize);
+				} else {
+					speex_decode(dsSpeex, NULL, pOut);
+					for (unsigned int i=0;i<iFrameSize;++i)
+						pOut[i] *= (1.0f / 32767.f);
+				}
 			}
 
 			if (! nextalive) {
@@ -505,7 +661,10 @@ bool AudioOutputSpeech::needSamples(unsigned int snum) {
 				for (unsigned int i=0;i<iFrameSize;++i)
 					pOut[i] *= fFadeIn[i];
 			}
+
+			jitter_buffer_tick(jbJitter);
 		}
+nextframe:
 		spx_uint32_t inlen = iFrameSize;
 		spx_uint32_t outlen = iOutputSize;
 		if (srs && bLastAlive)
@@ -513,8 +672,26 @@ bool AudioOutputSpeech::needSamples(unsigned int snum) {
 		iBufferFilled += outlen;
 	}
 
-	if (p)
-		p->setTalking(nextalive, ((ucFlags != 0) ? true : false));
+	if (p) {
+		ClientUser::TalkState ts;
+		if (! nextalive)
+			ucFlags = 0xFF;
+		switch (ucFlags) {
+			case 0:
+				ts = ClientUser::Talking;
+				break;
+			case 1:
+				ts = ClientUser::TalkingWhisperChannel;
+				break;
+			case 0xFF:
+				ts = ClientUser::TalkingOff;
+				break;
+			default:
+				ts = ClientUser::TalkingWhisper;
+				break;
+		}
+		p->setTalking(ts);
+	}
 
 	bool tmp = bLastAlive;
 	bLastAlive = nextalive;
@@ -604,7 +781,10 @@ AudioOutputSample *AudioOutput::playSample(const QString &filename, bool loop) {
 	if (handle == NULL)
 		return NULL;
 
-	while ((iMixerFreq == 0) && isRunning()) {}
+	while ((iMixerFreq == 0) && bRunning) {}
+
+	if (! iMixerFreq)
+		return NULL;
 
 	qrwlOutputs.lockForWrite();
 	AudioOutputSample *aos = new AudioOutputSample(filename, handle, loop, iMixerFreq);
@@ -615,19 +795,25 @@ AudioOutputSample *AudioOutput::playSample(const QString &filename, bool loop) {
 
 }
 
-void AudioOutput::addFrameToBuffer(ClientUser *user, const QByteArray &qbaPacket, unsigned int iSeq) {
+void AudioOutput::addFrameToBuffer(ClientUser *user, const QByteArray &qbaPacket, unsigned int iSeq, MessageHandler::UDPMessageType type) {
 	if (iChannels == 0)
 		return;
 	qrwlOutputs.lockForRead();
 	AudioOutputSpeech *aop = dynamic_cast<AudioOutputSpeech *>(qmOutputs.value(user));
 
-	if (! aop) {
+	if (! aop || (aop->umtType != type)) {
 		qrwlOutputs.unlock();
 
-		while ((iMixerFreq == 0) && isRunning()) {}
+		if (aop)
+			removeBuffer(aop);
+
+		while ((iMixerFreq == 0) && bRunning) {}
+
+		if (! iMixerFreq)
+			return;
 
 		qrwlOutputs.lockForWrite();
-		aop = new AudioOutputSpeech(user, iMixerFreq);
+		aop = new AudioOutputSpeech(user, iMixerFreq, type);
 		qmOutputs.replace(user,aop);
 	}
 
