@@ -32,6 +32,7 @@
 #include "AudioInput.h"
 #include "Global.h"
 #include "Settings.h"
+#include "Log.h"
 #include "MainWindow.h"
 
 CompletablePage::CompletablePage(QWizard *p) : QWizardPage(p) {
@@ -49,6 +50,7 @@ bool CompletablePage::isComplete() const {
 
 AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 	bInit = true;
+	bLastActive = false;
 
 	ticker = new QTimer(this);
 	ticker->setObjectName(QLatin1String("Ticker"));
@@ -62,8 +64,10 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 	if (AudioInputRegistrar::qmNew) {
 		foreach(AudioInputRegistrar *air, *AudioInputRegistrar::qmNew) {
 			qcbInput->addItem(air->name);
-			if (air->name == AudioInputRegistrar::current)
+			if (air->name == AudioInputRegistrar::current) {
 				qcbInput->setCurrentIndex(qcbInput->count() - 1);
+				qcbEcho->setEnabled(air->canEcho(qcbOutput->currentText()));
+			}
 			QList<audioDevice> ql= air->getDeviceChoices();
 		}
 	}
@@ -79,6 +83,7 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 			if (aor->name == AudioOutputRegistrar::current) {
 				qcbOutput->setCurrentIndex(qcbOutput->count() - 1);
 				bDelay = aor->usesOutputDelay();
+				qcbAttenuateOthers->setEnabled(aor->canMuteOthers());
 			}
 			QList<audioDevice> ql= aor->getDeviceChoices();
 		}
@@ -88,7 +93,20 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 		qcbOutput->setEnabled(false);
 	}
 
+	if (g.s.iQuality == 16000 && g.s.iFramesPerPacket == 6)
+		qrbQualityLow->setChecked(true);
+	else if (g.s.iQuality == 40000 && g.s.iFramesPerPacket == 2)
+		qrbQualityBalanced->setChecked(true);
+	else if (g.s.iQuality == 72000 && g.s.iFramesPerPacket == 1)
+		qrbQualityUltra->setChecked(true);
+	else
+		qrbQualityCustom->setChecked(true);
+
+	qrbQualityCustom->setVisible(qrbQualityCustom->isChecked());
+	qlQualityCustom->setVisible(qrbQualityCustom->isChecked());
+
 	qcbPositional->setChecked(g.s.bPositionalAudio);
+	qcbAttenuateOthers->setChecked(g.s.bAttenuateOthers);
 
 	on_qcbInput_activated(qcbInput->currentIndex());
 	on_qcbOutput_activated(qcbOutput->currentIndex());
@@ -148,12 +166,14 @@ AudioWizard::AudioWizard(QWidget *p) : QWizard(p) {
 	g.s.bMute = true;
 	g.s.bDeaf = false;
 
-	g.s.atTransmit = Settings::Continous;
 	bTransmitChanged = false;
 
 	iMaxPeak = 0;
 	iTicks = 0;
 
+	qpTalkingOn = QPixmap::fromImage(QImage(QLatin1String("skin:talking_on.svg")).scaled(64,64));
+	qpTalkingOff = QPixmap::fromImage(QImage(QLatin1String("skin:talking_off.svg")).scaled(64,64));
+
 	bInit = false;
 
 	connect(this, SIGNAL(currentIdChanged(int)), this, SLOT(showPage(int)));
@@ -234,6 +254,8 @@ void AudioWizard::on_qcbOutput_activated(int) {
 		qcbOutputDevice->addItem(d.first, d.second);
 	}
 
+	qcbAttenuateOthers->setEnabled(aor->canMuteOthers());
+
 	qcbOutputDevice->setEnabled(ql.count() > 1);
 
 	on_qcbOutputDevice_activated(0);
@@ -268,7 +290,7 @@ void AudioWizard::on_qcbOutputDevice_activated(int) {
 }
 
 void AudioWizard::on_qsOutputDelay_valueChanged(int v) {
-	qlOutputDelay->setText(tr("%1ms").arg(v*10));
+	qlOutputDelay->setText(tr("%1 ms").arg(v*10));
 	g.s.iOutputDelay = v;
 	restartAudio();
 }
@@ -277,7 +299,10 @@ void AudioWizard::on_qsMaxAmp_valueChanged(int v) {
 	g.s.iMinLoudness = qMin(v, 30000);
 }
 
-void AudioWizard::showPage(int) {
+void AudioWizard::showPage(int pageid) {
+	if (pageid == -1)
+		return;
+
 	CompletablePage *cp = qobject_cast<CompletablePage *>(currentPage());
 
 	AudioOutputPtr ao = g.ao;
@@ -307,13 +332,24 @@ void AudioWizard::showPage(int) {
 	} else {
 		g.s.bMute = false;
 	}
+
+	if ((cp == qwpTrigger) || (cp == qwpSettings)) {
+		if (! bTransmitChanged)
+			g.s.atTransmit = sOldSettings.atTransmit;
+		else if (qrPTT->isChecked())
+			g.s.atTransmit = Settings::PushToTalk;
+		else
+			g.s.atTransmit = Settings::VAD;
+	} else {
+		g.s.atTransmit = Settings::Continous;
+	}
 }
 
 int AudioWizard::nextId() const {
 	AudioOutputPtr ao = g.ao;
 
 	int nextid = QWizard::nextId();
-	if (currentPage() == qwpTrigger && ! g.s.bPositionalAudio)
+	if (currentPage() == qwpSettings && ! g.s.bPositionalAudio)
 		nextid++;
 	else if ((currentPage() == qwpDevice) && ! bDelay)
 		nextid++;
@@ -369,6 +405,11 @@ void AudioWizard::reject() {
 	g.s.lmLoopMode = Settings::None;
 	restartAudio();
 
+	AudioOutputPtr ao = g.ao;
+	if (ao)
+		ao->wipe();
+	aosSource = NULL;
+
 	QWizard::reject();
 }
 
@@ -384,33 +425,15 @@ void AudioWizard::accept() {
 	g.s.bDeaf = sOldSettings.bDeaf;
 	g.s.lmLoopMode = Settings::None;
 
-	const QList<QVariant> &buttons = skwPTT->getShortcut();
-	QList<Shortcut> ql;
-	bool found = false;
-	foreach(Shortcut s, g.s.qlShortcuts) {
-		if (s.iIndex == g.mw->gsPushTalk->idx) {
-			if (buttons.isEmpty())
-				continue;
-			else if (! found) {
-				s.qlButtons = buttons;
-				found = true;
-			}
-		}
-		ql << s;
+	// Switch TTS<->Sounds according to user selection
+	Settings::MessageLog mlReplace = qrbNotificationTTS->isChecked() ? Settings::LogSoundfile : Settings::LogTTS;
+	for (int i = Log::firstMsgType;i <= Log::lastMsgType; ++i) {
+		if (g.s.qmMessages[i] & mlReplace)
+			g.s.qmMessages[i] ^= Settings::LogSoundfile | Settings::LogTTS;
 	}
-	if (! found && ! buttons.isEmpty()) {
-		Shortcut s;
-		s.iIndex = g.mw->gsPushTalk->idx;
-		s.bSuppress = false;
-		s.qlButtons = buttons;
-		ql << s;
-	}
-	g.s.qlShortcuts = ql;
 
 	g.s.bUsage = qcbUsage->isChecked();
 	g.bPosTest = false;
-	GlobalShortcutEngine::engine->bNeedRemap = true;
-	GlobalShortcutEngine::engine->needRemap();
 	restartAudio();
 	QWizard::accept();
 }
@@ -453,6 +476,12 @@ void AudioWizard::on_Ticker_timeout() {
 	}
 	abVAD->update();
 
+	bool active = ai->isTransmitting();
+	if (active != bLastActive) {
+		bLastActive = active;
+		qlTalkIcon->setPixmap(active ? qpTalkingOn : qpTalkingOff);
+	}
+
 	if (! qgsScene) {
 		unsigned int nspeaker = 0;
 		const float *spos = ao->getSpeakerPos(nspeaker);
@@ -498,6 +527,7 @@ void AudioWizard::on_qsVAD_valueChanged(int v) {
 void AudioWizard::on_qrSNR_clicked(bool on) {
 	if (on) {
 		g.s.vsVAD = Settings::SignalToNoise;
+		g.s.atTransmit = Settings::VAD;
 		updateTriggerWidgets(false);
 		bTransmitChanged = true;
 	}
@@ -506,6 +536,7 @@ void AudioWizard::on_qrSNR_clicked(bool on) {
 void AudioWizard::on_qrAmplitude_clicked(bool on) {
 	if (on) {
 		g.s.vsVAD = Settings::Amplitude;
+		g.s.atTransmit = Settings::VAD;
 		updateTriggerWidgets(false);
 		bTransmitChanged = true;
 	}
@@ -513,6 +544,7 @@ void AudioWizard::on_qrAmplitude_clicked(bool on) {
 
 void AudioWizard::on_qrPTT_clicked(bool on) {
 	if (on) {
+		g.s.atTransmit = Settings::PushToTalk;
 		updateTriggerWidgets(true);
 		bTransmitChanged = true;
 	}
@@ -525,6 +557,32 @@ void AudioWizard::on_skwPTT_keySet(bool valid) {
 		qrSNR->setChecked(true);
 	updateTriggerWidgets(valid);
 	bTransmitChanged = true;
+
+	const QList<QVariant> &buttons = skwPTT->getShortcut();
+	QList<Shortcut> ql;
+	bool found = false;
+	foreach(Shortcut s, g.s.qlShortcuts) {
+		if (s.iIndex == g.mw->gsPushTalk->idx) {
+			if (buttons.isEmpty())
+				continue;
+			else if (! found) {
+				s.qlButtons = buttons;
+				found = true;
+			}
+		}
+		ql << s;
+	}
+	if (! found && ! buttons.isEmpty()) {
+		Shortcut s;
+		s.iIndex = g.mw->gsPushTalk->idx;
+		s.bSuppress = false;
+		s.qlButtons = buttons;
+		ql << s;
+	}
+	g.s.qlShortcuts = ql;
+
+	GlobalShortcutEngine::engine->bNeedRemap = true;
+	GlobalShortcutEngine::engine->needRemap();
 }
 
 void AudioWizard::on_qcbEcho_clicked(bool on) {
@@ -539,6 +597,7 @@ void AudioWizard::on_qcbHeadphone_clicked(bool on) {
 
 void AudioWizard::on_qcbPositional_clicked(bool on) {
 	g.s.bPositionalAudio = on;
+	g.s.bTransmitPosition = on;
 	restartAudio();
 }
 
@@ -546,3 +605,31 @@ void AudioWizard::updateTriggerWidgets(bool ptt) {
 	qwVAD->setEnabled(!ptt);
 	qwpTrigger->setComplete(!ptt || (skwPTT->qlButtons.count() > 0));
 }
+
+void AudioWizard::on_qcbAttenuateOthers_clicked(bool checked) {
+	g.s.bAttenuateOthers = checked;
+}
+
+void AudioWizard::on_qrbQualityLow_clicked() {
+	g.s.iQuality = 16000;
+	g.s.iFramesPerPacket = 6;
+	restartAudio();
+}
+
+void AudioWizard::on_qrbQualityBalanced_clicked() {
+	g.s.iQuality = 40000;
+	g.s.iFramesPerPacket = 2;
+	restartAudio();
+}
+
+void AudioWizard::on_qrbQualityUltra_clicked() {
+	g.s.iQuality = 72000;
+	g.s.iFramesPerPacket = 1;
+	restartAudio();
+}
+
+void AudioWizard::on_qrbQualityCustom_clicked() {
+	g.s.iQuality = sOldSettings.iQuality;
+	g.s.iFramesPerPacket = sOldSettings.iFramesPerPacket;
+	restartAudio();
+}
