@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -46,61 +46,30 @@ class CompletablePage : public QWizardPage {
 		bool isComplete() const;
 };
 
-class AudioWizard: public QWizard {
+#include "ui_AudioWizard.h"
+
+class AudioWizard: public QWizard, public Ui::AudioWizard {
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(AudioWizard)
 	protected:
-		QComboBox *qcbInput, *qcbInputDevice;
-		QComboBox *qcbOutput, *qcbOutputDevice;
-		QCheckBox *qcbEcho;
-		QCheckBox *qcbPositional;
-
-		QSlider *qsOutputDelay;
-		QLabel *qlOutputDelay;
-
-		AudioBar *abAmplify;
-		QSlider *qsMaxAmp;
-
-		QWidget *qwVAD;
-		AudioBar *abVAD;
-		QRadioButton *qrAmplitude, *qrSNR, *qrPTT;
-		QSlider *qsMinVAD, *qsMaxVAD;
-		ShortcutKeyWidget *skwPTT;
 		bool bTransmitChanged;
 
-		QSlider *qsHoldtime;
-		QLabel *qlHoldtime;
-
-		QLabel *qlAudioPath;
-
-		QGraphicsView *qgvView;
 		QGraphicsScene *qgsScene;
 		QGraphicsItem *qgiSource;
-		QCheckBox *qcbHeadphone;
 		AudioOutputSample *aosSource;
 		float fAngle;
 		float fX, fY;
 
-		QCheckBox *qcbUsage;
-
-		CompletablePage *qwpIntro, *qwpDevice, *qwpVolume, *qwpTrigger, *qwpDeviceTuning, *qwpPositional, *qwpDone;
-
-		CompletablePage *introPage();
-		CompletablePage *devicePage();
-		CompletablePage *volumePage();
-		CompletablePage *triggerPage();
-		CompletablePage *deviceTuningPage();
-		CompletablePage *positionalPage();
-		CompletablePage *donePage();
-
 		Settings sOldSettings;
 
 		QTimer *ticker;
 
 		bool bInit;
-
 		bool bDelay;
+		bool bLastActive;
+
+		QPixmap qpTalkingOn, qpTalkingOff;
 
 		int iMaxPeak;
 		int iTicks;
@@ -110,23 +79,26 @@ class AudioWizard: public QWizard {
 
 		bool eventFilter(QObject *, QEvent *);
 	public slots:
-		void on_Input_activated(int);
-		void on_InputDevice_activated(int);
-		void on_Output_activated(int);
-		void on_OutputDevice_activated(int);
-		void on_OutputDelay_valueChanged(int);
-		void on_MaxAmp_valueChanged(int);
+		void on_qcbInput_activated(int);
+		void on_qcbInputDevice_activated(int);
+		void on_qcbOutput_activated(int);
+		void on_qcbOutputDevice_activated(int);
+		void on_qsOutputDelay_valueChanged(int);
+		void on_qsMaxAmp_valueChanged(int);
 		void on_Ticker_timeout();
-		void on_VADmin_valueChanged(int);
-		void on_VADmax_valueChanged(int);
-		void on_Holdtime_valueChanged(int);
-		void on_Amplitude_clicked(bool);
-		void on_SNR_clicked(bool);
-		void on_PTT_clicked(bool);
-		void on_Echo_clicked(bool);
-		void on_Headphone_clicked(bool);
-		void on_Positional_clicked(bool);
-		void on_PTTKey_keySet(bool);
+		void on_qsVAD_valueChanged(int);
+		void on_qrAmplitude_clicked(bool);
+		void on_qrSNR_clicked(bool);
+		void on_qrPTT_clicked(bool);
+		void on_qcbEcho_clicked(bool);
+		void on_qcbHeadphone_clicked(bool);
+		void on_qcbPositional_clicked(bool);
+		void on_qcbAttenuateOthers_clicked(bool);
+		void on_skwPTT_keySet(bool, bool);
+		void on_qrbQualityUltra_clicked();
+		void on_qrbQualityBalanced_clicked();
+		void on_qrbQualityLow_clicked();
+		void on_qrbQualityCustom_clicked();
 		void showPage(int);
 		void updateTriggerWidgets(bool);
 	public:
