@@ -33,62 +33,125 @@
 #include "Channel.h"
 #include "Global.h"
 
-BanEditor::BanEditor(const MessageServerBanList *msbl, QWidget *p) : QDialog(p) {
+BanEditor::BanEditor(const MumbleProto::BanList &msg, QWidget *p) : QDialog(p) {
 	setupUi(this);
 
-	QRegExp rx(QLatin1String("\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}\\.\\d{1,3}"));
-	QValidator *validator = new QRegExpValidator(rx, this);
-	qleIP->setValidator(validator);
+	qlBans.clear();
+	for (int i=0;i < msg.bans_size(); ++i) {
+		const MumbleProto::BanList_BanEntry &be = msg.bans(i);
+		Ban b;
+		b.haAddress = be.address();
+		b.iMask = be.mask();
+		b.qsUsername = u8(be.name());
+		b.qsHash = u8(be.hash());
+		b.qsReason = u8(be.reason());
+		b.qdtStart = QDateTime::fromString(u8(be.start()), Qt::ISODate);
+		b.qdtStart.setTimeSpec(Qt::UTC);
+		if (! b.qdtStart.isValid())
+			b.qdtStart = QDateTime::currentDateTime();
+		b.iDuration = be.duration();
+		if (b.isValid())
+			qlBans << b;
+	}
 
-	qlBans = msbl->qlBans;
 	refreshBanList();
-
 }
 
 void BanEditor::accept() {
-	MessageServerBanList msbl;
+	MumbleProto::BanList msg;
 
-	msbl.bQuery = false;
-	msbl.qlBans = qlBans;
+	foreach(const Ban &b, qlBans) {
+		MumbleProto::BanList_BanEntry *be = msg.add_bans();
+		be->set_address(b.haAddress.toStdString());
+		be->set_mask(b.iMask);
+		be->set_name(u8(b.qsUsername));
+		be->set_hash(u8(b.qsHash));
+		be->set_reason(u8(b.qsReason));
+		be->set_start(u8(b.qdtStart.toString(Qt::ISODate)));
+		be->set_duration(b.iDuration);
+	}
 
-	g.sh->sendMessage(&msbl);
+	g.sh->sendMessage(msg);
 	QDialog::accept();
 }
 
 void BanEditor::on_qlwBans_currentRowChanged() {
-	QPair<quint32, int> ban;
 	int idx = qlwBans->currentRow();
 	if (idx < 0)
 		return;
-	ban = qlBans[idx];
+	const Ban &ban = qlBans.at(idx);
 
-	QHostAddress addr(ban.first);
+	int maskbits = ban.iMask;
+
+	const QHostAddress &addr = ban.haAddress.toAddress();
 	qleIP->setText(addr.toString());
-	qsbMask->setValue(ban.second);
+	if (! ban.haAddress.isV6())
+		maskbits -= 96;
+	qsbMask->setValue(maskbits);
+	qlUser->setText(ban.qsUsername);
+	qlHash->setText(ban.qsHash);
+	qleReason->setText(ban.qsReason);
+	qdteStart->setDateTime(ban.qdtStart.toLocalTime());
+	qdteEnd->setDateTime(ban.qdtStart.toLocalTime().addSecs(ban.iDuration));
+
+}
+
+Ban BanEditor::toBan(bool &ok) {
+	Ban b;
+
+	QHostAddress addr;
+
+	ok = addr.setAddress(qleIP->text());
+
+	if (ok) {
+		b.haAddress = addr;
+		b.iMask = qsbMask->value();
+		if (! b.haAddress.isV6())
+			b.iMask += 96;
+		b.qsUsername = qlUser->text();
+		b.qsHash = qlHash->text();
+		b.qsReason = qleReason->text();
+		b.qdtStart = qdteStart->dateTime().toUTC();
+		const QDateTime &qdte = qdteEnd->dateTime();
+		if (qdte <= b.qdtStart)
+			b.iDuration = 0;
+		else
+			b.iDuration = b.qdtStart.secsTo(qdte);
+
+		ok = b.isValid();
+	}
+	return b;
 }
 
 void BanEditor::on_qpbAdd_clicked() {
-	QPair<quint32, int> ban;
 	QHostAddress addr;
 
 	bool ok;
 
-	ok = addr.setAddress(qleIP->text());
+	qdteStart->setDateTime(QDateTime::currentDateTime());
+
+	Ban b = toBan(ok);
+
 	if (ok) {
-		quint32 base = addr.toIPv4Address();
-		quint32 nmask = qsbMask->value();
-		quint32 bitmask = (1<<(32-nmask))-1;
-		ban.first = base & ~bitmask;
-		ban.second = nmask;
-		qlBans << ban;
+		b.qsHash = QString();
+		b.qsUsername = QString();
+		qlBans << b;
 		refreshBanList();
-		qlwBans->setCurrentRow(qlBans.indexOf(ban));
+		qlwBans->setCurrentRow(qlBans.indexOf(b));
 	}
 }
 
 void BanEditor::on_qpbUpdate_clicked() {
-	on_qpbRemove_clicked();
-	on_qpbAdd_clicked();
+	int idx = qlwBans->currentRow();
+	if (idx >= 0) {
+		bool ok;
+		Ban b = toBan(ok);
+		if (ok) {
+			qlBans.replace(idx, b);
+			refreshBanList();
+			qlwBans->setCurrentRow(qlBans.indexOf(b));
+		}
+	}
 }
 
 void BanEditor::on_qpbRemove_clicked() {
@@ -99,15 +162,16 @@ void BanEditor::on_qpbRemove_clicked() {
 }
 
 void BanEditor::refreshBanList() {
-	QPair<quint32, int> ban;
-
 	qlwBans->clear();
 
 	qSort(qlBans);
 
-	foreach(ban,qlBans) {
-		QHostAddress addr(ban.first);
-		QString qs = QString::fromLatin1("%1/%2").arg(addr.toString()).arg(ban.second);
+	foreach(const Ban &ban, qlBans) {
+		int maskbits = ban.iMask;
+		const QHostAddress &addr=ban.haAddress.toAddress();
+		if (! ban.haAddress.isV6())
+			maskbits -= 96;
+		QString qs = QString::fromLatin1("%1/%2").arg(addr.toString()).arg(maskbits);
 		qlwBans->addItem(qs);
 	}
 }
