@@ -42,6 +42,8 @@ ClientUser::ClientUser(QObject *p) : QObject(p) {
 	iTextureWidth = 0;
 	fPowerMin = fPowerMax = 0.0f;
 	fAverageAvailable = 0.0f;
+	iFrames = 0;
+	iSequence = 0;
 }
 
 ClientUser *ClientUser::get(unsigned int uiSession) {
@@ -50,6 +52,30 @@ ClientUser *ClientUser::get(unsigned int uiSession) {
 	return p;
 }
 
+ClientUser *ClientUser::getByHash(QString _qsHash) {
+	QReadLocker lock(&c_qrwlUsers);
+
+	ClientUser *cu;
+	foreach(cu, c_qmUsers) {
+		if (cu->qsHash == _qsHash)
+			return cu;
+	}
+
+	return NULL;
+}
+
+unsigned int ClientUser::getUiSession(ClientUser *cu) {
+	QReadLocker lock(&c_qrwlUsers);
+
+	return c_qmUsers.key(cu);
+}
+
+bool ClientUser::isValid(unsigned int uiSession) {
+	QReadLocker lock(&c_qrwlUsers);
+
+	return c_qmUsers.contains(uiSession);
+}
+
 ClientUser *ClientUser::add(unsigned int uiSession, QObject *po) {
 	QWriteLocker lock(&c_qrwlUsers);
 
@@ -168,3 +194,10 @@ void Channel::addClientUser(ClientUser *p) {
 	addUser(p);
 	p->setParent(this);
 }
+
+QDataStream &operator<<(QDataStream &qds, const ClientUser::JitterRecord &jr) {
+	qds << static_cast<qint8>(qBound(-128, jr.iSequence, 127));
+	qds << static_cast<qint8>(qBound(-128, jr.iFrames, 127));
+	qds << static_cast<quint32>(qMin(4294967295ULL, jr.uiElapsed));
+	return qds;
+}
