@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -89,6 +89,7 @@ ServerView::ServerView(QWidget *p) : QTreeWidget(p) {
 
 	siPublic->setExpanded(false);
 
+	qmContinentNames.insert(QLatin1String("af"), tr("Africa"));
 	qmContinentNames.insert(QLatin1String("as"), tr("Asia"));
 	qmContinentNames.insert(QLatin1String("na"), tr("North America"));
 	qmContinentNames.insert(QLatin1String("sa"), tr("South America"));
@@ -715,9 +716,9 @@ ConnectDialog::ConnectDialog(QWidget *p) : QDialog(p) {
 		qlPublicServers.clear();
 	}
 
-	qdbbButtonBox->button(QDialogButtonBox::Ok)->setText(tr("Connect"));
+	qdbbButtonBox->button(QDialogButtonBox::Ok)->setText(tr("&Connect"));
 
-	QPushButton *qpb = new QPushButton(tr("Add New..."), this);
+	QPushButton *qpb = new QPushButton(tr("&Add New..."), this);
 	qpb->setDefault(false);
 	qpb->setAutoDefault(false);
 	connect(qpb, SIGNAL(clicked()), qaFavoriteAddNew, SIGNAL(triggered()));
@@ -754,7 +755,7 @@ ConnectDialog::ConnectDialog(QWidget *p) : QDialog(p) {
 	connect(qagFilters, SIGNAL(triggered(QAction *)), this, SLOT(onFiltersTriggered(QAction *)));
 
 	qmPopup = new QMenu(this);
-	qmFilters = new QMenu(tr("Filters"), this);
+	qmFilters = new QMenu(tr("&Filters"), this);
 	qmFilters->addAction(qaShowAll);
 	qmFilters->addAction(qaShowReachable);
 	qmFilters->addAction(qaShowPopulated);
@@ -1239,6 +1240,9 @@ void ConnectDialog::startDns(ServerItem *si) {
 			si->qlAddresses = qhDNSCache.value(host);
 	}
 
+	if (qtwServers->currentItem() == si)
+		qdbbButtonBox->button(QDialogButtonBox::Ok)->setEnabled(! si->qlAddresses.isEmpty());
+
 	if (! si->qlAddresses.isEmpty()) {
 		foreach(const QHostAddress &qha, si->qlAddresses) {
 			qhPings[qpAddress(qha, si->usPort)].insert(si);
