@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -32,8 +32,6 @@
 #define _CONNECTDIALOG_H
 
 #include "mumble_pch.hpp"
-#include "ui_ConnectDialog.h"
-#include "ui_ConnectDialogEdit.h"
 #include "Timer.h"
 #include "Database.h"
 
@@ -51,17 +49,76 @@ struct PublicInfo {
 	QString qsIp;
 	QString qsCountry;
 	QString qsCountryCode;
+	QString qsContinentCode;
 	unsigned short usPort;
 };
 
-class ServerItem : public QTreeWidgetItem {
+struct PingStats {
+private:
+	Q_DISABLE_COPY(PingStats);
+protected:
+	void init();
+public:
+	quint32 uiVersion;
+	quint32 uiPing;
+	quint32 uiPingSort;
+	quint32 uiUsers;
+	quint32 uiMaxUsers;
+	quint32 uiBandwidth;
+	quint32 uiSent;
+	quint32 uiRecv;
+
+	double dPing;
+
+	typedef boost::accumulators::accumulator_set<double, boost::accumulators::stats<boost::accumulators::tag::count, boost::accumulators::tag::extended_p_square> > asQuantileType;
+	asQuantileType *asQuantile;
+
+	void reset();
+
+	PingStats();
+	~PingStats();
+};
+
+class ServerItem;
+
+class ServerView : public QTreeWidget {
+		Q_OBJECT;
+		Q_DISABLE_COPY(ServerView);
+	public:
+		ServerItem *siFavorite, *siLAN, *siPublic;
+		QMap<QString, QString> qmContinentNames;
+		QMap<QString, ServerItem *> qmCountry;
+		QMap<QString, ServerItem *> qmContinent;
+
+		ServerView(QWidget *);
+		~ServerView();
+
+		void fixupName(ServerItem *si);
+
+		ServerItem *getParent(const QString &continent, const QString &countrycode, const QString &countryname, const QString &usercontinentcode, const QString &usercountrycode);
 	protected:
-		void initAccumulator();
+		virtual QMimeData *mimeData(const QList<QTreeWidgetItem *>) const;
+		virtual QStringList mimeTypes() const;
+		virtual Qt::DropActions supportedDropActions() const;
+		virtual bool dropMimeData(QTreeWidgetItem *, int, const QMimeData *, Qt::DropAction);
+};
+
+#include "ui_ConnectDialog.h"
+#include "ui_ConnectDialogEdit.h"
+
+class ServerItem : public QTreeWidgetItem, public PingStats {
+		Q_DISABLE_COPY(ServerItem);
+	protected:
+		void init();
 	public:
 		enum ItemType { FavoriteType, LANType, PublicType };
 
 		static QMap<QString, QIcon> qmIcons;
 
+		bool bParent;
+		ServerItem *siParent;
+		QList<ServerItem *> qlChildren;
+
 		QString qsName;
 
 		QString qsHostname;
@@ -72,6 +129,7 @@ class ServerItem : public QTreeWidgetItem {
 
 		QString qsCountry;
 		QString qsCountryCode;
+		QString qsContinentCode;
 
 		QString qsUrl;
 
@@ -80,33 +138,36 @@ class ServerItem : public QTreeWidgetItem {
 
 		QList<QHostAddress> qlAddresses;
 
-		typedef boost::accumulators::accumulator_set<double, boost::accumulators::stats<boost::accumulators::tag::count, boost::accumulators::tag::non_coherent_tail_mean<boost::accumulators::right> > > asRightType;
-		asRightType *asRight;
-
 		ItemType itType;
-		quint32 uiPing;
-		quint32 uiUsers;
-		quint32 uiMaxUsers;
-		quint32 uiBandwidth;
-		quint32 uiSent;
-
-		double dPing;
 
 		ServerItem(const FavoriteServer &fs);
 		ServerItem(const PublicInfo &pi);
 		ServerItem(const QString &name, const QString &host, unsigned short port, const QString &uname);
 		ServerItem(const BonjourRecord &br);
+		ServerItem(const QString &name, ItemType itype, const QString &continent = QString(), const QString &country = QString());
+		ServerItem(const ServerItem *si);
+		~ServerItem();
+		static ServerItem *fromMimeData(const QMimeData *mime, QWidget *p = NULL);
+
+		void addServerItem(ServerItem *child);
 
 		FavoriteServer toFavoriteServer() const;
+		QMimeData *toMimeData() const;
 
 		static QIcon loadIcon(const QString &name);
 
-		void setDatas();
+		void setDatas(double ping = 0.0, quint32 users = 0, quint32 maxusers = 0);
 		bool operator< (const QTreeWidgetItem &) const;
 
 		QVariant data(int column, int role) const;
 
 		void hideCheck();
+
+#if QT_VERSION < 0x040500
+		void emitDataChanged();
+	private:
+		bool m_emitDataChanged;
+#endif
 };
 
 class ConnectDialogEdit : public QDialog, protected Ui::ConnectDialogEdit {
@@ -118,6 +179,8 @@ class ConnectDialogEdit : public QDialog, protected Ui::ConnectDialogEdit {
 	public slots:
 		void validate();
 		void accept();
+
+		void on_qleServer_textChanged(const QString&);
 	public:
 		QString qsName, qsHostname, qsUsername;
 		unsigned short usPort;
@@ -125,13 +188,14 @@ class ConnectDialogEdit : public QDialog, protected Ui::ConnectDialogEdit {
 };
 
 class ConnectDialog : public QDialog, public Ui::ConnectDialog {
+		friend class ServerView;
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(ConnectDialog)
 	protected:
 		static QList<PublicInfo> qlPublicServers;
+		static QString qsUserCountry, qsUserCountryCode, qsUserContinentCode;
 		static Timer tPublicServers;
-		static int iPingIndex;
 
 		QMenu *qmPopup, *qmFilters;
 		QActionGroup *qagFilters;
@@ -139,26 +203,37 @@ class ConnectDialog : public QDialog, public Ui::ConnectDialog {
 		bool bPublicInit;
 
 		Timer tPing;
-		Timer tCurrent, tHover;
+		Timer tCurrent, tHover, tRestart;
 		QUdpSocket *qusSocket4;
 		QUdpSocket *qusSocket6;
 		QTimer *qtPingTick;
+		QList<ServerItem *> qlItems;
+
+		QList<QString> qlDNSLookup;
+		QSet<QString> qsDNSActive;
+		QHash<QString, QSet<ServerItem *> > qhDNSWait;
+		QHash<QString, QList<QHostAddress> > qhDNSCache;
+
 		QHash<qpAddress, quint64> qhPingRand;
 		QHash<qpAddress, QSet<ServerItem *> > qhPings;
-		QMap<QString, ServerItem *> qmLookups;
-		QHash<QString, QList<QHostAddress> > qhDNSCache;
+
+		QMap<QPair<QString, unsigned short>, unsigned int> qmPingCache;
+
 		bool bIPv4;
 		bool bIPv6;
+		int iPingIndex;
 
 		bool bLastFound;
 
 		QMap<QString, QIcon> qmIcons;
 
-		void pingList();
-		void sendPing(ServerItem *, const QHostAddress &, unsigned short port);
+		void sendPing(const QHostAddress &, unsigned short port);
 
 		void initList();
 		void fillList();
+
+		void startDns(ServerItem *);
+		void stopDns(ServerItem *);
 	public slots:
 		void accept();
 		void finished();
@@ -167,27 +242,35 @@ class ConnectDialog : public QDialog, public Ui::ConnectDialog {
 		void lookedUp(QHostInfo);
 		void timeTick();
 
+		void on_qaFavoriteAdd_triggered();
+		void on_qaFavoriteAddNew_triggered();
+		void on_qaFavoriteEdit_triggered();
+		void on_qaFavoriteRemove_triggered();
+		void on_qaFavoriteCopy_triggered();
+		void on_qaFavoritePaste_triggered();
+		void on_qaUrl_triggered();
+		void onFiltersTriggered(QAction *);
+		void on_qtwServers_currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *);
+		void on_qtwServers_itemDoubleClicked(QTreeWidgetItem *, int);
+		void on_qtwServers_customContextMenuRequested(const QPoint &);
+		void on_qtwServers_itemExpanded(QTreeWidgetItem *);
+		void OnSortChanged(int, Qt::SortOrder);
+	public:
+		QString qsServer, qsUsername, qsPassword;
+		unsigned short usPort;
+		ConnectDialog(QWidget *parent);
+		~ConnectDialog();
+
 #ifdef USE_BONJOUR
+	protected:
+		QList<BonjourRecord> qlBonjourActive;
+	public slots:
 		void onUpdateLanList(const QList<BonjourRecord> &);
 		void onLanBrowseError(DNSServiceErrorType);
 
 		void onResolved(BonjourRecord, QString, int);
 		void onLanResolveError(BonjourRecord, DNSServiceErrorType);
 #endif
-		void on_qaFavoriteAdd_triggered();
-		void on_qaFavoriteAddNew_triggered();
-		void on_qaFavoriteEdit_triggered();
-		void on_qaFavoriteRemove_triggered();
-		void on_qaUrl_triggered();
-		void onFiltersTriggered(QAction *);
-		void on_qtwServers_currentItemChanged(QTreeWidgetItem *, QTreeWidgetItem *);
-		void on_qtwServers_itemDoubleClicked(QTreeWidgetItem *, int);
-		void on_qtwServers_customContextMenuRequested(const QPoint &);
-	public:
-		QString qsServer, qsUsername, qsPassword;
-		unsigned short usPort;
-		ConnectDialog(QWidget *parent);
-		~ConnectDialog();
 };
 
 #else
