@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -84,6 +84,13 @@ Database::Database() {
 		qFatal("Database: Failed initialization");
 	}
 
+	QFileInfo fi(db.databaseName());
+
+	if (! fi.isWritable()) {
+		QMessageBox::critical(NULL, tr("Mumble"), tr("The database '%1' is read-only. Mumble can not store server settings (ie. SSL certificates) until you fix this problem.").arg(fi.filePath()), QMessageBox::Ok | QMessageBox::Default, QMessageBox::NoButton);
+		qWarning("Database: Database is read-only");
+	}
+
 	QSqlQuery query;
 
 	query.exec(QLatin1String("CREATE TABLE IF NOT EXISTS `servers` (`id` INTEGER PRIMARY KEY AUTOINCREMENT, `name` TEXT, `hostname` TEXT, `port` INTEGER DEFAULT 64738, `username` TEXT, `password` TEXT)"));
@@ -109,6 +116,12 @@ Database::Database() {
 	query.exec(QLatin1String("CREATE UNIQUE INDEX IF NOT EXISTS `friends_name` ON `friends`(`name`)"));
 	query.exec(QLatin1String("CREATE UNIQUE INDEX IF NOT EXISTS `friends_hash` ON `friends`(`hash`)"));
 
+	query.exec(QLatin1String("CREATE TABLE IF NOT EXISTS `muted` (`id` INTEGER PRIMARY KEY AUTOINCREMENT, `hash` TEXT)"));
+	query.exec(QLatin1String("CREATE UNIQUE INDEX IF NOT EXISTS `muted_hash` ON `muted`(`hash`)"));
+
+	query.exec(QLatin1String("CREATE TABLE IF NOT EXISTS `pingcache` (`id` INTEGER PRIMARY KEY AUTOINCREMENT, `hostname` TEXT, `port` INTEGER, `ping` INTEGER)"));
+	query.exec(QLatin1String("CREATE UNIQUE INDEX IF NOT EXISTS `pingcache_host_port` ON `pingcache`(`hostname`,`port`)"));
+
 	query.exec(QLatin1String("DELETE FROM `comments` WHERE `seen` < datetime('now', '-1 years')"));
 }
 
@@ -122,7 +135,7 @@ QList<FavoriteServer> Database::getFavorites() {
 		FavoriteServer fs;
 		fs.qsName = query.value(0).toString();
 		fs.qsHostname = query.value(1).toString();
-		fs.usPort = query.value(2).toUInt();
+		fs.usPort = static_cast<unsigned short>(query.value(2).toUInt());
 		fs.qsUsername = query.value(3).toString();
 		fs.qsPassword = query.value(4).toString();
 		fs.qsUrl = query.value(5).toString();
@@ -152,6 +165,61 @@ void Database::setFavorites(const QList<FavoriteServer> &servers) {
 	QSqlDatabase::database().commit();
 }
 
+bool Database::isLocalMuted(const QString &hash) {
+	QSqlQuery query;
+
+	query.prepare(QLatin1String("SELECT `hash` FROM `muted` WHERE `hash` = ?"));
+	query.addBindValue(hash);
+	query.exec();
+	while (query.next()) {
+		return true;
+	}
+	return false;
+}
+
+void Database::setLocalMuted(const QString &hash, bool muted) {
+	QSqlQuery query;
+
+	if (muted)
+		query.prepare(QLatin1String("INSERT INTO `muted` (`hash`) VALUES (?)"));
+	else
+		query.prepare(QLatin1String("DELETE FROM `muted` WHERE `hash` = ?"));
+	query.addBindValue(hash);
+	query.exec();
+}
+
+QMap<QPair<QString, unsigned short>, unsigned int> Database::getPingCache() {
+	QSqlQuery query;
+	QMap<QPair<QString, unsigned short>, unsigned int> map;
+
+	query.prepare(QLatin1String("SELECT `hostname`, `port`, `ping` FROM `pingcache`"));
+	query.exec();
+	while (query.next()) {
+		map.insert(QPair<QString, unsigned short>(query.value(0).toString(), query.value(1).toUInt()), query.value(2).toUInt());
+	}
+	return map;
+}
+
+void Database::setPingCache(const QMap<QPair<QString, unsigned short>, unsigned int> &map) {
+	QSqlQuery query;
+	QMap<QPair<QString, unsigned short>, unsigned int>::const_iterator i;
+
+	QSqlDatabase::database().transaction();
+
+	query.prepare(QLatin1String("DELETE FROM `pingcache`"));
+	query.exec();
+
+	query.prepare(QLatin1String("REPLACE INTO `pingcache` (`hostname`, `port`, `ping`) VALUES (?,?,?)"));
+	for (i = map.constBegin(); i != map.constEnd(); ++i) {
+		query.addBindValue(i.key().first);
+		query.addBindValue(i.key().second);
+		query.addBindValue(i.value());
+		query.exec();
+	}
+
+	QSqlDatabase::database().commit();
+}
+
 bool Database::seenComment(const QString &hash, const QString &comment) {
 	QSqlQuery query;
 
