@@ -1,4 +1,5 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -49,6 +50,7 @@ class QSettings;
 class Overlay;
 class LCD;
 class BonjourClient;
+class CELTCodec;
 
 struct Global {
 private:
@@ -72,6 +74,7 @@ public:
 	Timer tDoublePush;
 	quint64 uiDoublePush;
 	int iTarget;
+	int iPrevTarget;
 	bool bPushToMute;
 	bool bCenterPosition;
 	bool bPosTest;
@@ -80,8 +83,12 @@ public:
 	ChanACL::Permissions pPermissions;
 	int iMaxBandwidth;
 	int iAudioBandwidth;
-	int iAudioQuality;
 	QDir qdBasePath;
+	QString qsRegionalHost;
+	QMap<int, CELTCodec *> qmCodecs;
+	int iCodecAlpha, iCodecBeta;
+	bool bPreferAlpha;
+	bool bAttenuateOthers;
 	Global();
 	~Global();
 };
