@@ -50,6 +50,7 @@ GlobalShortcutEngine *GlobalShortcutEngine::platformInit() {
 
 GlobalShortcutWin::GlobalShortcutWin() {
 	pDI = NULL;
+	uiHardwareDevices = 0;
 	moveToThread(this);
 	start(QThread::LowestPriority);
 }
@@ -76,11 +77,11 @@ void GlobalShortcutWin::run() {
 	while (! g.mw)
 		this->msleep(20);
 
-	pDI->EnumDevices(DI8DEVCLASS_ALL, EnumDevicesCB, static_cast<void *>(this), DIEDFL_ALLDEVICES);
-
 	GetModuleHandleEx(GET_MODULE_HANDLE_EX_FLAG_FROM_ADDRESS | GET_MODULE_HANDLE_EX_FLAG_UNCHANGED_REFCOUNT, (wchar_t *) &HookKeyboard, &hSelf);
+#ifdef QT_NO_DEBUG
 	hhKeyboard = SetWindowsHookEx(WH_KEYBOARD_LL, HookKeyboard, hSelf, 0);
 	hhMouse = SetWindowsHookEx(WH_MOUSE_LL, HookMouse, hSelf, 0);
+#endif
 
 	timer = new QTimer(this);
 	connect(timer, SIGNAL(timeout()), this, SLOT(timeTicked()));
@@ -98,6 +99,7 @@ void GlobalShortcutWin::run() {
 			id->pDID->Unacquire();
 			id->pDID->Release();
 		}
+		delete id;
 	}
 	pDI->Release();
 }
@@ -174,10 +176,18 @@ BOOL GlobalShortcutWin::EnumDevicesCB(LPCDIDEVICEINSTANCE pdidi, LPVOID pContext
 	QString sname = QString::fromUtf16(reinterpret_cast<const ushort *>(pdidi->tszInstanceName));
 
 	InputDevice *id = new InputDevice;
+	id->pDID = NULL;
 	id->name = name;
 	id->guid = pdidi->guidInstance;
 	id->vguid = QVariant(QUuid(id->guid).toString());
 
+	foreach(InputDevice *dev, cbgsw->qhInputDevices) {
+		if (dev->guid == id->guid) {
+			delete id;
+			return DIENUM_CONTINUE;
+		}
+	}
+
 	if (FAILED(hr = cbgsw->pDI->CreateDevice(pdidi->guidInstance, &id->pDID, NULL)))
 		qFatal("GlobalShortcutWin: CreateDevice: %lx", hr);
 
@@ -189,7 +199,7 @@ BOOL GlobalShortcutWin::EnumDevicesCB(LPCDIDEVICEINSTANCE pdidi, LPVOID pContext
 		qSort(types);
 
 		int nbuttons = types.count();
-		DIOBJECTDATAFORMAT *rgodf = new DIOBJECTDATAFORMAT[nbuttons];
+		STACKVAR(DIOBJECTDATAFORMAT, rgodf, nbuttons);
 		DIDATAFORMAT df;
 		ZeroMemory(&df, sizeof(df));
 		df.dwSize = sizeof(df);
@@ -226,17 +236,22 @@ BOOL GlobalShortcutWin::EnumDevicesCB(LPCDIDEVICEINSTANCE pdidi, LPVOID pContext
 			qFatal("GlobalShortcutWin::SetProperty");
 
 		qWarning("Adding device %s %s %s:%d", qPrintable(QUuid(id->guid).toString()),qPrintable(name),qPrintable(sname),id->qhNames.count());
-
 		cbgsw->qhInputDevices[id->guid] = id;
 	} else {
 		id->pDID->Release();
-		delete(id);
+		delete id;
 	}
 
 	return DIENUM_CONTINUE;
 }
 
 void GlobalShortcutWin::timeTicked() {
+	if (g.mw->uiNewHardware != uiHardwareDevices) {
+		uiHardwareDevices = g.mw->uiNewHardware;
+
+		pDI->EnumDevices(DI8DEVCLASS_ALL, EnumDevicesCB, static_cast<void *>(this), DIEDFL_ATTACHEDONLY);
+	}
+
 	if (bNeedRemap)
 		remap();
 
@@ -245,7 +260,24 @@ void GlobalShortcutWin::timeTicked() {
 		DWORD   dwItems = DX_SAMPLE_BUFFER_SIZE;
 		HRESULT hr;
 
-		id->pDID->Acquire();
+		hr = id->pDID->Acquire();
+
+		switch (hr) {
+			case DI_OK:
+			case S_FALSE:
+				break;
+			case DIERR_UNPLUGGED:
+			case DIERR_GENERIC:
+				qWarning("Removing device %s", qPrintable(QUuid(id->guid).toString()));
+				id->pDID->Release();
+				qhInputDevices.remove(id->guid);
+				delete id;
+				return;
+			case DIERR_OTHERAPPHASPRIO:
+				continue;
+			default:
+				break;
+		}
 		id->pDID->Poll();
 
 		hr = id->pDID->GetDeviceData(sizeof(DIDEVICEOBJECTDATA), rgdod, &dwItems, 0);
