@@ -32,6 +32,8 @@
 #include "Log.h"
 #include "TextToSpeech.h"
 #include "MainWindow.h"
+#include "Channel.h"
+#include "ServerHandler.h"
 #include "Global.h"
 
 #ifdef Q_OS_MAC
@@ -238,6 +240,41 @@ QString Log::msgName(MsgType t) const {
 	return tr(msgNames[t]);
 }
 
+const char *Log::colorClasses[] = {
+	"time",
+	"server",
+	"privilege"
+};
+
+QString Log::msgColor(const QString &text, LogColorType t) {
+	QString classname;
+
+	return QString::fromLatin1("<span class='log-%1'>%2</span>").arg(QString::fromLatin1(colorClasses[t])).arg(text);
+}
+
+QString Log::formatChannel(::Channel *c) {
+	return QString::fromLatin1("<a href='channelid://%1/%3' class='log-channel'>%2</a>").arg(c->iId).arg(c->qsName).arg(QString::fromLatin1(g.sh->qbaDigest.toBase64()));
+}
+
+QString Log::formatClientUser(ClientUser *cu, LogColorType t) {
+	QString className;
+	if (t == Log::Target) {
+		className = QString::fromLatin1("target");
+	} else if (t == Log::Source) {
+		className = QString::fromLatin1("source");
+	}
+
+	if (cu) {
+		if (cu->qsHash.isEmpty()) {
+			return QString::fromLatin1("<a href='clientid://%2/%4' class='log-user log-%1'>%3</a>").arg(className).arg(cu->uiSession).arg(cu->qsName).arg(QString::fromLatin1(g.sh->qbaDigest.toBase64()));
+		} else {
+			return QString::fromLatin1("<a href='clientid://%2' class='log-user log-%1'>%3</a>").arg(className).arg(cu->qsHash).arg(cu->qsName);
+		}
+	} else {
+		return QString::fromLatin1("<span class='log-server log-%1'>%2</span>").arg(className).arg(tr("the server"));
+	}
+}
+
 void Log::setIgnore(MsgType t, int ignore) {
 	qmIgnore.insert(t, ignore);
 }
@@ -315,6 +352,9 @@ QString Log::validHtml(const QString &html, bool allowReplacement) {
 	qslValid << QLatin1String("http");
 	qslValid << QLatin1String("https");
 	qslValid << QLatin1String("ftp");
+	qslValid << QLatin1String("clientid");
+	qslValid << QLatin1String("channelid");
+	qslValid << QLatin1String("spotify");
 
 	QRectF qr = dw.availableGeometry(dw.screenNumber(g.mw));
 	qtd.setTextWidth(qr.width() / 2);
@@ -391,7 +431,7 @@ void Log::log(MsgType mt, const QString &console, const QString &terse) {
 		} else if (! g.mw->qteLog->document()->isEmpty()) {
 			tc.insertBlock();
 		}
-		tc.insertHtml(QString::fromLatin1("[%2] %1\n").arg(validHtml(console, true), dt.time().toString(Qt::DefaultLocaleShortDate)));
+		tc.insertHtml(QString::fromLatin1("[%2] %1\n").arg(validHtml(console, true), Log::msgColor(dt.time().toString(Qt::DefaultLocaleShortDate), Log::Time)));
 		tc.movePosition(QTextCursor::End);
 		g.mw->qteLog->setTextCursor(tc);
 		g.mw->qteLog->ensureCursorVisible();
