@@ -1,4 +1,5 @@
 /* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -59,6 +60,9 @@ class LogConfig : public ConfigWidget, public Ui::LogConfig {
 		void browseForAudioFile();
 };
 
+class ClientUser;
+class Channel;
+
 class Log : public QObject {
 		friend class LogConfig;
 	private:
@@ -66,11 +70,13 @@ class Log : public QObject {
 		Q_DISABLE_COPY(Log)
 	public:
 		enum MsgType { DebugInfo, CriticalError, Warning, Information, ServerConnected, ServerDisconnected, UserJoin, UserLeave, YouKicked, UserKicked, SelfMute, OtherSelfMute, YouMuted, YouMutedOther, OtherMutedOther, ChannelJoin, ChannelLeave, PermissionDenied, TextMessage };
+		enum LogColorType { Time, Server, Privilege, Source, Target };
 		static const MsgType firstMsgType = DebugInfo;
 		static const MsgType lastMsgType = TextMessage;
 	protected:
 		QHash<MsgType, int> qmIgnore;
 		static const char *msgNames[];
+		static const char *colorClasses[];
 		TextToSpeech *tts;
 		unsigned int uiLastId;
 		QDate qdDate;
@@ -80,19 +86,34 @@ class Log : public QObject {
 		void setIgnore(MsgType t, int ignore = 1 << 30);
 		void clearIgnore();
 		static QString validHtml(const QString &html, bool allowReplacement = false);
+		static QString imageToImg(const QByteArray &format, const QByteArray &image);
+		static QString imageToImg(QImage img);
+		static QString msgColor(const QString &text, LogColorType t);
+		static QString formatClientUser(ClientUser *cu, LogColorType t);
+		static QString formatChannel(::Channel *c);
 	public slots:
 		void log(MsgType t, const QString &console, const QString &terse=QString());
 };
 
+class ValidDocument : public QTextDocument {
+	private:
+		Q_OBJECT
+		Q_DISABLE_COPY(ValidDocument)
+	protected:
+		QStringList qslValidImage;
+		bool bValid;
+		QVariant loadResource(int, const QUrl &);
+	public:
+		ValidDocument(bool httpimages, QObject *p = NULL);
+		bool isValid() const;
+};
+
 class LogDocument : public QTextDocument {
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(LogDocument)
 	public:
-		QVariant qvVariant;
-		int iType;
-		QUrl quUrl;
-		LogDocument(QObject *p);
+		LogDocument(QObject *p = NULL);
 		QVariant loadResource(int, const QUrl &);
 	public slots:
 		void receivedHead();
