@@ -82,7 +82,7 @@ void LookConfig::load(const Settings &r) {
 	loadComboBox(qcbChannelDrag, 0);
 
 	// Load Layout checkbox state
-	switch (r.iWindowLayout) {
+	switch (r.wlWindowLayout) {
 		case Settings::LayoutClassic:
 			qrbLClassic->setChecked(true);
 			break;
@@ -94,7 +94,7 @@ void LookConfig::load(const Settings &r) {
 			break;
 		case Settings::LayoutCustom:
 		default:
-			s.iWindowLayout = Settings::LayoutCustom;
+			s.wlWindowLayout = Settings::LayoutCustom;
 			qrbLCustom->setChecked(true);
 			break;
 	}
@@ -113,14 +113,16 @@ void LookConfig::load(const Settings &r) {
 		}
 	}
 
+	loadComboBox(qcbAlwaysOnTop, r.aotbAlwaysOnTop);
+
 	qleCSS->setText(r.qsSkin);
 	loadComboBox(qcbExpand, r.ceExpand);
 	loadComboBox(qcbChannelDrag, r.ceChannelDrag);
 	loadCheckBox(qcbShowChatbar, r.bShowChatbar);
 	loadCheckBox(qcbUsersTop, r.bUserTop);
-	loadCheckBox(qcbAlwaysOnTop, r.bAlwaysOnTop);
 	loadCheckBox(qcbAskOnQuit, r.bAskOnQuit);
 	loadCheckBox(qcbHideTray, r.bHideTray);
+	loadCheckBox(qcbStateInTray, r.bStateInTray);
 	loadCheckBox(qcbShowUserCount, r.bShowUserCount);
 }
 
@@ -142,22 +144,23 @@ void LookConfig::save() const {
 
 	// Save Layout radioboxes state
 	if (qrbLClassic->isChecked()) {
-		s.iWindowLayout = Settings::LayoutClassic;
+		s.wlWindowLayout = Settings::LayoutClassic;
 	} else if (qrbLStacked->isChecked()) {
-		s.iWindowLayout = Settings::LayoutStacked;
+		s.wlWindowLayout = Settings::LayoutStacked;
 	} else if (qrbLHybrid->isChecked()) {
-		s.iWindowLayout = Settings::LayoutHybrid;
+		s.wlWindowLayout = Settings::LayoutHybrid;
 	} else {
-		s.iWindowLayout = Settings::LayoutCustom;
+		s.wlWindowLayout = Settings::LayoutCustom;
 	}
 
 	s.ceExpand=static_cast<Settings::ChannelExpand>(qcbExpand->currentIndex());
 	s.ceChannelDrag=static_cast<Settings::ChannelDrag>(qcbChannelDrag->currentIndex());
 	s.bShowChatbar = qcbShowChatbar->isChecked();
 	s.bUserTop=qcbUsersTop->isChecked();
-	s.bAlwaysOnTop = qcbAlwaysOnTop->isChecked();
+	s.aotbAlwaysOnTop = static_cast<Settings::AlwaysOnTopBehaviour>(qcbAlwaysOnTop->currentIndex());
 	s.bAskOnQuit = qcbAskOnQuit->isChecked();
 	s.bHideTray = qcbHideTray->isChecked();
+	s.bStateInTray = qcbStateInTray->isChecked();
 	s.bShowUserCount = qcbShowUserCount->isChecked();
 }
 
@@ -166,8 +169,8 @@ void LookConfig::accept() const {
 		qApp->setStyle(s.qsStyle);
 	}
 	if (s.qsSkin.isEmpty()) {
-		qApp->setStyleSheet(QString());
-		g.mw->qteLog->document()->setDefaultStyleSheet(QString());
+		qApp->setStyleSheet(MainWindow::defaultStyleSheet);
+		g.mw->qteLog->document()->setDefaultStyleSheet(qApp->styleSheet());
 	} else {
 		QFile file(s.qsSkin);
 		file.open(QFile::ReadOnly);
@@ -185,6 +188,7 @@ bool LookConfig::expert(bool b) {
 	qcbUsersTop->setVisible(b);
 	qcbStyle->setVisible(b);
 	qliStyle->setVisible(b);
+	qcbStateInTray->setVisible(b);
 	return true;
 }
 
