@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -57,6 +57,10 @@
 #include "NetworkConfig.h"
 #include "ACL.h"
 
+#ifdef Q_OS_WIN
+#include "TaskList.h"
+#endif
+
 /*!
   \fn void MainWindow::findDesiredChannel()
   This function tries to join a desired channel on connect. It gets called
@@ -96,16 +100,32 @@
   \see MainWindow::msgPermissionQuery(const MumbleProto::PermissionQuery &msg)
 */
 
+/*!
+  \fn QPair<QByteArray, QImage> MainWindow::openImageFile()
+  Presents a file open dialog, opens the selected picture and returns it.
+  \return Pair consisting of the raw file contents and the image. Unitialized on error or cancel.
+*/
+
 MessageBoxEvent::MessageBoxEvent(QString m) : QEvent(static_cast<QEvent::Type>(MB_QEVENT)) {
 	msg = m;
 }
 
+OpenURLEvent::OpenURLEvent(QUrl u) : QEvent(static_cast<QEvent::Type>(OU_QEVENT)) {
+	url = u;
+}
+
+const QString MainWindow::defaultStyleSheet = QLatin1String(".log-channel{text-decoration:none;}.log-user{text-decoration:none;}");
+
 MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
 	qiIconMuteSelf.addFile(QLatin1String("skin:muted_self.svg"));
 	qiIconMuteServer.addFile(QLatin1String("skin:muted_server.svg"));
 	qiIconMuteSuppressed.addFile(QLatin1String("skin:muted_suppressed.svg"));
 	qiIconDeafSelf.addFile(QLatin1String("skin:deafened_self.svg"));
 	qiIconDeafServer.addFile(QLatin1String("skin:deafened_server.svg"));
+	qiTalkingOff.addFile(QLatin1String("skin:talking_off.svg"));
+	qiTalkingOn.addFile(QLatin1String("skin:talking_on.svg"));
+	qiTalkingWhisperChannel.addFile(QLatin1String("skin:talking_alt.svg"));
+	qiTalkingWhisper.addFile(QLatin1String("skin:talking_whisper.svg"));
 
 	qiIcon.addFile(QLatin1String("skin:mumble.svg"));
 #ifdef Q_OS_MAC
@@ -132,6 +152,9 @@ MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
 	tokenEdit = NULL;
 	bNoHide = false;
 
+	cmUid = -1;
+	cmCid = -1;
+
 	qtReconnect = new QTimer(this);
 	qtReconnect->setInterval(10000);
 	qtReconnect->setSingleShot(true);
@@ -154,7 +177,10 @@ MainWindow::MainWindow(QWidget *p) : QMainWindow(p) {
 	on_qmUser_aboutToShow();
 	on_qmConfig_aboutToShow();
 
-	setOnTop(g.s.bAlwaysOnTop);
+	setOnTop(g.s.aotbAlwaysOnTop == Settings::OnTopAlways ||
+	         (g.s.bMinimalView && g.s.aotbAlwaysOnTop == Settings::OnTopInMinimal) ||
+	         (!g.s.bMinimalView && g.s.aotbAlwaysOnTop == Settings::OnTopInNormal));
+
 #ifdef NO_UPDATE_CHECK
 	delete qaHelpVersionCheck;
 #endif
@@ -225,6 +251,11 @@ void MainWindow::setupGui()  {
 	QMenu *qmWindow = new QMenu(tr("&Window"), this);
 	menubar->insertMenu(qmHelp->menuAction(), qmWindow);
 	qmWindow->addAction(tr("Minimize"), this, SLOT(showMinimized()), QKeySequence(tr("Ctrl+M")));
+
+	qtvUsers->setAttribute(Qt::WA_MacShowFocusRect, false);
+	qleChat->setFrame(false);
+	qleChat->setAttribute(Qt::WA_MacShowFocusRect, false);
+	qteLog->setFrameStyle(QFrame::NoFrame);
 #endif
 
 	LogDocument *ld = new LogDocument(qteLog);
@@ -235,6 +266,7 @@ void MainWindow::setupGui()  {
 	pmModel = new UserModel(qtvUsers);
 	qtvUsers->setModel(pmModel);
 	qtvUsers->setRowHidden(0, QModelIndex(), true);
+	qtvUsers->ensurePolished();
 
 	qaServerConnect->setShortcuts(QKeySequence::Open);
 	qaServerDisconnect->setShortcuts(QKeySequence::Close);
@@ -261,20 +293,23 @@ void MainWindow::setupGui()  {
 	dtbChatDockTitle = new DockTitleBar();
 	qdwChat->setTitleBarWidget(dtbChatDockTitle);
 	qdwChat->installEventFilter(dtbChatDockTitle);
-	qleChat->setDefaultText(tr("Not connected"));
+	qleChat->setDefaultText(tr("Not connected"), true);
 	qleChat->setEnabled(false);
 
+	connect(qtvUsers->selectionModel(),
+	        SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)),
+	        SLOT(qtvUserCurrentChanged(const QModelIndex &, const QModelIndex &)));
+
+	setupView(false);
+
 	if (g.s.bMinimalView && ! g.s.qbaMinimalViewGeometry.isNull())
 		restoreGeometry(g.s.qbaMinimalViewGeometry);
 	else if (! g.s.bMinimalView && ! g.s.qbaMainWindowGeometry.isNull())
 		restoreGeometry(g.s.qbaMainWindowGeometry);
 
+	Settings::WindowLayout wlTmp = g.s.wlWindowLayout;
 	restoreState(g.s.qbaMainWindowState);
-	qtvUsers->header()->restoreState(g.s.qbaHeaderState);
-
-	connect(qtvUsers->selectionModel(),
-	        SIGNAL(currentChanged(const QModelIndex &, const QModelIndex &)),
-	        SLOT(qtvUserCurrentChanged(const QModelIndex &, const QModelIndex &)));
+	g.s.wlWindowLayout = wlTmp;
 
 	setupView(false);
 
@@ -358,12 +393,90 @@ void MainWindow::updateTrayIcon() {
 		qstiIcon->setIcon(qiIconMuteServer);
 	} else if (p && p->bSuppress) {
 		qstiIcon->setIcon(qiIconMuteSuppressed);
+	} else if (p && g.s.bStateInTray) {
+		switch (p->tsState) {
+			case ClientUser::Talking:
+				qstiIcon->setIcon(qiTalkingOn);
+				break;
+			case ClientUser::TalkingWhisper:
+				qstiIcon->setIcon(qiTalkingWhisper);
+				break;
+			case ClientUser::TalkingWhisperChannel:
+				qstiIcon->setIcon(qiTalkingWhisperChannel);
+				break;
+			case ClientUser::TalkingOff:
+			default:
+				qstiIcon->setIcon(qiTalkingOff);
+				break;
+		}
 	} else {
 		qstiIcon->setIcon(qiIcon);
 	}
 }
 
+Channel *MainWindow::getContextMenuChannel() {
+	if (cmCid >= 0)
+		return Channel::get(cmCid);
+
+	return pmModel->getChannel(qtvUsers->currentIndex());
+}
+
+ClientUser *MainWindow::getContextMenuUser() {
+	if (cmUid >= 0)
+		return ClientUser::get(cmUid);
+
+	return pmModel->getUser(qtvUsers->currentIndex());
+}
+
+bool MainWindow::handleSpecialContextMenu(const QUrl &url, const QPoint &_pos, bool focus) {
+	if (url.scheme() == QString::fromLatin1("clientid")) {
+		bool ok = false;
+		QString x(url.host());
+		if (x.length() == 40) {
+			ClientUser *cu = ClientUser::getByHash(x);
+			if (cu) {
+				cmUid = ClientUser::getUiSession(cu);
+				ok = true;
+			}
+		} else {
+			QByteArray qbaServerDigest = QByteArray::fromBase64(url.path().remove(0, 1).toLatin1());
+			cmUid = url.host().toInt(&ok, 10);
+			ok = ok && (qbaServerDigest == g.sh->qbaDigest);
+		}
+		if (ok && ClientUser::isValid(cmUid)) {
+			if (focus)
+				qtvUsers->setCurrentIndex(pmModel->index(ClientUser::get(cmUid)));
+			else
+				qmUser->exec(_pos, NULL);
+		}
+		cmUid = -1;
+	} else if (url.scheme() == QString::fromLatin1("channelid")) {
+		bool ok;
+		QByteArray qbaServerDigest = QByteArray::fromBase64(url.path().remove(0, 1).toLatin1());
+		cmCid = url.host().toInt(&ok, 10);
+		ok = ok && (qbaServerDigest == g.sh->qbaDigest);
+		if (ok) {
+			if (focus)
+				qtvUsers->setCurrentIndex(pmModel->index(Channel::get(cmCid)));
+			else
+				qmChannel->exec(_pos, NULL);
+		}
+		cmCid = -1;
+	} else {
+		return false;
+	}
+	return true;
+}
+
 void MainWindow::on_qteLog_customContextMenuRequested(const QPoint &mpos) {
+	QString link = qteLog->anchorAt(mpos);
+	if (! link.isEmpty()) {
+		QUrl l(link);
+
+		if (handleSpecialContextMenu(l, qteLog->mapToGlobal(mpos)))
+			return;
+	}
+
 #if QT_VERSION >= 0x040400
 	QMenu *menu = qteLog->createStandardContextMenu(mpos);
 #else
@@ -420,10 +533,16 @@ void MainWindow::openUrl(const QUrl &url) {
 		patch = rx.cap(3).toInt();
 	}
 
-	if ((major != 1) || (minor != 2) || (patch > 0)) {
-		g.l->log(Log::Warning, tr("This version of Mumble can't handle URLs for Mumble version %1.%2.%3").arg(major).arg(minor).arg(patch));
+#ifdef Q_OS_MAC
+	if ((major == 1) && (minor == 1)) {
+		launchCompatibilityClient(url);
 		return;
-	}
+	} else
+#endif
+		if ((major != 1) || (minor != 2) || (patch > 1)) {
+			g.l->log(Log::Warning, tr("This version of Mumble can't handle URLs for Mumble version %1.%2.%3").arg(major).arg(minor).arg(patch));
+			return;
+		}
 
 	QString host = url.host();
 	unsigned short port = static_cast<unsigned short>(url.port(64738));
@@ -456,11 +575,12 @@ void MainWindow::openUrl(const QUrl &url) {
 	if (g.sh && g.sh->isRunning()) {
 		on_qaServerDisconnect_triggered();
 		g.sh->wait();
+		QCoreApplication::instance()->processEvents();
 	}
 
 	rtLast = MumbleProto::Reject_RejectType_None;
 	qaServerDisconnect->setEnabled(true);
-	g.l->log(Log::ServerConnected, tr("Connecting to server %1.").arg(host));
+	g.l->log(Log::Information, tr("Connecting to server %1.").arg(Log::msgColor(host, Log::Server)));
 	g.sh->setConnectionInfo(host, port, user, pw);
 	g.sh->start(QThread::TimeCriticalPriority);
 }
@@ -495,7 +615,7 @@ void MainWindow::findDesiredChannel() {
 			g.sh->sendMessage(mpus);
 		}
 		qtvUsers->setCurrentIndex(pmModel->index(chan));
-	} else {
+	} else if (g.uiSession) {
 		qtvUsers->setCurrentIndex(pmModel->index(ClientUser::get(g.uiSession)->cChannel));
 	}
 	updateMenuPermissions();
@@ -521,8 +641,8 @@ void MainWindow::setupView(bool toggle_minimize) {
 	bNoHide = true;
 
 	// Update window layout
-	int iTmp = g.s.iWindowLayout;
-	switch (iTmp) {
+	Settings::WindowLayout wlTmp = g.s.wlWindowLayout;
+	switch (wlTmp) {
 		case Settings::LayoutClassic:
 			removeDockWidget(qdwLog);
 			addDockWidget(Qt::LeftDockWidgetArea, qdwLog);
@@ -544,10 +664,10 @@ void MainWindow::setupView(bool toggle_minimize) {
 			if (g.s.bShowChatbar) qdwChat->show();
 			break;
 		default:
-			iTmp = Settings::LayoutCustom;
+			wlTmp = Settings::LayoutCustom;
 			break;
 	}
-	g.s.iWindowLayout = iTmp;
+	g.s.wlWindowLayout = wlTmp;
 
 	QRect geom = frameGeometry();
 
@@ -568,14 +688,15 @@ void MainWindow::setupView(bool toggle_minimize) {
 		f = Qt::Tool;
 #endif
 
-	if (g.s.bAlwaysOnTop)
+	if (g.s.aotbAlwaysOnTop == Settings::OnTopAlways ||
+	        (g.s.bMinimalView && g.s.aotbAlwaysOnTop == Settings::OnTopInMinimal) ||
+	        (!g.s.bMinimalView && g.s.aotbAlwaysOnTop == Settings::OnTopInNormal))
 		f |= Qt::WindowStaysOnTopHint;
 
 	setWindowFlags(f);
 
 	qdwLog->setVisible(showit);
 	qdwChat->setVisible(showit && g.s.bShowChatbar);
-	qtvUsers->header()->setVisible(showit);
 	menuBar()->setVisible(showit);
 
 	if (toggle_minimize) {
@@ -606,13 +727,14 @@ void MainWindow::on_qaServerConnect_triggered() {
 	if (g.sh && g.sh->isRunning() && res == QDialog::Accepted) {
 		on_qaServerDisconnect_triggered();
 		g.sh->wait();
+		QCoreApplication::instance()->processEvents();
 	}
 
 	if (res == QDialog::Accepted) {
 		qsDesiredChannel = QString();
 		rtLast = MumbleProto::Reject_RejectType_None;
 		qaServerDisconnect->setEnabled(true);
-		g.l->log(Log::ServerConnected, tr("Connecting to server %1.").arg(cd->qsServer));
+		g.l->log(Log::Information, tr("Connecting to server %1.").arg(Log::msgColor(cd->qsServer, Log::Server)));
 		g.sh->setConnectionInfo(cd->qsServer, cd->usPort, cd->qsUsername, cd->qsPassword);
 		g.sh->start(QThread::TimeCriticalPriority);
 	}
@@ -622,7 +744,7 @@ void MainWindow::on_qaServerConnect_triggered() {
 void MainWindow::on_Reconnect_timeout() {
 	if (g.sh->isRunning())
 		return;
-	g.l->log(Log::ServerDisconnected, tr("Reconnecting."));
+	g.l->log(Log::Information, tr("Reconnecting."));
 	g.sh->start(QThread::TimeCriticalPriority);
 }
 
@@ -785,7 +907,8 @@ void MainWindow::on_qaServerTokens_triggered() {
 }
 
 void MainWindow::on_qmUser_aboutToShow() {
-	ClientUser *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
+
 	bool self = p && (p->uiSession == g.uiSession);
 
 	qmUser->clear();
@@ -797,7 +920,14 @@ void MainWindow::on_qmUser_aboutToShow() {
 	qmUser->addAction(qaUserMute);
 	qmUser->addAction(qaUserDeaf);
 	qmUser->addAction(qaUserLocalMute);
-	qmUser->addAction(qaUserComment);
+
+	if (self)
+		qmUser->addAction(qaUserComment);
+	else {
+		qmUser->addAction(qaUserCommentView);
+		qmUser->addAction(qaUserCommentReset);
+	}
+
 	qmUser->addAction(qaUserTextMessage);
 
 	if (p && (p->iId < 0) && (g.pPermissions & ((self ? ChanACL::SelfRegister : ChanACL::Register) | ChanACL::Write))) {
@@ -840,9 +970,12 @@ void MainWindow::on_qmUser_aboutToShow() {
 		qaUserKick->setEnabled(false);
 		qaUserBan->setEnabled(false);
 		qaUserMute->setEnabled(false);
-		qaUserLocalMute->setEnabled(false);
 		qaUserDeaf->setEnabled(false);
 		qaUserTextMessage->setEnabled(false);
+		qaUserLocalMute->setEnabled(false);
+		qaUserComment->setEnabled(false);
+		qaUserCommentReset->setEnabled(false);
+		qaUserCommentView->setEnabled(false);
 	} else {
 		qaUserKick->setEnabled(! self);
 		qaUserBan->setEnabled(! self);
@@ -850,7 +983,9 @@ void MainWindow::on_qmUser_aboutToShow() {
 		qaUserDeaf->setEnabled(! self || p->bDeaf);
 		qaUserTextMessage->setEnabled(true);
 		qaUserLocalMute->setEnabled(! self);
-		qaUserComment->setEnabled(self || (g.pPermissions & (ChanACL::Move | ChanACL::Write)));
+		qaUserComment->setEnabled(self);
+		qaUserCommentReset->setEnabled(!p->qsComment.isEmpty() && (g.pPermissions & (ChanACL::Move | ChanACL::Write)));
+		qaUserCommentView->setEnabled(! p->qsComment.isEmpty());
 
 		qaUserMute->setChecked(p->bMute || p->bSuppress);
 		qaUserDeaf->setChecked(p->bDeaf);
@@ -860,7 +995,7 @@ void MainWindow::on_qmUser_aboutToShow() {
 }
 
 void MainWindow::on_qaUserMute_triggered() {
-	User *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 	if (!p)
 		return;
 
@@ -878,7 +1013,7 @@ void MainWindow::on_qaUserMute_triggered() {
 }
 
 void MainWindow::on_qaUserLocalMute_triggered() {
-	ClientUser *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 	if (!p)
 		return;
 
@@ -890,7 +1025,7 @@ void MainWindow::on_qaUserLocalMute_triggered() {
 }
 
 void MainWindow::on_qaUserDeaf_triggered() {
-	User *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 	if (!p)
 		return;
 
@@ -901,7 +1036,7 @@ void MainWindow::on_qaUserDeaf_triggered() {
 }
 
 void MainWindow::on_qaUserRegister_triggered() {
-	User *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 	if (!p)
 		return;
 
@@ -926,7 +1061,7 @@ void MainWindow::on_qaUserRegister_triggered() {
 }
 
 void MainWindow::on_qaUserFriendAdd_triggered() {
-	ClientUser *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 	if (!p)
 		return;
 
@@ -939,7 +1074,7 @@ void MainWindow::on_qaUserFriendUpdate_triggered() {
 }
 
 void MainWindow::on_qaUserFriendRemove_triggered() {
-	ClientUser *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 	if (!p)
 		return;
 
@@ -948,7 +1083,7 @@ void MainWindow::on_qaUserFriendRemove_triggered() {
 }
 
 void MainWindow::on_qaUserKick_triggered() {
-	User *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 	if (!p)
 		return;
 
@@ -970,7 +1105,7 @@ void MainWindow::on_qaUserKick_triggered() {
 }
 
 void MainWindow::on_qaUserBan_triggered() {
-	User *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 	if (!p)
 		return;
 
@@ -992,7 +1127,7 @@ void MainWindow::on_qaUserBan_triggered() {
 }
 
 void MainWindow::on_qaUserTextMessage_triggered() {
-	User *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 
 	if (!p)
 		return;
@@ -1012,14 +1147,14 @@ void MainWindow::on_qaUserTextMessage_triggered() {
 			mptm.add_session(p->uiSession);
 			mptm.set_message(u8(msg));
 			g.sh->sendMessage(mptm);
-			g.l->log(Log::TextMessage, tr("To %1: %2").arg(p->qsName).arg(texm->message()), tr("Message to %1").arg(p->qsName));
+			g.l->log(Log::TextMessage, tr("To %1: %2").arg(Log::msgColor(p->qsName, Log::Target)).arg(texm->message()), tr("Message to %1").arg(p->qsName));
 		}
 	}
 	delete texm;
 }
 
 void MainWindow::on_qaUserComment_triggered() {
-	User *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = getContextMenuUser();
 
 	if (!p)
 		return;
@@ -1042,6 +1177,38 @@ void MainWindow::on_qaUserComment_triggered() {
 	delete texm;
 }
 
+void MainWindow::on_qaUserCommentView_triggered() {
+	ClientUser *p = getContextMenuUser();
+
+	if (!p)
+		return;
+
+	::TextMessage *texm = new ::TextMessage(this, tr("View comment on user %1").arg(p->qsName));
+
+	texm->rteMessage->setText(p->qsComment, true);
+	texm->setAttribute(Qt::WA_DeleteOnClose, true);
+	texm->show();
+}
+
+void MainWindow::on_qaUserCommentReset_triggered() {
+	ClientUser *p = getContextMenuUser();
+
+	if (!p)
+		return;
+
+	unsigned int session = p->uiSession;
+
+	int ret = QMessageBox::question(this, tr("Mumble"),
+	                                tr("Are you sure you want to reset the comment of user %1?").arg(p->qsName),
+	                                QMessageBox::Yes, QMessageBox::No);
+	if (ret == QMessageBox::Yes) {
+		MumbleProto::UserState mpus;
+		mpus.set_session(session);
+		mpus.set_comment(std::string());
+		g.sh->sendMessage(mpus);
+	}
+}
+
 void MainWindow::on_qaQuit_triggered() {
 	bSuppressAskOnQuit = true;
 	this->close();
@@ -1050,7 +1217,7 @@ void MainWindow::on_qaQuit_triggered() {
 void MainWindow::on_qleChat_returnPressed() {
 	if (qleChat->text().isEmpty() || g.uiSession == 0) return; // Check if text & connection is available
 
-	User *p = pmModel->getUser(qtvUsers->currentIndex());
+	ClientUser *p = pmModel->getUser(qtvUsers->currentIndex());
 	Channel *c = pmModel->getChannel(qtvUsers->currentIndex());
 
 	MumbleProto::TextMessage mptm;
@@ -1069,11 +1236,11 @@ void MainWindow::on_qleChat_returnPressed() {
 			c = ClientUser::get(g.uiSession)->cChannel;
 
 		mptm.add_channel_id(c->iId);
-		g.l->log(Log::TextMessage, tr("To channel %1: %2").arg(c->qsName).arg(qsText), tr("Message to channel %1").arg(c->qsName));
+		g.l->log(Log::TextMessage, tr("To %1: %2").arg(Log::formatChannel(c)).arg(qsText), tr("Message to channel %1").arg(c->qsName));
 	} else {
 		// User message
 		mptm.add_session(p->uiSession);
-		g.l->log(Log::TextMessage, tr("To %1: %2").arg(p->qsName).arg(qsText), tr("Message to %1").arg(p->qsName));
+		g.l->log(Log::TextMessage, tr("To %1: %2").arg(Log::formatClientUser(p, Log::Target)).arg(qsText), tr("Message to %1").arg(p->qsName));
 	}
 
 	g.sh->sendMessage(mptm);
@@ -1104,9 +1271,12 @@ void MainWindow::on_qmConfig_aboutToShow() {
 }
 
 void MainWindow::on_qmChannel_aboutToShow() {
-	QModelIndex idx = qtvUsers->currentIndex();
-
 	qmChannel->clear();
+
+	if (cmCid >= 0 && g.uiSession != 0 && cmCid != ClientUser::get(g.uiSession)->cChannel->iId) {
+		qmChannel->addAction(qaChannelJoin);
+		qmChannel->addSeparator();
+	}
 	qmChannel->addAction(qaChannelAdd);
 	qmChannel->addAction(qaChannelACL);
 	qmChannel->addAction(qaChannelRemove);
@@ -1141,7 +1311,7 @@ void MainWindow::on_qmChannel_aboutToShow() {
 		msg = true;
 		descUpdate = true;
 
-		Channel *c = pmModel->getChannel(idx);
+		Channel *c = getContextMenuChannel();
 		Channel *home = ClientUser::get(g.uiSession)->cChannel;
 
 		if (c && c->bTemporary)
@@ -1172,8 +1342,19 @@ void MainWindow::on_qmChannel_aboutToShow() {
 	updateMenuPermissions();
 }
 
+void MainWindow::on_qaChannelJoin_triggered() {
+	Channel *c = getContextMenuChannel();
+
+	if (c) {
+		MumbleProto::UserState mpus;
+		mpus.set_session(g.uiSession);
+		mpus.set_channel_id(c->iId);
+		g.sh->sendMessage(mpus);
+	}
+}
+
 void MainWindow::on_qaChannelAdd_triggered() {
-	Channel *c = pmModel->getChannel(qtvUsers->currentIndex());
+	Channel *c = getContextMenuChannel();
 	if (aclEdit) {
 		aclEdit->reject();
 		delete aclEdit;
@@ -1191,7 +1372,7 @@ void MainWindow::on_qaChannelAdd_triggered() {
 
 void MainWindow::on_qaChannelRemove_triggered() {
 	int ret;
-	Channel *c = pmModel->getChannel(qtvUsers->currentIndex());
+	Channel *c = getContextMenuChannel();
 	if (! c)
 		return;
 
@@ -1211,7 +1392,7 @@ void MainWindow::on_qaChannelRemove_triggered() {
 }
 
 void MainWindow::on_qaChannelACL_triggered() {
-	Channel *c = pmModel->getChannel(qtvUsers->currentIndex());
+	Channel *c = getContextMenuChannel();
 	int id = c ? c->iId : 0;
 
 	MumbleProto::ACL mpacl;
@@ -1229,7 +1410,7 @@ void MainWindow::on_qaChannelACL_triggered() {
 
 void MainWindow::on_qaChannelLink_triggered() {
 	Channel *c = ClientUser::get(g.uiSession)->cChannel;
-	Channel *l = pmModel->getChannel(qtvUsers->currentIndex());
+	Channel *l = getContextMenuChannel();
 	if (! l)
 		l = Channel::get(0);
 
@@ -1241,7 +1422,7 @@ void MainWindow::on_qaChannelLink_triggered() {
 
 void MainWindow::on_qaChannelUnlink_triggered() {
 	Channel *c = ClientUser::get(g.uiSession)->cChannel;
-	Channel *l = pmModel->getChannel(qtvUsers->currentIndex());
+	Channel *l = getContextMenuChannel();
 	if (! l)
 		l = Channel::get(0);
 
@@ -1262,7 +1443,7 @@ void MainWindow::on_qaChannelUnlinkAll_triggered() {
 }
 
 void MainWindow::on_qaChannelSendMessage_triggered() {
-	Channel *c = pmModel->getChannel(qtvUsers->currentIndex());
+	Channel *c = getContextMenuChannel();
 
 	if (!c)
 		return;
@@ -1284,15 +1465,16 @@ void MainWindow::on_qaChannelSendMessage_triggered() {
 		g.sh->sendMessage(mptm);
 
 		if (texm->bTreeMessage)
-			g.l->log(Log::TextMessage, tr("To tree %1: %2").arg(c->qsName).arg(texm->message()), tr("Message to tree %1").arg(c->qsName));
+			g.l->log(Log::TextMessage, tr("(Tree) %1: %2").arg(Log::formatChannel(c)).arg(texm->message()), tr("Message to tree %1").arg(c->qsName));
 		else
-			g.l->log(Log::TextMessage, tr("To channel %1: %2").arg(c->qsName).arg(texm->message()), tr("Message to channel %1").arg(c->qsName));
+			g.l->log(Log::TextMessage, tr("%1: %2").arg(Log::formatChannel(c)).arg(texm->message()), tr("Message to channel %1").arg(c->qsName));
 	}
 	delete texm;
 }
 
 void MainWindow::updateMenuPermissions() {
-	Channel *c = g.uiSession ? pmModel->getChannel(qtvUsers->currentIndex()) : NULL;
+	ClientUser *cu = getContextMenuUser();
+	Channel *c = g.uiSession ? (cu ? cu->cChannel : getContextMenuChannel()) : NULL;
 	ChanACL::Permissions p = static_cast<ChanACL::Permissions>(c ? c->uiPermissions : ChanACL::None);
 
 	if (c && ! p) {
@@ -1354,6 +1536,26 @@ void MainWindow::updateMenuPermissions() {
 	qleChat->setEnabled(p & (ChanACL::Write | ChanACL::TextMessage));
 }
 
+void MainWindow::talkingChanged() {
+	if (g.s.bAttenuateOthersOnTalk) {
+		ClientUser *p=ClientUser::get(g.uiSession);
+		switch (p->tsState) {
+			case ClientUser::Talking:
+			case ClientUser::TalkingWhisper:
+			case ClientUser::TalkingWhisperChannel:
+				g.bAttenuateOthers = true;
+				break;
+			case ClientUser::TalkingOff:
+			default:
+				g.bAttenuateOthers = false;
+				break;
+		}
+	}
+
+	if (g.s.bStateInTray)
+		updateTrayIcon();
+}
+
 void MainWindow::on_qaAudioReset_triggered() {
 	AudioInputPtr ai = g.ai;
 	if (ai)
@@ -1431,8 +1633,10 @@ void MainWindow::on_qaAudioUnlink_triggered() {
 
 void MainWindow::on_qaConfigDialog_triggered() {
 	ConfigDialog *dlg= new ConfigDialog(this);
-	if (dlg->exec() == QDialog::Accepted)
+	if (dlg->exec() == QDialog::Accepted) {
 		setupView(false);
+		updateTrayIcon();
+	}
 	delete dlg;
 }
 
@@ -1505,6 +1709,7 @@ void MainWindow::on_gsDeafSelf_down(QVariant v) {
 }
 
 void MainWindow::on_PushToTalk_triggered(bool down, QVariant) {
+	g.iPrevTarget = 0;
 	if (down) {
 		g.uiDoublePush = g.tDoublePush.restart();
 		g.iPushToTalk++;
@@ -1551,7 +1756,7 @@ Channel *MainWindow::mapChannel(int idx) const {
 					c = c->cParent;
 				break;
 			default:
-				c = pmModel->getSubChannel(c, -4 - idx);
+				c = pmModel->getSubChannel(ClientUser::get(g.uiSession)->cChannel, -4 - idx);
 				break;
 		}
 	} else {
@@ -1561,6 +1766,7 @@ Channel *MainWindow::mapChannel(int idx) const {
 }
 
 void MainWindow::updateTarget() {
+	g.iPrevTarget = g.iTarget;
 	if (qsCurrentTargets.isEmpty()) {
 		g.bCenterPosition = false;
 		g.iTarget = 0;
@@ -1677,10 +1883,10 @@ void MainWindow::on_gsWhisper_triggered(bool down, QVariant scdata) {
 
 		g.iPushToTalk++;
 	} else if (g.iPushToTalk) {
+		g.iPushToTalk--;
+
 		qsCurrentTargets.remove(st);
 		updateTarget();
-
-		g.iPushToTalk--;
 	}
 }
 
@@ -1720,9 +1926,37 @@ void MainWindow::serverConnected() {
 		mpus.set_self_deaf(g.s.bDeaf);
 		g.sh->sendMessage(mpus);
 	}
+
+#ifdef Q_OS_WIN
+	TaskList::addToRecentList(g.s.qsLastServer, host, port);
+#endif
+}
+
+static QString getPathToChannel(Channel *c) {
+	QString out;
+
+	if (!c)
+		return out;
+
+	Channel *tmp = c;
+	while (tmp->cParent) {
+		// skip root channel
+		if (tmp->iId == 0)
+			break;
+
+		out.prepend(QString::fromLatin1("/"));
+		out.prepend(tmp->qsName);
+
+		tmp = tmp->cParent;
+	}
+
+	return out;
 }
 
 void MainWindow::serverDisconnected(QAbstractSocket::SocketError err, QString reason) {
+	if (g.uiSession)
+		qsDesiredChannel = getPathToChannel(ClientUser::get(g.uiSession)->cChannel);
+
 	g.uiSession = 0;
 	g.pPermissions = ChanACL::None;
 	qaServerDisconnect->setEnabled(false);
@@ -1780,7 +2014,7 @@ void MainWindow::serverDisconnected(QAbstractSocket::SocketError err, QString re
 
 	if (! g.sh->qlErrors.isEmpty()) {
 		foreach(QSslError e, g.sh->qlErrors)
-			g.l->log(Log::ServerDisconnected, tr("SSL Verification failed: %1").arg(e.errorString()));
+			g.l->log(Log::Warning, tr("SSL Verification failed: %1").arg(e.errorString()));
 		if (! g.sh->qscCert.isEmpty()) {
 			QSslCertificate c = g.sh->qscCert.at(0);
 			QString basereason;
@@ -1817,9 +2051,7 @@ void MainWindow::serverDisconnected(QAbstractSocket::SocketError err, QString re
 				break;
 			}
 		}
-	}
-#ifdef USE_DBUS
-	else if (err == QAbstractSocket::SslHandshakeFailedError) {
+	} else if (err == QAbstractSocket::SslHandshakeFailedError) {
 		if (QMessageBox::warning(this, tr("SSL Version mismatch"), tr("This server is using an older encryption standard. It might be an older 1.1 based Mumble server.<br />Would you like to launch the compatibility client to connect to it?"), QMessageBox::Yes | QMessageBox::No, QMessageBox::Yes) == QMessageBox::Yes) {
 
 			QString host, user, pw;
@@ -1833,30 +2065,22 @@ void MainWindow::serverDisconnected(QAbstractSocket::SocketError err, QString re
 			url.setUserName(user);
 			url.addQueryItem(QLatin1String("version"), QLatin1String("1.1.8"));
 
+#ifdef USE_DBUS
 			QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble11x"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
 
 			QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
 			if (reply.type() == QDBusMessage::ReplyMessage) {
 				this->close();
 				return;
-			} else {
-				QString executable = QApplication::instance()->applicationFilePath();
-				int idx = executable.lastIndexOf(QLatin1String("mumble"));
-				if (idx >= 0) {
-					QStringList args;
-					args << url.toString();
-
-					executable.replace(idx, 6, QLatin1String("mumble11x"));
-					if (QProcess::startDetached(executable, args))
-						this->close();
-					return;
-				}
-			}
-			QMessageBox::critical(this, tr("Failed to launch compatibilty client"), tr("The compatibility client could not be found, or failed to start.<br />Note that the compatibility client is an optional component for most installations, and might not be installed."), QMessageBox::Ok, QMessageBox::Ok);
-		}
-	}
+			} else
 #endif
-	else {
+			{
+				if (launchCompatibilityClient(url))
+					return;
+			}
+			QMessageBox::critical(this, tr("Failed to launch compatibility client"), tr("The compatibility client could not be found, or failed to start.<br />Note that the compatibility client is an optional component for most installations, and might not be installed."), QMessageBox::Ok, QMessageBox::Ok);
+		}
+	} else {
 		bool ok = false;
 		bool matched = false;
 
@@ -1965,7 +2189,7 @@ void MainWindow::qtvUserCurrentChanged(const QModelIndex &, const QModelIndex &)
 	Channel *c = pmModel->getChannel(qtvUsers->currentIndex());
 
 	if (g.uiSession == 0) {
-		qleChat->setDefaultText(tr("Not connected"));
+		qleChat->setDefaultText(tr("Not connected"), true);
 	} else if (p == NULL || p->uiSession == g.uiSession) {
 		// Channel tree target
 		if (c == NULL) // If no channel selected fallback to current one
@@ -1988,6 +2212,10 @@ void MainWindow::customEvent(QEvent *evt) {
 		MessageBoxEvent *mbe=static_cast<MessageBoxEvent *>(evt);
 		g.l->log(Log::Information, mbe->msg);
 		return;
+	} else if (evt->type() == OU_QEVENT) {
+		OpenURLEvent *oue=static_cast<OpenURLEvent *>(evt);
+		openUrl(oue->url);
+		return;
 	} else if (evt->type() != SERVERSEND_EVENT) {
 		return;
 	}
@@ -2012,11 +2240,20 @@ void MainWindow::customEvent(QEvent *evt) {
 #undef MUMBLE_MH_MSG
 }
 
+
 void MainWindow::on_qteLog_anchorClicked(const QUrl &url) {
-	QDesktopServices::openUrl(url);
+	if (!handleSpecialContextMenu(url, QCursor::pos(), true)) {
+		if (url.scheme() != QLatin1String("file")
+		        && url.scheme() != QLatin1String("qrc")
+		        && !url.isRelative())
+			QDesktopServices::openUrl(url);
+	}
 }
 
 void MainWindow::on_qteLog_highlighted(const QUrl &url) {
+	if (url.scheme() == QString::fromLatin1("clientid") || url.scheme() == QString::fromLatin1("channelid"))
+		return;
+
 	if (! url.isValid())
 		QToolTip::hideText();
 	else
@@ -2024,11 +2261,11 @@ void MainWindow::on_qteLog_highlighted(const QUrl &url) {
 }
 
 void MainWindow::on_qdwChat_dockLocationChanged(Qt::DockWidgetArea) {
-	g.s.iWindowLayout = Settings::LayoutCustom;
+	g.s.wlWindowLayout = Settings::LayoutCustom;
 }
 
 void MainWindow::on_qdwLog_dockLocationChanged(Qt::DockWidgetArea) {
-	g.s.iWindowLayout = Settings::LayoutCustom;
+	g.s.wlWindowLayout = Settings::LayoutCustom;
 }
 
 void MainWindow::context_triggered() {
@@ -2077,3 +2314,50 @@ QPair<QByteArray, QImage> MainWindow::openImageFile() {
 
 	return retval;
 }
+
+bool MainWindow::launchCompatibilityClient(const QUrl &url) {
+#ifdef Q_OS_MAC
+	FSRef fref;
+	OSStatus err;
+
+	err = LSFindApplicationForInfo(kLSUnknownCreator, CFSTR("net.sourceforge.mumble.Mumble11x"), CFSTR("Mumble11x.app"), &fref, NULL);
+	if (err == noErr) {
+		CFMutableArrayRef arguments = CFArrayCreateMutable(kCFAllocatorDefault, 2, &kCFTypeArrayCallBacks);
+		if (arguments) {
+			CFArrayAppendValue(arguments, CFSTR("Mumble11x"));
+			QString qsUrlString = url.toString();
+			CFStringRef urlString = CFStringCreateWithCharacters(kCFAllocatorDefault, reinterpret_cast<const UniChar *>(qsUrlString.unicode()), qsUrlString.length());
+			CFArrayAppendValue(arguments, urlString);
+			CFRelease(urlString);
+		}
+
+		LSApplicationParameters parm;
+		memset(&parm, 0, sizeof(LSApplicationParameters));
+		parm.flags = kLSLaunchDefaults;
+		parm.application = &fref;
+		parm.argv = arguments;
+		err = LSOpenApplication(&parm, NULL);
+
+		CFRelease(arguments);
+
+		if (err == noErr) {
+			this->close();
+			return true;
+		}
+	}
+#else
+	QString executable = QApplication::instance()->applicationFilePath();
+	int idx = executable.lastIndexOf(QLatin1String("mumble"));
+	if (idx >= 0) {
+		QStringList args;
+		args << url.toString();
+
+		executable.replace(idx, 6, QLatin1String("mumble11x"));
+		if (QProcess::startDetached(executable, args))
+			this->close();
+		return true;
+	}
+#endif
+
+	return false;
+}
