@@ -1,4 +1,5 @@
 /* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -35,17 +36,24 @@
 
 #define TI_QEVENT (QEvent::User + 938)
 #define MB_QEVENT (QEvent::User + 939)
+#define OU_QEVENT (QEvent::User + 940)
 
 class ACLEditor;
 class BanEditor;
+class UserEdit;
 class ServerHandler;
 class GlobalShortcut;
 class TextToSpeech;
-class PlayerModel;
+class UserModel;
+class Tokens;
+class Channel;
+
+struct ShortcutTarget;
 
 #include "Message.h"
 #include "Usage.h"
 #include "ui_MainWindow.h"
+#include "CustomElements.h"
 
 class MessageBoxEvent : public QEvent {
 	public:
@@ -53,92 +61,116 @@ class MessageBoxEvent : public QEvent {
 		MessageBoxEvent(QString msg);
 };
 
-class LogTitleBar : public QWidget {
-	private:
-		Q_OBJECT
-		Q_DISABLE_COPY(LogTitleBar)
-	protected:
-		QTimer *qtTick;
-		int size;
-		int newsize;
+class OpenURLEvent : public QEvent {
 	public:
-		LogTitleBar();
-		QSize sizeHint() const;
-		QSize minimumSizeHint() const;
-	public slots:
-		void tick();
-	protected:
-		bool eventFilter(QObject *, QEvent *);
+		QUrl url;
+		OpenURLEvent(QUrl url);
 };
 
 class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWindow {
-		friend class PlayerModel;
+		friend class UserModel;
 	private:
 		Q_OBJECT
 		Q_DISABLE_COPY(MainWindow)
 	public:
-		PlayerModel *pmModel;
+		UserModel *pmModel;
 		QSystemTrayIcon *qstiIcon;
 		QMenu *qmTray;
-		QIcon qiIcon, qiIconMute, qiIconDeaf;
+		QIcon qiIcon, qiIconMuteSelf, qiIconMuteServer, qiIconDeafSelf, qiIconDeafServer, qiIconMuteSuppressed;
+		QIcon qiTalkingOn, qiTalkingWhisper, qiTalkingWhisperChannel, qiTalkingOff;
 
 		GlobalShortcut *gsPushTalk, *gsResetAudio, *gsMuteSelf, *gsDeafSelf;
-		GlobalShortcut *gsUnlink, *gsCenterPos, *gsPushMute, *gsMetaChannel, *gsToggleOverlay;
-		GlobalShortcut *gsAltTalk, *gsMinimal, *gsVolumeUp, *gsVolumeDown;
-		LogTitleBar *ltbDockTitle;
+		GlobalShortcut *gsUnlink, *gsPushMute, *gsMetaChannel, *gsToggleOverlay;
+		GlobalShortcut *gsMinimal, *gsVolumeUp, *gsVolumeDown, *gsWhisper;
+		DockTitleBar *dtbLogDockTitle, *dtbChatDockTitle;
 
 		ACLEditor *aclEdit;
 		BanEditor *banEdit;
+		UserEdit *userEdit;
+		Tokens *tokenEdit;
 
-		MessageServerReject::RejectType rtLast;
+		MumbleProto::Reject_RejectType rtLast;
 		QString qsDesiredChannel;
 
+		bool bSuppressAskOnQuit;
+		bool bAutoUnmute;
+
 		void recheckTTS();
 		void msgBox(QString msg);
 		void setOnTop(bool top);
 		void updateTrayIcon();
+		QPair<QByteArray, QImage> openImageFile();
+		static const QString defaultStyleSheet;
+
+#ifdef Q_OS_WIN
+		bool winEvent(MSG *, long *);
+		unsigned int uiNewHardware;
+#endif
 	protected:
 		Usage uUsage;
 		QTimer *qtReconnect;
 
 		QList<QAction *> qlServerActions;
 		QList<QAction *> qlChannelActions;
-		QList<QAction *> qlPlayerActions;
+		QList<QAction *> qlUserActions;
+
+		QSet<ShortcutTarget> qsCurrentTargets;
+		QHash<QList<ShortcutTarget>, int> qmTargets;
+		QMap<int, int> qmTargetUse;
+		Channel *mapChannel(int idx) const;
+		int iTargetCounter;
 
 		void createActions();
 		void setupGui();
 		void customEvent(QEvent *evt);
 		void findDesiredChannel();
 		void setupView(bool toggle_minimize = true);
+		void setupIconMenu(bool top = false);
+		bool launchCompatibilityClient(const QUrl &url);
 		bool bNoHide;
 		virtual void closeEvent(QCloseEvent *e);
 		virtual void hideEvent(QHideEvent *e);
+
+	private:
+		qint32 cmUid;
+		qint32 cmCid;
+		bool handleSpecialContextMenu(const QUrl &url, const QPoint &_pos, bool focus = false);
+		Channel* getContextMenuChannel();
+		ClientUser* getContextMenuUser();
+
 	public slots:
-		void on_qtvPlayers_customContextMenuRequested(const QPoint &pos);
-		void on_qtvPlayers_doubleClicked(const QModelIndex &idx);
 		void on_qmServer_aboutToShow();
 		void on_qaServerConnect_triggered();
 		void on_qaServerDisconnect_triggered();
 		void on_qaServerBanList_triggered();
+		void on_qaServerUserList_triggered();
 		void on_qaServerInformation_triggered();
-		void on_qmPlayer_aboutToShow();
-		void on_qaPlayerKick_triggered();
-		void on_qaPlayerBan_triggered();
-		void on_qaPlayerMute_triggered();
-		void on_qaPlayerDeaf_triggered();
-		void on_qaPlayerLocalMute_triggered();
-		void on_qaPlayerTextMessage_triggered();
+		void on_qaServerTexture_triggered();
+		void on_qaServerTextureRemove_triggered();
+		void on_qaServerTokens_triggered();
+		void on_qmUser_aboutToShow();
+		void on_qaUserComment_triggered();
+		void on_qaUserCommentReset_triggered();
+		void on_qaUserCommentView_triggered();
+		void on_qaUserKick_triggered();
+		void on_qaUserBan_triggered();
+		void on_qaUserMute_triggered();
+		void on_qaUserDeaf_triggered();
+		void on_qaUserLocalMute_triggered();
+		void on_qaUserTextMessage_triggered();
+		void on_qaUserRegister_triggered();
+		void on_qaUserFriendAdd_triggered();
+		void on_qaUserFriendRemove_triggered();
+		void on_qaUserFriendUpdate_triggered();
 		void on_qmChannel_aboutToShow();
+		void on_qaChannelJoin_triggered();
 		void on_qaChannelAdd_triggered();
 		void on_qaChannelRemove_triggered();
 		void on_qaChannelACL_triggered();
-		void on_qaChannelRename_triggered();
-		void on_qaChannelDescUpdate_triggered();
 		void on_qaChannelLink_triggered();
 		void on_qaChannelUnlink_triggered();
 		void on_qaChannelUnlinkAll_triggered();
 		void on_qaChannelSendMessage_triggered();
-		void on_qaChannelSendTreeMessage_triggered();
 		void on_qaAudioReset_triggered();
 		void on_qaAudioMute_triggered();
 		void on_qaAudioDeaf_triggered();
@@ -149,6 +181,7 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_qaConfigHideFrame_triggered();
 		void on_qmConfig_aboutToShow();
 		void on_qaConfigMinimal_triggered();
+		void on_qaConfigCert_triggered();
 		void on_qaAudioWizard_triggered();
 		void on_qaHelpWhatsThis_triggered();
 		void on_qaHelpAbout_triggered();
@@ -156,59 +189,40 @@ class MainWindow : public QMainWindow, public MessageHandler, public Ui::MainWin
 		void on_qaHelpAboutQt_triggered();
 		void on_qaHelpVersionCheck_triggered();
 		void on_qaQuit_triggered();
+		void on_qleChat_returnPressed();
+		void on_qleChat_tabPressed();
+		void on_qleChat_ctrlSpacePressed();
 		void on_qteLog_customContextMenuRequested(const QPoint &pos);
 		void on_qteLog_anchorClicked(const QUrl &);
 		void on_qteLog_highlighted(const QUrl & link);
-		void on_PushToTalk_triggered(bool);
-		void on_PushToMute_triggered(bool);
-		void on_AltPushToTalk_triggered(bool);
-		void on_CenterPos_triggered(bool);
-		void on_VolumeUp_triggered(bool);
-		void on_VolumeDown_triggered(bool);
+		void on_qdwChat_dockLocationChanged(Qt::DockWidgetArea);
+		void on_qdwLog_dockLocationChanged(Qt::DockWidgetArea);
+		void on_PushToTalk_triggered(bool, QVariant);
+		void on_PushToMute_triggered(bool, QVariant);
+		void on_VolumeUp_triggered(bool, QVariant);
+		void on_VolumeDown_triggered(bool, QVariant);
+		void on_gsMuteSelf_down(QVariant);
+		void on_gsDeafSelf_down(QVariant);
+		void on_gsWhisper_triggered(bool, QVariant);
 		void on_Reconnect_timeout();
 		void on_Icon_activated(QSystemTrayIcon::ActivationReason);
+		void qtvUserCurrentChanged(const QModelIndex &, const QModelIndex &);
 		void serverConnected();
-		void serverDisconnected(QString reason);
-		void pushLink(bool down);
+		void serverDisconnected(QAbstractSocket::SocketError, QString reason);
 		void viewCertificate(bool);
 		void openUrl(const QUrl &url);
 		void context_triggered();
+		void updateTarget();
+		void updateMenuPermissions();
+		void talkingChanged();
 	public:
 		MainWindow(QWidget *parent);
 		~MainWindow();
 
 		// From msgHandler. Implementation in Messages.cpp
-		virtual void msgSpeex(Connection *, MessageSpeex *);
-		virtual void msgServerAuthenticate(Connection *, MessageServerAuthenticate *);
-		virtual void msgPing(Connection *, MessagePing *);
-		virtual void msgPingStats(Connection *, MessagePingStats *);
-		virtual void msgServerReject(Connection *, MessageServerReject *);
-		virtual void msgServerSync(Connection *, MessageServerSync *);
-		virtual void msgServerJoin(Connection *, MessageServerJoin *);
-		virtual void msgServerLeave(Connection *, MessageServerLeave *);
-		virtual void msgPlayerMute(Connection *, MessagePlayerMute *);
-		virtual void msgPlayerDeaf(Connection *, MessagePlayerDeaf *);
-		virtual void msgPlayerSelfMuteDeaf(Connection *, MessagePlayerSelfMuteDeaf *);
-		virtual void msgPlayerKick(Connection *, MessagePlayerKick *);
-		virtual void msgPlayerBan(Connection *, MessagePlayerBan *);
-		virtual void msgPlayerMove(Connection *, MessagePlayerMove *);
-		virtual void msgPlayerRename(Connection *, MessagePlayerRename *);
-		virtual void msgChannelAdd(Connection *, MessageChannelAdd *);
-		virtual void msgChannelRemove(Connection *, MessageChannelRemove *);
-		virtual void msgChannelMove(Connection *, MessageChannelMove *);
-		virtual void msgChannelLink(Connection *, MessageChannelLink *);
-		virtual void msgChannelRename(Connection *, MessageChannelRename *);
-		virtual void msgChannelDescUpdate(Connection *, MessageChannelDescUpdate *);
-		virtual void msgServerBanList(Connection *, MessageServerBanList *);
-		virtual void msgTextMessage(Connection *, MessageTextMessage *);
-		virtual void msgPermissionDenied(Connection *, MessagePermissionDenied *);
-		virtual void msgEditACL(Connection *, MessageEditACL *);
-		virtual void msgQueryUsers(Connection *, MessageQueryUsers *);
-		virtual void msgTexture(Connection *, MessageTexture *);
-		virtual void msgCryptSetup(Connection *, MessageCryptSetup *);
-		virtual void msgCryptSync(Connection *, MessageCryptSync *);
-		virtual void msgContextAddAction(Connection *, MessageContextAddAction *);
-		virtual void msgContextAction(Connection *, MessageContextAction *);
+#define MUMBLE_MH_MSG(x) void msg##x(const MumbleProto:: x &);
+		MUMBLE_MH_ALL
+#undef MUMBLE_MH_MSG
 };
 
 #else
