@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -121,6 +121,10 @@ void MainWindow::msgServerSync(const MumbleProto::ServerSync &msg) {
 		g.s.qlShortcuts << sc;
 		GlobalShortcutEngine::engine->bNeedRemap = true;
 	}
+
+	ClientUser *p=ClientUser::get(g.uiSession);
+	connect(p, SIGNAL(talkingChanged()), this, SLOT(talkingChanged()));
+	updateTrayIcon();
 }
 
 void MainWindow::msgPermissionDenied(const MumbleProto::PermissionDenied &msg) {
@@ -129,11 +133,13 @@ void MainWindow::msgPermissionDenied(const MumbleProto::PermissionDenied &msg) {
 				VICTIM_INIT;
 				SELF_INIT;
 				Channel *c = Channel::get(msg.channel_id());
+				if (! c)
+					return;
 				QString pname = ChanACL::permName(static_cast<ChanACL::Permissions>(msg.permission()));
 				if (pDst == pSelf)
-					g.l->log(Log::PermissionDenied, tr("You were denied %1 privileges in %2.").arg(pname).arg(c->qsName));
+					g.l->log(Log::PermissionDenied, tr("You were denied %1 privileges in %2.").arg(Log::msgColor(pname, Log::Privilege)).arg(Log::formatChannel(c)));
 				else
-					g.l->log(Log::PermissionDenied, tr("%3 was denied %1 privileges in %2.").arg(pname).arg(c->qsName).arg(pDst->qsName));
+					g.l->log(Log::PermissionDenied, tr("%3 was denied %1 privileges in %2.").arg(Log::msgColor(pname, Log::Privilege)).arg(Log::formatChannel(c)).arg(Log::formatClientUser(pDst, Log::Target)));
 			}
 			break;
 		case MumbleProto::PermissionDenied_DenyType_SuperUser: {
@@ -174,14 +180,25 @@ void MainWindow::msgPermissionDenied(const MumbleProto::PermissionDenied &msg) {
 				if (pDst == pSelf)
 					g.l->log(Log::PermissionDenied, tr("You need a certificate to perform this operation."));
 				else
-					g.l->log(Log::PermissionDenied, tr("%1 does not have a certificate.").arg(pDst->qsName));
+					g.l->log(Log::PermissionDenied, tr("%1 does not have a certificate.").arg(Log::formatClientUser(pDst, Log::Target)));
 			}
 			break;
-		default:
-			if (msg.has_reason()) {
-				g.l->log(Log::PermissionDenied, tr("Denied: %1.").arg(u8(msg.reason())));
-			} else {
-				g.l->log(Log::PermissionDenied, tr("Permission denied."));
+		case MumbleProto::PermissionDenied_DenyType_UserName: {
+				if (msg.has_name())
+					g.l->log(Log::PermissionDenied, tr("Invalid username: %1.").arg(u8(msg.name())));
+				else
+					g.l->log(Log::PermissionDenied, tr("Invalid username."));
+			}
+			break;
+		case MumbleProto::PermissionDenied_DenyType_ChannelFull: {
+				g.l->log(Log::PermissionDenied, tr("Channel is full."));
+			}
+			break;
+		default: {
+				if (msg.has_reason())
+					g.l->log(Log::PermissionDenied, tr("Denied: %1.").arg(u8(msg.reason())));
+				else
+					g.l->log(Log::PermissionDenied, tr("Permission denied."));
 			}
 			break;
 	}
@@ -194,13 +211,14 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 	ACTOR_INIT;
 	SELF_INIT;
 	ClientUser *pDst=ClientUser::get(msg.session());
+	bool bNewUser = false;
 
 	if (! pDst) {
 		if (msg.has_name()) {
 			pDst = pmModel->addUser(msg.session(), u8(msg.name()));
-			g.l->log(Log::UserJoin, tr("Joined server: %1.").arg(pDst->qsName));
 			if (! msg.has_texture())
 				g.o->verifyTexture(pDst);
+			bNewUser = true;
 		} else {
 			return;
 		}
@@ -218,6 +236,9 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 			pDst->setLocalMute(true);
 	}
 
+	if (bNewUser)
+		g.l->log(Log::UserJoin, tr("%1 connected.").arg(Log::formatClientUser(pDst, Log::Source)));
+
 	if (msg.has_self_deaf() || msg.has_self_mute()) {
 		if (msg.has_self_mute())
 			pDst->setSelfMute(msg.self_mute());
@@ -227,11 +248,11 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 		if (pSelf && pDst != pSelf && (pDst->cChannel == pSelf->cChannel)) {
 			QString name = pDst->qsName;
 			if (pDst->bSelfMute && pDst->bSelfDeaf)
-				g.l->log(Log::OtherSelfMute, tr("%1 is now muted and deafened.").arg(name));
+				g.l->log(Log::OtherSelfMute, tr("%1 is now muted and deafened.").arg(Log::formatClientUser(pDst, Log::Target)));
 			else if (pDst->bSelfMute)
-				g.l->log(Log::OtherSelfMute, tr("%1 is now muted.").arg(name));
+				g.l->log(Log::OtherSelfMute, tr("%1 is now muted.").arg(Log::formatClientUser(pDst, Log::Target)));
 			else
-				g.l->log(Log::OtherSelfMute, tr("%1 is now unmuted.").arg(name));
+				g.l->log(Log::OtherSelfMute, tr("%1 is now unmuted.").arg(Log::formatClientUser(pDst, Log::Target)));
 		}
 	}
 
@@ -244,85 +265,82 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 			pDst->setSuppress(msg.suppress());
 
 		if (pSelf && ((pDst->cChannel == pSelf->cChannel) || (pSrc == pSelf))) {
-			QString vic = pDst->qsName;
-			QString admin = pSrc ? pSrc->qsName : tr("the server");
-
 			if (pDst == pSelf) {
 				if (msg.has_mute() && msg.has_deaf() && pDst->bMute && pDst->bDeaf) {
-					g.l->log(Log::YouMuted, tr("You were muted and deafened by %1.").arg(admin));
+					g.l->log(Log::YouMuted, tr("You were muted and deafened by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
 				} else if (msg.has_mute() && msg.has_deaf() && !pDst->bMute && !pDst->bDeaf) {
-					g.l->log(Log::YouMuted, tr("You were unmuted and undeafened by %1.").arg(admin));
+					g.l->log(Log::YouMuted, tr("You were unmuted and undeafened by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
 				} else {
 					if (msg.has_mute()) {
 						if (pDst->bMute)
-							g.l->log(Log::YouMuted, tr("You were muted by %1.").arg(admin));
+							g.l->log(Log::YouMuted, tr("You were muted by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
 						else
-							g.l->log(Log::YouMuted, tr("You were unmuted by %1.").arg(admin));
+							g.l->log(Log::YouMuted, tr("You were unmuted by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
 					}
 
 					if (msg.has_deaf()) {
 						if (!pDst->bDeaf)
-							g.l->log(Log::YouMuted, tr("You were undeafened by %1.").arg(admin));
+							g.l->log(Log::YouMuted, tr("You were undeafened by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
 					}
 				}
 
 				if (msg.has_suppress()) {
 					if (pDst->bSuppress)
-						g.l->log(Log::YouMuted, tr("You were suppressed by %1.").arg(admin));
+						g.l->log(Log::YouMuted, tr("You were suppressed by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
 					else
-						g.l->log(Log::YouMuted, tr("You were unsuppressed by %1.").arg(admin));
+						g.l->log(Log::YouMuted, tr("You were unsuppressed by %1.").arg(Log::formatClientUser(pSrc, Log::Source)));
 				}
 
 				updateTrayIcon();
 			} else if (pSrc == pSelf) {
 				if (msg.has_mute() && msg.has_deaf() && pDst->bMute && pDst->bDeaf) {
-					g.l->log(Log::YouMutedOther, tr("You muted and deafened %1.").arg(vic));
+					g.l->log(Log::YouMutedOther, tr("You muted and deafened %1.").arg(Log::formatClientUser(pDst, Log::Target)));
 				} else if (msg.has_mute() && msg.has_deaf() && !pDst->bMute && !pDst->bDeaf) {
-					g.l->log(Log::YouMutedOther, tr("You unmuted and undeafened %1.").arg(vic));
+					g.l->log(Log::YouMutedOther, tr("You unmuted and undeafened %1.").arg(Log::formatClientUser(pDst, Log::Target)));
 				} else {
 					if (msg.has_mute()) {
 						if (pDst->bMute)
-							g.l->log(Log::YouMutedOther, tr("You muted %1.").arg(vic));
+							g.l->log(Log::YouMutedOther, tr("You muted %1.").arg(Log::formatClientUser(pDst, Log::Target)));
 						else
-							g.l->log(Log::YouMutedOther, tr("You unmuted %1.").arg(vic));
+							g.l->log(Log::YouMutedOther, tr("You unmuted %1.").arg(Log::formatClientUser(pDst, Log::Target)));
 					}
 
 					if (msg.has_deaf()) {
 						if (!pDst->bDeaf)
-							g.l->log(Log::YouMutedOther, tr("You undeafened %1.").arg(vic));
+							g.l->log(Log::YouMutedOther, tr("You undeafened %1.").arg(Log::formatClientUser(pDst, Log::Target)));
 					}
 				}
 
 				if (msg.has_suppress()) {
 					if (pDst->bSuppress)
-						g.l->log(Log::YouMutedOther, tr("You suppressed %1.").arg(vic));
+						g.l->log(Log::YouMutedOther, tr("You suppressed %1.").arg(Log::formatClientUser(pDst, Log::Target)));
 					else
-						g.l->log(Log::YouMutedOther, tr("You unsuppressed %1.").arg(vic));
+						g.l->log(Log::YouMutedOther, tr("You unsuppressed %1.").arg(Log::formatClientUser(pDst, Log::Target)));
 				}
 			} else {
 				if (msg.has_mute() && msg.has_deaf() && pDst->bMute && pDst->bDeaf) {
-					g.l->log(Log::OtherMutedOther, tr("%1 muted and deafened by %2.").arg(vic, admin));
+					g.l->log(Log::OtherMutedOther, tr("%1 muted and deafened by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
 				} else if (msg.has_mute() && msg.has_deaf() && !pDst->bMute && !pDst->bDeaf) {
-					g.l->log(Log::OtherMutedOther, tr("%1 unmuted and undeafened by %2.").arg(vic, admin));
+					g.l->log(Log::OtherMutedOther, tr("%1 unmuted and undeafened by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
 				} else {
 					if (msg.has_mute()) {
 						if (pDst->bMute)
-							g.l->log(Log::OtherMutedOther, tr("%1 muted by %2.").arg(vic, admin));
+							g.l->log(Log::OtherMutedOther, tr("%1 muted by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
 						else
-							g.l->log(Log::OtherMutedOther, tr("%1 unmuted by %2.").arg(vic, admin));
+							g.l->log(Log::OtherMutedOther, tr("%1 unmuted by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
 					}
 
 					if (msg.has_deaf()) {
 						if (!pDst->bDeaf)
-							g.l->log(Log::OtherMutedOther, tr("%1 undeafened by %2.").arg(vic, admin));
+							g.l->log(Log::OtherMutedOther, tr("%1 undeafened by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
 					}
 				}
 
 				if (msg.has_suppress()) {
 					if (pDst->bSuppress)
-						g.l->log(Log::OtherMutedOther, tr("%1 suppressed by %2.").arg(vic, admin));
+						g.l->log(Log::OtherMutedOther, tr("%1 suppressed by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
 					else
-						g.l->log(Log::OtherMutedOther, tr("%1 unsuppressed by %2.").arg(vic, admin));
+						g.l->log(Log::OtherMutedOther, tr("%1 unsuppressed by %2.").arg(Log::formatClientUser(pDst, Log::Target), Log::formatClientUser(pSrc, Log::Source)));
 				}
 			}
 		}
@@ -339,17 +357,14 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 		if (c != old) {
 			bool log = pSelf && !((pDst == pSelf) && (pSrc == pSelf));
 
-			QString pname = pDst->qsName;
-			QString admin = pSrc ? pSrc->qsName : tr("the server");
-
 			if (log) {
 				if (pDst == pSelf) {
-					g.l->log(Log::ChannelJoin, tr("You were moved to %1 by %2.").arg(c->qsName).arg(admin));
+					g.l->log(Log::ChannelJoin, tr("You were moved to %1 by %2.").arg(Log::formatChannel(c)).arg(Log::formatClientUser(pSrc, Log::Source)));
 				} else if (pDst->cChannel == ClientUser::get(g.uiSession)->cChannel) {
 					if (pDst == pSrc)
-						g.l->log(Log::ChannelLeave, tr("%1 moved to %2.").arg(pname).arg(c->qsName));
+						g.l->log(Log::ChannelLeave, tr("%1 moved to %2.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(c)));
 					else
-						g.l->log(Log::ChannelLeave, tr("%1 moved to %2 by %3.").arg(pname).arg(c->qsName).arg(admin));
+						g.l->log(Log::ChannelLeave, tr("%1 moved to %2 by %3.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(c)).arg(Log::formatClientUser(pSrc, Log::Source)));
 				}
 			}
 
@@ -357,9 +372,9 @@ void MainWindow::msgUserState(const MumbleProto::UserState &msg) {
 
 			if (log && (pDst != pSelf) && (pDst->cChannel == pSelf->cChannel)) {
 				if (pDst == pSrc)
-					g.l->log(Log::ChannelJoin, tr("%1 entered channel.").arg(pname));
+					g.l->log(Log::ChannelJoin, tr("%1 entered channel.").arg(Log::formatClientUser(pDst, Log::Target)));
 				else
-					g.l->log(Log::ChannelJoin, tr("%1 moved in from %2 by %3.").arg(pname).arg(old->qsName).arg(admin));
+					g.l->log(Log::ChannelJoin, tr("%1 moved in from %2 by %3.").arg(Log::formatClientUser(pDst, Log::Target)).arg(Log::formatChannel(old)).arg(Log::formatClientUser(pSrc, Log::Source)));
 			}
 		}
 	}
@@ -380,21 +395,20 @@ void MainWindow::msgUserRemove(const MumbleProto::UserRemove &msg) {
 	ACTOR_INIT;
 	SELF_INIT;
 
-	QString admin = pSrc ? pSrc->qsName : tr("the server");
 	QString reason = u8(msg.reason());
 
 	if (pDst == pSelf) {
 		if (msg.ban())
-			g.l->log(Log::YouKicked, tr("You were kicked and banned from the server by %1: %2.").arg(admin).arg(reason));
+			g.l->log(Log::YouKicked, tr("You were kicked and banned from the server by %1: %2.").arg(Log::formatClientUser(pSrc, Log::Source)).arg(reason));
 		else
-			g.l->log(Log::YouKicked, tr("You were kicked from the server by %1: %2.").arg(admin).arg(reason));
+			g.l->log(Log::YouKicked, tr("You were kicked from the server by %1: %2.").arg(Log::formatClientUser(pSrc, Log::Source)).arg(reason));
 	} else if (pSrc) {
 		if (msg.ban())
-			g.l->log((pSrc == pSelf) ? Log::YouKicked : Log::UserKicked, tr("%3 was kicked and banned from the server by %1: %2.").arg(admin).arg(reason).arg(pDst->qsName));
+			g.l->log((pSrc == pSelf) ? Log::YouKicked : Log::UserKicked, tr("%3 was kicked and banned from the server by %1: %2.").arg(Log::formatClientUser(pSrc, Log::Source)).arg(reason).arg(Log::formatClientUser(pDst, Log::Target)));
 		else
-			g.l->log((pSrc == pSelf) ? Log::YouKicked : Log::UserKicked, tr("%3 was kicked from the server by %1: %2.").arg(admin).arg(reason).arg(pDst->qsName));
+			g.l->log((pSrc == pSelf) ? Log::YouKicked : Log::UserKicked, tr("%3 was kicked from the server by %1: %2.").arg(Log::formatClientUser(pSrc, Log::Source)).arg(reason).arg(Log::formatClientUser(pDst, Log::Target)));
 	} else {
-		g.l->log(Log::UserLeave, tr("Left server: %1.").arg(pDst->qsName));
+		g.l->log(Log::UserLeave, tr("%1 disconnected.").arg(Log::formatClientUser(pDst, Log::Source)));
 	}
 	if (pDst != pSelf)
 		pmModel->removeUser(pDst);
@@ -478,8 +492,16 @@ void MainWindow::msgChannelRemove(const MumbleProto::ChannelRemove &msg) {
 
 void MainWindow::msgTextMessage(const MumbleProto::TextMessage &msg) {
 	ACTOR_INIT;
-	const QString &name = pSrc ? pSrc->qsName : tr("the server", "message from");
-	g.l->log(Log::TextMessage, tr("From %1: %2").arg(name).arg(u8(msg.message())),
+	QString target;
+	const QString &name = pSrc ? Log::formatClientUser(pSrc, Log::Source) : tr("Server", "message from");
+
+	if (msg.tree_id_size() > 0) {
+		target += tr("(Tree) ");
+	} else if (msg.channel_id_size() > 0) {
+		target += tr("(Channel) ");
+	}
+
+	g.l->log(Log::TextMessage, tr("%2%1: %3").arg(name).arg(target).arg(u8(msg.message())),
 	         tr("Message from %1").arg(name));
 }
 
