@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -29,7 +29,7 @@
 */
 
 #include "Overlay.h"
-#include "Player.h"
+#include "User.h"
 #include "Channel.h"
 #include "Global.h"
 #include "Message.h"
@@ -60,25 +60,33 @@ void OverlayConfig::load(const Settings &r) {
 	loadCheckBox(qcbTop, r.bOverlayTop);
 	loadCheckBox(qcbBottom, r.bOverlayBottom);
 
-	loadSlider(qsX, static_cast<int>(lroundf(r.fOverlayX * 100.0f)));
-	loadSlider(qsY, 100 - static_cast<int>(lroundf(r.fOverlayY * 100.0f)));
+	loadSlider(qsX, iroundf(r.fOverlayX * 100.0f));
+	loadSlider(qsY, 100 - iroundf(r.fOverlayY * 100.0f));
 	qfFont = r.qfOverlayFont;
-	qcPlayer = r.qcOverlayPlayer;
+	qcUser = r.qcOverlayUser;
 	qcTalking= r.qcOverlayTalking;
-	qcAltTalking= r.qcOverlayAltTalking;
+	qcWhisper= r.qcOverlayWhisper;
 	qcChannel = r.qcOverlayChannel;
 	qcChannelTalking = r.qcOverlayChannelTalking;
 	qlCurrentFont->setText(qfFont.family());
-	loadSlider(qsMaxHeight, static_cast<int>(lroundf(r.fOverlayHeight * 1000.0f)));
-	setColorLabel(qlPlayer, qcPlayer);
+	loadSlider(qsMaxHeight, iroundf(r.fOverlayHeight * 1000.0f));
+	setColorLabel(qlUser, qcUser);
 	setColorLabel(qlTalking, qcTalking);
-	setColorLabel(qlAltTalking, qcAltTalking);
+	setColorLabel(qlWhisper, qcWhisper);
 	setColorLabel(qlChannel, qcChannel);
 	setColorLabel(qlChannelTalking, qcChannelTalking);
 }
 
-bool OverlayConfig::expert(bool) {
-	return false;
+bool OverlayConfig::expert(bool b) {
+	qcbUserTextures->setVisible(b);
+
+	qcbLeft->setVisible(b);
+	qcbRight->setVisible(b);
+	qcbTop->setVisible(b);
+	qcbBottom->setVisible(b);
+
+	qgbFont->setVisible(b);
+	return true;
 }
 
 void OverlayConfig::setColorLabel(QLabel *label, QColor col) {
@@ -104,50 +112,30 @@ void OverlayConfig::on_qsMaxHeight_valueChanged(int v) {
 	qfFont.setPointSize(v);
 }
 
+#if QT_VERSION >= 0x040500
+#define UPDCOLOR(var, text) { QColor col = QColorDialog::getColor(qc##var, this, text, QColorDialog::ShowAlphaChannel); if (col.isValid()) { qc##var = col; setColorLabel(ql##var, qc##var); } }
+#else
+#define UPDCOLOR(var, text) { QColor col = QColorDialog::getColor(qc##var, this); if (col.isValid()) { qc##var = col; setColorLabel(ql##var, qc##var); } }
+#endif
 
-void OverlayConfig::on_qpbPlayer_clicked() {
-	bool ok;
-	QRgb rgb=QColorDialog::getRgba(qcPlayer.rgba(), &ok, this);
-	if (ok) {
-		qcPlayer = QColor::fromRgba(rgb);
-		setColorLabel(qlPlayer, qcPlayer);
-	}
+void OverlayConfig::on_qpbUser_clicked() {
+	UPDCOLOR(User, tr("Color for users"));
 }
 
 void OverlayConfig::on_qpbTalking_clicked() {
-	bool ok;
-	QRgb rgb=QColorDialog::getRgba(qcTalking.rgba(), &ok, this);
-	if (ok) {
-		qcTalking = QColor::fromRgba(rgb);
-		setColorLabel(qlTalking, qcTalking);
-	}
+	UPDCOLOR(Talking, tr("Color for talking users"));
 }
 
-void OverlayConfig::on_qpbAltTalking_clicked() {
-	bool ok;
-	QRgb rgb=QColorDialog::getRgba(qcAltTalking.rgba(), &ok, this);
-	if (ok) {
-		qcAltTalking = QColor::fromRgba(rgb);
-		setColorLabel(qlAltTalking, qcAltTalking);
-	}
+void OverlayConfig::on_qpbWhisper_clicked() {
+	UPDCOLOR(Whisper, tr("Color for whispering users"));
 }
 
 void OverlayConfig::on_qpbChannel_clicked() {
-	bool ok;
-	QRgb rgb=QColorDialog::getRgba(qcChannel.rgba(), &ok, this);
-	if (ok) {
-		qcChannel = QColor::fromRgba(rgb);
-		setColorLabel(qlChannel, qcChannel);
-	}
+	UPDCOLOR(Channel, tr("Color for channels"));
 }
 
 void OverlayConfig::on_qpbChannelTalking_clicked() {
-	bool ok;
-	QRgb rgb=QColorDialog::getRgba(qcChannelTalking.rgba(), &ok, this);
-	if (ok) {
-		qcChannelTalking = QColor::fromRgba(rgb);
-		setColorLabel(qlChannelTalking, qcChannelTalking);
-	}
+	UPDCOLOR(ChannelTalking, tr("Color for active channels"));
 }
 
 QString OverlayConfig::title() const {
@@ -171,9 +159,9 @@ void OverlayConfig::save() const {
 	s.bOverlayBottom = qcbBottom->isChecked();
 	s.qfOverlayFont = qfFont;
 	s.fOverlayHeight = static_cast<float>(qsMaxHeight->value()) / 1000.0f;
-	s.qcOverlayPlayer = qcPlayer;
+	s.qcOverlayUser = qcUser;
 	s.qcOverlayTalking = qcTalking;
-	s.qcOverlayAltTalking = qcAltTalking;
+	s.qcOverlayWhisper = qcWhisper;
 	s.qcOverlayChannel = qcChannel;
 	s.qcOverlayChannelTalking = qcChannelTalking;
 }
@@ -183,7 +171,17 @@ void OverlayConfig::accept() const {
 	g.o->setActive(s.bOverlayEnable);
 }
 
+bool Overlay::TextLine::operator <(const Overlay::TextLine &other) const {
+	if (iPriority < other.iPriority)
+		return true;
+	else if (iPriority > other.iPriority)
+		return false;
+
+	return QString::localeAwareCompare(qsText, other.qsText) < 0;
+}
+
 Overlay::Overlay() : QObject() {
+	d = NULL;
 	qlOverlay = new QLibrary(this);
 
 #ifdef Q_OS_WIN
@@ -193,8 +191,7 @@ Overlay::Overlay() : QObject() {
 	if (! qlOverlay->load()) {
 		QMessageBox::critical(NULL, tr("Mumble"), tr("Failed to load overlay library. This means either that:\n"
 		                      "- the library (mumble_ol.dll) wasn't found in the directory you ran Mumble from\n"
-		                      "- you're on an OS earlier than WinXP SP2\n"
-		                      "- you do not have the June 2007 updated version of DX9.0c"), QMessageBox::Ok, QMessageBox::NoButton);
+		                      "- you're on an OS earlier than WinXP SP2"), QMessageBox::Ok, QMessageBox::NoButton);
 		qWarning("Overlay failure");
 	} else {
 		sm.resolve(qlOverlay);
@@ -203,15 +200,15 @@ Overlay::Overlay() : QObject() {
 	sm.resolve(qlOverlay);
 #endif
 
+	if (! sm.sm) {
+		QMessageBox::warning(NULL, tr("Mumble"), tr("Failed to initialize overlay memory. This usually means that the shared memory is "
+		                     "locked by the OS, and you need to reboot to release it."), QMessageBox::Ok, QMessageBox::NoButton);
+	} else {
 #ifndef QT_NO_DEBUG
-	if (sm.sm)
 		sm.sm->bDebug = true;
 #else
-	if (sm.sm)
 		sm.sm->bDebug = false;
 #endif
-
-	if (sm.sm) {
 		sm.sm->version[0] = OVERLAY_VERSION_MAJ;
 		sm.sm->version[1] = OVERLAY_VERSION_MIN;
 		sm.sm->version[2] = OVERLAY_VERSION_PATCH;
@@ -219,10 +216,10 @@ Overlay::Overlay() : QObject() {
 	}
 
 	QImage img;
-	img.load(QLatin1String("skin:muted_overlay.png"));
+	img = QIcon(QLatin1String("skin:muted_self.svg")).pixmap(60,60).toImage();
 	qbaMuted = QByteArray(reinterpret_cast<const char *>(img.bits()), img.numBytes());
 
-	img.load(QLatin1String("skin:deafened_overlay.png"));
+	img = QIcon(QLatin1String("skin:deafened_self.svg")).pixmap(60,60).toImage();
 	qbaDeafened = QByteArray(reinterpret_cast<const char *>(img.bits()), img.numBytes());
 
 	qtTimer=new QTimer(this);
@@ -237,6 +234,8 @@ Overlay::Overlay() : QObject() {
 
 Overlay::~Overlay() {
 	setActive(false);
+	if (d)
+		delete d;
 	qlOverlay->unload();
 }
 
@@ -263,6 +262,9 @@ void Overlay::toggleShow() {
 	}
 	g.s.osOverlay = ns;
 
+	if (! sm.sm)
+		return;
+
 	if (sm.tryLock()) {
 		sm.sm->bShow = (g.s.osOverlay != Settings::Nothing);
 		sm.unlock();
@@ -293,36 +295,35 @@ void Overlay::forceSettings() {
 	updateOverlay();
 }
 
-void Overlay::textureResponse(int id, const QByteArray &texture) {
-	QString s = qhQueried.value(id);
-	if (s.isEmpty())
+void Overlay::verifyTexture(ClientUser *cp) {
+	qsForce.insert(cp->uiSession);
+	if (! cp->qbaTexture.isEmpty() && (cp->qbaTexture.size() != TEXTURE_SIZE)) {
+		cp->qbaTexture = qUncompress(cp->qbaTexture);
+		if (cp->qbaTexture.size() != TEXTURE_SIZE)
+			cp->qbaTexture = QByteArray();
+	}
+	if (cp->qbaTexture.isEmpty()) {
+		cp->iTextureWidth = 0;
 		return;
+	}
 
-	QByteArray t = qUncompress(texture);
-
-	if (t.size() != TEXTURE_SIZE)
-		return;
-
-	const unsigned char *data = reinterpret_cast<const unsigned char *>(t.constData());
-
+	unsigned char *data = reinterpret_cast<unsigned char *>(cp->qbaTexture.data());
 	int width = 0;
-	for (int y=0;y<TEXT_HEIGHT;y++) {
-		for (int x=0;x<TEXT_WIDTH; x++) {
+	for (int y=0;y<TEXT_HEIGHT;++y) {
+		for (int x=0;x<TEXT_WIDTH; ++x) {
 			if ((x > width) && (data[(y*TEXT_WIDTH+x)*4] != 0x00))
 				width = x;
 		}
 	}
-	qhUserTextures[id] = UserTexture(width, t);
-	qsForce.insert(id);
-	setTexts(qlCurrentTexts);
+	cp->iTextureWidth = width;
 }
 
 typedef QPair<QString, quint32> qpChanCol;
 
 void Overlay::updateOverlay() {
-	quint32 colPlayer = g.s.qcOverlayPlayer.rgba();
+	quint32 colUser = g.s.qcOverlayUser.rgba();
 	quint32 colTalking = g.s.qcOverlayTalking.rgba();
-	quint32 colAltTalking = g.s.qcOverlayAltTalking.rgba();
+	quint32 colWhisper = g.s.qcOverlayWhisper.rgba();
 	quint32 colChannel = g.s.qcOverlayChannel.rgba();
 	quint32 colChannelTalking = g.s.qcOverlayChannelTalking.rgba();
 	QString str;
@@ -332,15 +333,17 @@ void Overlay::updateOverlay() {
 		return;
 
 	if (g.uiSession) {
-		Channel *home = ClientPlayer::get(g.uiSession)->cChannel;
+		Channel *home = ClientUser::get(g.uiSession)->cChannel;
 		foreach(Channel *c, home->allLinks()) {
 			if (home == c)
 				continue;
 
 			bool act = false;
-			foreach(Player *p, c->qlPlayers) {
-				act = act || p->bTalking;
-				if (p->bTalking)
+			foreach(User *p, c->qlUsers) {
+				ClientUser *u = static_cast<ClientUser *>(p);
+				bool talking = (u->tsState != ClientUser::TalkingOff);
+				act = act || talking;
+				if (talking)
 					linkchans << qpChanCol(p->qsName + QString::fromLatin1("[") + c->qsName + QString::fromLatin1("]"), colChannelTalking);
 			}
 			if (! act)
@@ -352,50 +355,56 @@ void Overlay::updateOverlay() {
 	QList<TextLine> lines;
 
 	if (g.uiSession) {
-
 		if (g.s.bOverlayTop) {
 			foreach(qpChanCol cc, linkchans) {
 				if ((g.s.osOverlay == Settings::All) || (cc.second == colChannelTalking)) {
-					lines << TextLine(cc.first, cc.second);
+					lines << TextLine(cc.first, cc.second, 0);
 				}
 			}
 			if (linkchans.count() > 0) {
-				lines << TextLine(QString(), 0);
+				lines << TextLine(QString(), 0, 0);
 			}
 		}
 
-		foreach(Player *p, ClientPlayer::get(g.uiSession)->cChannel->qlPlayers) {
-			if ((g.s.osOverlay == Settings::All) || p->bTalking || ((p == ClientPlayer::get(g.uiSession)) && g.s.bOverlayAlwaysSelf)) {
-				if (g.s.bOverlayUserTextures && (p->iId >= 0) && (! qhQueried.contains(p->iId))) {
-					qhQueried.insert(p->iId, p->qsName);
-					MessageTexture mt;
-					mt.iPlayerId = p->iId;
-					g.sh->sendMessage(&mt);
-				}
-				QString name = p->qsName;
+		foreach(User *p, ClientUser::get(g.uiSession)->cChannel->qlUsers) {
+			ClientUser *u = static_cast<ClientUser *>(p);
+			if ((g.s.osOverlay == Settings::All) || (u->tsState != ClientUser::TalkingOff) || ((u == ClientUser::get(g.uiSession)) && g.s.bOverlayAlwaysSelf)) {
+				QString name = u->qsName;
 				Decoration dec = None;
-				if (p->bDeaf || p->bSelfDeaf)
+				if (u->bDeaf || u->bSelfDeaf)
 					dec = Deafened;
-				else if (p->bMute || p->bSelfMute || p->bLocalMute)
+				else if (u->bMute || u->bSelfMute || u->bLocalMute)
 					dec = Muted;
-				lines << TextLine(name, p->bTalking ? (p->bAltSpeak ? colAltTalking : colTalking) : colPlayer, p->iId, dec);
+				quint32 col;
+				switch (u->tsState) {
+					case ClientUser::TalkingOff:
+						col = colUser;
+						break;
+					case ClientUser::Talking:
+						col = colTalking;
+						break;
+					default:
+						col = colWhisper;
+						break;
+				}
+				lines << TextLine(name, col, 1, u->uiSession, dec);
 			}
 		}
 
 		if (! g.s.bOverlayTop) {
 			if (linkchans.count() > 0) {
-				lines << TextLine(QString(), 0);
+				lines << TextLine(QString(), 0, 2);
 			}
 			foreach(qpChanCol cc, linkchans) {
 				if ((g.s.osOverlay == Settings::All) || (cc.second == colChannelTalking)) {
-					lines << TextLine(cc.first, cc.second);
+					lines << TextLine(cc.first, cc.second, 2);
 				}
 			}
 		}
 	} else {
-		qhUserTextures.clear();
 		clearCache();
 	}
+	qSort(lines);
 	setTexts(lines);
 }
 
@@ -435,12 +444,12 @@ void Overlay::clearCache() {
 
 	qhTextures.clear();
 	qhWidths.clear();
-	qhQueried.clear();
 }
 
 void Overlay::setTexts(const QList<TextLine> &lines) {
 	foreach(const TextLine &e, lines) {
-		if ((! e.qsText.isEmpty()) && (! qhTextures.contains(e.qsText)) && (! qhUserTextures.contains(e.iPlayer))) {
+		ClientUser *cp = ClientUser::get(e.uiSession);
+		if ((! e.qsText.isEmpty()) && (! qhTextures.contains(e.qsText)) && !(g.s.bOverlayUserTextures && cp && cp->iTextureWidth)) {
 			unsigned char *td = new unsigned char[TEXTURE_SIZE];
 			memset(td, 0, TEXTURE_SIZE);
 
@@ -467,7 +476,7 @@ void Overlay::setTexts(const QList<TextLine> &lines) {
 		}
 	}
 
-	if (! sm.tryLock())
+	if (! sm.sm || ! sm.tryLock())
 		return;
 
 	int i;
@@ -482,17 +491,17 @@ void Overlay::setTexts(const QList<TextLine> &lines) {
 		tl.qsText.left(127).toWCharArray(te->text);
 		te->color = lines[i].uiColor;
 
-		if ((i >= qlCurrentTexts.count()) || (qlCurrentTexts[i].dDecor != tl.dDecor) || (qlCurrentTexts[i].qsText != tl.qsText) || qsForce.contains(tl.iPlayer)) {
+		if ((i >= qlCurrentTexts.count()) || (qlCurrentTexts[i].dDecor != tl.dDecor) || (qlCurrentTexts[i].qsText != tl.qsText) || qsForce.contains(tl.uiSession)) {
 			if (tl.qsText.isNull()) {
 				te->width = 0;
 			} else {
 				int width = 0;
 				const unsigned char *src = NULL;
+				ClientUser *cp = ClientUser::get(tl.uiSession);
 
-				if (qhUserTextures.contains(tl.iPlayer)) {
-					const UserTexture &ut=qhUserTextures.value(tl.iPlayer);
-					width = ut.first;
-					src = reinterpret_cast<const unsigned char *>(ut.second.constData());
+				if (g.s.bOverlayUserTextures && cp && cp->iTextureWidth) {
+					width = cp->iTextureWidth;
+					src = reinterpret_cast<const unsigned char *>(cp->qbaTexture.constData());
 				} else {
 					width = qhWidths[tl.qsText];
 					src = qhTextures[tl.qsText];
@@ -529,7 +538,7 @@ void Overlay::setTexts(const QList<TextLine> &lines) {
 				te->width = static_cast<short>(width);
 				te->uiCounter++;
 			}
-			qsForce.remove(tl.iPlayer);
+			qsForce.remove(tl.uiSession);
 		}
 	}
 
