@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -28,14 +28,17 @@
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
 
-#include "Player.h"
+#include "User.h"
 #include "Channel.h"
 #include "Overlay.h"
+#include "MainWindow.h"
 #include "Global.h"
 
 typedef void (__cdecl *HooksProc)();
 typedef SharedMem * (__cdecl *GetSharedMemProc)();
+typedef unsigned int (__cdecl *GetOverlayMagicVersionProc)();
 typedef void (__cdecl *PrepProc)();
+typedef void (__cdecl *PrepDXGIProc)();
 
 class SharedMemoryPrivate {
 	public:
@@ -54,14 +57,27 @@ SharedMemory::~SharedMemory() {
 }
 
 void SharedMemory::resolve(QLibrary *lib) {
+	GetOverlayMagicVersionProc gompvp = (GetOverlayMagicVersionProc)lib->resolve("GetOverlayMagicVersion");
+	if (! gompvp)
+		return;
+
+	if (gompvp() != OVERLAY_MAGIC_NUMBER)
+		return;
+
 	GetSharedMemProc gsmp = (GetSharedMemProc)lib->resolve("GetSharedMemory");
 	if (gsmp)
 		sm=gsmp();
 	d->hMutex = CreateMutex(NULL, false, L"MumbleSharedMutex");
 
-	PrepProc pp = (PrepProc) lib->resolve("PrepareD3D9");
-	if (pp)
-		pp();
+	if (sm) {
+		PrepProc pp = (PrepProc) lib->resolve("PrepareD3D9");
+		if (pp)
+			pp();
+
+		PrepDXGIProc pdxgi = (PrepDXGIProc) lib->resolve("PrepareDXGI");
+		if (pdxgi)
+			pdxgi();
+	}
 }
 
 bool SharedMemory::tryLock() {
@@ -73,15 +89,15 @@ void SharedMemory::unlock() {
 	ReleaseMutex(d->hMutex);
 }
 
-class OverlayPrivate {
+class OverlayPrivateWin : public OverlayPrivate {
 	public:
 		HooksProc hpInstall, hpRemove;
 };
 
 void Overlay::platformInit() {
-	d = new OverlayPrivate();
-	d->hpInstall = (HooksProc)qlOverlay->resolve("InstallHooks");
-	d->hpRemove = (HooksProc)qlOverlay->resolve("RemoveHooks");
+	d = new OverlayPrivateWin();
+	static_cast<OverlayPrivateWin *>(d)->hpInstall = (HooksProc)qlOverlay->resolve("InstallHooks");
+	static_cast<OverlayPrivateWin *>(d)->hpRemove = (HooksProc)qlOverlay->resolve("RemoveHooks");
 }
 
 void Overlay::setActive(bool act) {
@@ -89,9 +105,9 @@ void Overlay::setActive(bool act) {
 		return;
 
 	if (act)
-		d->hpInstall();
+		static_cast<OverlayPrivateWin *>(d)->hpInstall();
 	else
-		d->hpRemove();
+		static_cast<OverlayPrivateWin *>(d)->hpRemove();
 }
 
 void Overlay::on_Timer_timeout() {
