@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -42,7 +42,7 @@ class PluginConfig : public ConfigWidget, public Ui::PluginConfig {
 		Q_DISABLE_COPY(PluginConfig)
 	protected:
 		void refillPluginList();
-		QHash<QListWidgetItem *, PluginInfo *> qhInfos;
+		QHash<int, PluginInfo *> qhInfos;
 	public:
 		PluginConfig(Settings &st);
 		virtual QString title() const;
