@@ -1,4 +1,5 @@
 /* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -124,13 +125,15 @@ Settings::Settings() {
 	bTTS = true;
 	iTTSVolume = 75;
 	iTTSThreshold = 250;
-	iQuality = 32000;
+	iQuality = 40000;
 	fVolume = 1.0f;
 	fOtherVolume = 0.5f;
+	bAttenuateOthersOnTalk = false;
+	bAttenuateOthers = true;
 	iMinLoudness = 1000;
 	iVoiceHold = 50;
 	iJitterBufferSize = 1;
-	iFramesPerPacket = 1;
+	iFramesPerPacket = 2;
 	iNoiseSuppress = -30;
 	iIdleTime = 0;
 	vsVAD = SignalToNoise;
@@ -156,21 +159,23 @@ Settings::Settings() {
 	bPluginCheck = true;
 #endif
 
+	qsImagePath = QDesktopServices::storageLocation(QDesktopServices::PicturesLocation);
+
 	bFirstTime = true;
 	ceExpand = ChannelsWithUsers;
 	ceChannelDrag = Ask;
 	bShowChatbar = true;
 	bMinimalView = false;
 	bHideFrame = false;
-	bAlwaysOnTop = false;
+	aotbAlwaysOnTop = OnTopNever;
 	bAskOnQuit = true;
 	bHideTray = true;
+	bStateInTray = true;
 	bUsage = true;
-	bAdvancedACLCfg = false;
-	iWindowLayout = LayoutClassic;
+	bShowUserCount = false;
+	wlWindowLayout = LayoutClassic;
 
-	bHideUnreachable = true;
-	bHideEmpty = false;
+	ssFilter = ShowReachable;
 
 	iOutputDelay = 5;
 
@@ -187,7 +192,7 @@ Settings::Settings() {
 	bPositionalHeadphone = false;
 	fAudioMinDistance = 1.0f;
 	fAudioMaxDistance = 15.0f;
-	fAudioMaxDistVolume = 0.15f;
+	fAudioMaxDistVolume = 0.80f;
 	fAudioBloom = 0.5f;
 
 	bOverlayEnable = true;
@@ -200,8 +205,8 @@ Settings::Settings() {
 	bOverlayBottom = true;
 	bOverlayLeft = true;
 	bOverlayRight = false;
-#ifdef Q_OS_WIN
-	qfOverlayFont = QFont(QLatin1String("Comic Sans MS"), 20);
+#if defined(Q_OS_WIN) || defined(Q_OS_MAC)
+	qfOverlayFont = QFont(QLatin1String("Verdana"), 20);
 #else
 	qfOverlayFont = QFont(QLatin1String("Arial"), 20);
 #endif
@@ -217,6 +222,7 @@ Settings::Settings() {
 
 	// Network settings
 	bTCPCompat = false;
+	bQoS = true;
 	bReconnect = true;
 	ptProxyType = NoProxy;
 	usProxyPort = 0;
@@ -227,7 +233,7 @@ Settings::Settings() {
 	bSuppressIdentity = false;
 
 #if defined(AUDIO_TEST)
-	lmLoopMode = Local;
+	lmLoopMode = Server;
 #else
 	lmLoopMode = None;
 #endif
@@ -240,8 +246,26 @@ Settings::Settings() {
 	for (int i=Log::firstMsgType;i<=Log::lastMsgType;++i)
 		qmMessageSounds.insert(i, QString());
 
+	qmMessageSounds[Log::CriticalError] = QLatin1String(":/Critical.ogg");
+	qmMessageSounds[Log::PermissionDenied] = QLatin1String(":/PermissionDenied.ogg");
+	qmMessageSounds[Log::SelfMute] = QLatin1String(":/SelfMutedDeafened.ogg");
+	qmMessageSounds[Log::ServerConnected] = QLatin1String(":/ServerConnected.ogg");
+	qmMessageSounds[Log::ServerDisconnected] = QLatin1String(":/ServerDisconnected.ogg");
+	qmMessageSounds[Log::TextMessage] = QLatin1String(":/TextMessage.ogg");
+	qmMessageSounds[Log::ChannelJoin] = QLatin1String(":/UserJoinedChannel.ogg");
+	qmMessageSounds[Log::ChannelLeave] = QLatin1String(":/UserLeftChannel.ogg");
+	qmMessageSounds[Log::YouMutedOther] = QLatin1String(":/UserMutedYouOrByYou.ogg");
+	qmMessageSounds[Log::YouMuted] = QLatin1String(":/UserMutedYouOrByYou.ogg");
+	qmMessageSounds[Log::YouKicked] = QLatin1String(":/UserKickedYouOrByYou.ogg");
+
 	qmMessages[Log::DebugInfo] = Settings::LogConsole;
+	qmMessages[Log::Warning] = Settings::LogConsole | Settings::LogBalloon;
 	qmMessages[Log::Information] = Settings::LogConsole;
+	qmMessages[Log::UserJoin] = Settings::LogConsole;
+	qmMessages[Log::UserLeave] = Settings::LogConsole;
+	qmMessages[Log::UserKicked] = Settings::LogConsole;
+	qmMessages[Log::OtherSelfMute] = Settings::LogConsole;
+	qmMessages[Log::OtherMutedOther] = Settings::LogConsole;
 }
 
 bool Settings::doEcho() const {
@@ -272,6 +296,9 @@ BOOST_TYPEOF_REGISTER_TYPE(Settings::OverlayShow)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::ProxyType)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::ChannelExpand)
 BOOST_TYPEOF_REGISTER_TYPE(Settings::ChannelDrag)
+BOOST_TYPEOF_REGISTER_TYPE(Settings::ServerShow)
+BOOST_TYPEOF_REGISTER_TYPE(Settings::WindowLayout)
+BOOST_TYPEOF_REGISTER_TYPE(Settings::AlwaysOnTopBehaviour)
 BOOST_TYPEOF_REGISTER_TYPE(QString)
 BOOST_TYPEOF_REGISTER_TYPE(QByteArray)
 BOOST_TYPEOF_REGISTER_TYPE(QColor)
@@ -294,6 +321,8 @@ void Settings::load() {
 	SAVELOAD(iMinLoudness, "audio/loudness");
 	SAVELOAD(fVolume, "audio/volume");
 	SAVELOAD(fOtherVolume, "audio/othervolume");
+	SAVELOAD(bAttenuateOthers, "audio/attenuateothers");
+	SAVELOAD(bAttenuateOthersOnTalk, "audio/attenuateothersontalk");
 	LOADENUM(vsVAD, "audio/vadsource");
 	SAVELOAD(fVADmin, "audio/vadmin");
 	SAVELOAD(fVADmax, "audio/vadmax");
@@ -332,6 +361,9 @@ void Settings::load() {
 	SAVELOAD(qsOSSInput, "oss/input");
 	SAVELOAD(qsOSSOutput, "oss/output");
 
+	SAVELOAD(qsCoreAudioInput, "coreaudio/input");
+	SAVELOAD(qsCoreAudioOutput, "coreaudio/output");
+
 	SAVELOAD(iPortAudioInput, "portaudio/input");
 	SAVELOAD(iPortAudioOutput, "portaudio/output");
 
@@ -362,6 +394,7 @@ void Settings::load() {
 
 	// Network settings
 	SAVELOAD(bTCPCompat, "net/tcponly");
+	SAVELOAD(bQoS, "net/qos");
 	SAVELOAD(bReconnect, "net/reconnect");
 	SAVELOAD(bSuppressIdentity, "net/suppress");
 	LOADENUM(ptProxyType, "net/proxytype");
@@ -379,31 +412,32 @@ void Settings::load() {
 	SAVELOAD(qsSkin, "ui/skin");
 	LOADENUM(ceExpand, "ui/expand");
 	LOADENUM(ceChannelDrag, "ui/drag");
-	SAVELOAD(bAlwaysOnTop, "ui/alwaysontop");
+	LOADENUM(aotbAlwaysOnTop, "ui/alwaysontop");
 	SAVELOAD(bAskOnQuit, "ui/askonquit");
 	SAVELOAD(bShowChatbar, "ui/shotchatbar");
 	SAVELOAD(bMinimalView, "ui/minimalview");
 	SAVELOAD(bHideFrame, "ui/hideframe");
 	SAVELOAD(bUserTop, "ui/usertop");
-	SAVELOAD(bFirstTime, "ui/firsttime");
+	SAVELOAD(bFirstTime, "ui/firsttime120");
 	SAVELOAD(qbaMainWindowGeometry, "ui/geometry");
 	SAVELOAD(qbaMainWindowState, "ui/state");
 	SAVELOAD(qbaMinimalViewGeometry, "ui/minimalviewgeometry");
 	SAVELOAD(qbaConfigGeometry, "ui/ConfigGeometry");
-	SAVELOAD(iWindowLayout, "ui/WindowLayout");
+	LOADENUM(wlWindowLayout, "ui/WindowLayout");
 	SAVELOAD(qbaSplitterState, "ui/splitter");
 	SAVELOAD(qbaHeaderState, "ui/header");
 	SAVELOAD(qsUsername, "ui/username");
 	SAVELOAD(qsLastServer, "ui/server");
-	SAVELOAD(bHideUnreachable, "ui/hideunreachable");
-	SAVELOAD(bHideEmpty, "ui/hideempty");
+	LOADENUM(ssFilter, "ui/serverfilter");
 #ifndef NO_UPDATE_CHECK
 	SAVELOAD(bUpdateCheck, "ui/updatecheck");
 	SAVELOAD(bPluginCheck, "ui/plugincheck");
 #endif
 	SAVELOAD(bHideTray, "ui/hidetray");
+	SAVELOAD(bStateInTray, "ui/stateintray");
 	SAVELOAD(bUsage, "ui/usage");
-	SAVELOAD(bAdvancedACLCfg, "ui/advancedaclcfg");
+	SAVELOAD(bShowUserCount, "ui/showusercount");
+	SAVELOAD(qsImagePath, "ui/imagepath");
 
 	SAVELOAD(iLCDUserViewMinColWidth, "lcd/userview/mincolwidth");
 	SAVELOAD(iLCDUserViewSplitterWidth, "lcd/userview/splitterwidth");
@@ -447,6 +481,12 @@ void Settings::load() {
 		qmLCDDevices.insert(d, g.qs->value(d, true).toBool());
 	}
 	g.qs->endGroup();
+
+	g.qs->beginGroup(QLatin1String("audio/plugins"));
+	foreach(const QString &d, g.qs->childKeys()) {
+		qmPositionalAudioPlugins.insert(d, g.qs->value(d, true).toBool());
+	}
+	g.qs->endGroup();
 }
 
 #undef SAVELOAD
@@ -466,6 +506,8 @@ void Settings::save() {
 	SAVELOAD(iMinLoudness, "audio/loudness");
 	SAVELOAD(fVolume, "audio/volume");
 	SAVELOAD(fOtherVolume, "audio/othervolume");
+	SAVELOAD(bAttenuateOthers, "audio/attenuateothers");
+	SAVELOAD(bAttenuateOthersOnTalk, "audio/attenuateothersontalk");
 	SAVELOAD(vsVAD, "audio/vadsource");
 	SAVELOAD(fVADmin, "audio/vadmin");
 	SAVELOAD(fVADmax, "audio/vadmax");
@@ -504,6 +546,9 @@ void Settings::save() {
 	SAVELOAD(qsOSSInput, "oss/input");
 	SAVELOAD(qsOSSOutput, "oss/output");
 
+	SAVELOAD(qsCoreAudioInput, "coreaudio/input");
+	SAVELOAD(qsCoreAudioOutput, "coreaudio/output");
+
 	SAVELOAD(iPortAudioInput, "portaudio/input");
 	SAVELOAD(iPortAudioOutput, "portaudio/output");
 
@@ -534,6 +579,7 @@ void Settings::save() {
 
 	// Network settings
 	SAVELOAD(bTCPCompat, "net/tcponly");
+	SAVELOAD(bQoS, "net/qos");
 	SAVELOAD(bReconnect, "net/reconnect");
 	SAVELOAD(ptProxyType, "net/proxytype");
 	SAVELOAD(qsProxyHost, "net/proxyhost");
@@ -550,29 +596,30 @@ void Settings::save() {
 	SAVELOAD(qsSkin, "ui/skin");
 	SAVELOAD(ceExpand, "ui/expand");
 	SAVELOAD(ceChannelDrag, "ui/drag");
-	SAVELOAD(bAlwaysOnTop, "ui/alwaysontop");
+	SAVELOAD(aotbAlwaysOnTop, "ui/alwaysontop");
 	SAVELOAD(bAskOnQuit, "ui/askonquit");
 	SAVELOAD(bShowChatbar, "ui/showchatbar");
 	SAVELOAD(bMinimalView, "ui/minimalview");
 	SAVELOAD(bHideFrame, "ui/hideframe");
 	SAVELOAD(bUserTop, "ui/usertop");
-	SAVELOAD(bFirstTime, "ui/firsttime");
+	SAVELOAD(bFirstTime, "ui/firsttime120");
 	SAVELOAD(qbaMainWindowGeometry, "ui/geometry");
 	SAVELOAD(qbaMainWindowState, "ui/state");
 	SAVELOAD(qbaMinimalViewGeometry, "ui/minimalviewgeometry");
 	SAVELOAD(qbaConfigGeometry, "ui/ConfigGeometry");
-	SAVELOAD(iWindowLayout, "ui/WindowLayout");
+	SAVELOAD(wlWindowLayout, "ui/WindowLayout");
 	SAVELOAD(qbaSplitterState, "ui/splitter");
 	SAVELOAD(qbaHeaderState, "ui/header");
 	SAVELOAD(qsUsername, "ui/username");
 	SAVELOAD(qsLastServer, "ui/server");
-	SAVELOAD(bHideUnreachable, "ui/hideunreachable");
-	SAVELOAD(bHideEmpty, "ui/hideempty");
+	SAVELOAD(ssFilter, "ui/serverfilter");
 	SAVELOAD(bUpdateCheck, "ui/updatecheck");
 	SAVELOAD(bPluginCheck, "ui/plugincheck");
 	SAVELOAD(bHideTray, "ui/hidetray");
+	SAVELOAD(bStateInTray, "ui/stateintray");
 	SAVELOAD(bUsage, "ui/usage");
-	SAVELOAD(bAdvancedACLCfg, "ui/advancedaclcfg");
+	SAVELOAD(bShowUserCount, "ui/showusercount");
+	SAVELOAD(qsImagePath, "ui/imagepath");
 
 	SAVELOAD(iLCDUserViewMinColWidth, "lcd/userview/mincolwidth");
 	SAVELOAD(iLCDUserViewSplitterWidth, "lcd/userview/splitterwidth");
@@ -616,4 +663,14 @@ void Settings::save() {
 			g.qs->remove(d);
 	}
 	g.qs->endGroup();
+
+	g.qs->beginGroup(QLatin1String("audio/plugins"));
+	foreach(const QString &d, qmPositionalAudioPlugins.keys()) {
+		bool v = qmPositionalAudioPlugins.value(d);
+		if (!v)
+			g.qs->setValue(d, v);
+		else
+			g.qs->remove(d);
+	}
+	g.qs->endGroup();
 }
