@@ -1,4 +1,5 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
+   Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
 
@@ -79,6 +80,7 @@ struct Settings {
 	enum OverlayShow { Nothing, Talking, All };
 	enum ChannelExpand { NoChannels, ChannelsWithUsers, AllChannels };
 	enum ChannelDrag { Ask, DoNothing, Move };
+	enum ServerShow { ShowPopulated, ShowReachable, ShowAll };
 	typedef QPair<QList<QSslCertificate>, QSslKey> KeyPair;
 
 	AudioTransmit atTransmit;
@@ -107,6 +109,8 @@ struct Settings {
 	QString qsAudioInput, qsAudioOutput;
 	float fVolume;
 	float fOtherVolume;
+	bool bAttenuateOthersOnTalk;
+	bool bAttenuateOthers;
 	int iOutputDelay;
 
 	QString qsALSAInput, qsALSAOutput;
@@ -116,9 +120,8 @@ struct Settings {
 	QString qsASIOclass;
 	QList<QVariant> qlASIOmic;
 	QList<QVariant> qlASIOspeaker;
-
+	QString qsCoreAudioInput, qsCoreAudioOutput;
 	QString qsWASAPIInput, qsWASAPIOutput;
-
 	QByteArray qbaDXInput, qbaDXOutput;
 
 	bool bEcho;
@@ -126,6 +129,7 @@ struct Settings {
 	bool bPositionalAudio;
 	bool bPositionalHeadphone;
 	float fAudioMinDistance, fAudioMaxDistance, fAudioMaxDistVolume, fAudioBloom;
+	QMap<QString, bool> qmPositionalAudioPlugins;
 
 	bool bOverlayEnable;
 	bool bOverlayUserTextures;
@@ -159,29 +163,34 @@ struct Settings {
 	QByteArray qbaMainWindowGeometry, qbaMainWindowState, qbaMinimalViewGeometry, qbaSplitterState, qbaHeaderState;
 	QByteArray qbaConfigGeometry;
 	enum WindowLayout { LayoutClassic, LayoutStacked, LayoutHybrid, LayoutCustom };
-	int iWindowLayout;
+	WindowLayout wlWindowLayout;
 	ChannelExpand ceExpand;
 	ChannelDrag ceChannelDrag;
 	bool bShowChatbar;
 	bool bMinimalView;
 	bool bHideFrame;
-	bool bAlwaysOnTop;
+	enum AlwaysOnTopBehaviour { OnTopNever, OnTopAlways, OnTopInMinimal, OnTopInNormal };
+	AlwaysOnTopBehaviour aotbAlwaysOnTop;
 	bool bAskOnQuit;
 	bool bHideTray;
+	bool bStateInTray;
 	bool bUsage;
-	bool bAdvancedACLCfg;
+	bool bShowUserCount;
 
 	QString qsUsername;
 	QString qsLastServer;
-	bool bHideUnreachable;
-	bool bHideEmpty;
+	ServerShow ssFilter;
+
+	QString qsImagePath;
 
 	bool bUpdateCheck;
 	bool bPluginCheck;
 
 	// Network settings
 	enum ProxyType { NoProxy, HttpProxy, Socks5Proxy };
-	bool bTCPCompat, bReconnect;
+	bool bTCPCompat;
+	bool bReconnect;
+	bool bQoS;
 	ProxyType ptProxyType;
 	QString qsProxyHost, qsProxyUsername, qsProxyPassword;
 	unsigned short usProxyPort;
