@@ -259,6 +259,7 @@ void WASAPIInput::run() {
 	DWORD dwTaskIndex = 0;
 	HANDLE hMmThread;
 	float *tbuff = NULL;
+	bool doecho = g.s.doEcho();
 
 	CoInitialize(NULL);
 
@@ -307,7 +308,7 @@ void WASAPIInput::run() {
 		CoTaskMemFree(devname);
 	}
 
-	if (g.s.doEcho()) {
+	if (doecho) {
 		if (! g.s.qsWASAPIOutput.isEmpty()) {
 			STACKVAR(wchar_t, devname, g.s.qsWASAPIOutput.length());
 			g.s.qsWASAPIOutput.toWCharArray(devname);
@@ -384,7 +385,7 @@ void WASAPIInput::run() {
 	iMicChannels = micpwfx->nChannels;
 	iMicFreq = micpwfx->nSamplesPerSec;
 
-	if (g.s.doEcho()) {
+	if (doecho) {
 		hr = pEchoDevice->Activate(__uuidof(IAudioClient), CLSCTX_ALL, NULL, (void **) &pEchoAudioClient);
 		if (FAILED(hr)) {
 			qWarning("WASAPIInput: Activate Echo AudioClient failed");
@@ -531,9 +532,23 @@ WASAPIOutput::~WASAPIOutput() {
 }
 
 void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
+	HRESULT hr;
+
+	if (! talking) {
+		QMap<ISimpleAudioVolume *, VolumePair>::const_iterator i;
+		for (i=qmVolumes.constBegin(); i != qmVolumes.constEnd(); ++i) {
+			float fVolume = 1.0f;
+			hr = i.key()->GetMasterVolume(&fVolume);
+			if (qFuzzyCompare(i.value().second, fVolume))
+				hr = i.key()->SetMasterVolume(i.value().first, NULL);
+			i.key()->Release();
+		}
+		qmVolumes.clear();
+		return;
+	}
+
 	IAudioSessionManager2 *pAudioSessionManager = NULL;
 	int max = 0;
-	HRESULT hr;
 	DWORD dwMumble = GetCurrentProcessId();
 
 	static int version = -1;
@@ -550,14 +565,11 @@ void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
 			version = 1;
 	}
 
-	if (talking) {
-		qmVolumes.clear();
-		if (qFuzzyCompare(g.s.fOtherVolume, 1.0f))
-			return;
-	} else if (qmVolumes.isEmpty()) {
+	qmVolumes.clear();
+	if (qFuzzyCompare(g.s.fOtherVolume, 1.0f))
 		return;
-	}
 
+	// FIXME: Try to keep the session object around when returning volume.
 
 	if (SUCCEEDED(hr = pDevice->Activate(version ? __uuidof(IAudioSessionManager2) : __uuidof(IAudioSessionManager), CLSCTX_ALL, NULL, (void **) &pAudioSessionManager))) {
 		IAudioSessionEnumerator *pEnumerator = NULL;
@@ -569,6 +581,9 @@ void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
 		} else {
 			hr = pAudioSessionManager->GetSessionEnumerator(&pEnumerator);
 		}
+
+		QSet<QUuid> seen;
+
 		if (SUCCEEDED(hr)) {
 			if (SUCCEEDED(hr = pEnumerator->GetCount(&max))) {
 				for (int i=0;i<max;++i) {
@@ -579,37 +594,29 @@ void WASAPIOutput::setVolumes(IMMDevice *pDevice, bool talking) {
 							DWORD pid;
 							if (SUCCEEDED(hr = pControl2->GetProcessId(&pid)) && (pid != dwMumble)) {
 								AudioSessionState ass;
-								if (SUCCEEDED(hr = pControl2->GetState(&ass)) && (! talking || (ass != AudioSessionStateExpired))) {
+								if (SUCCEEDED(hr = pControl2->GetState(&ass)) && (ass != AudioSessionStateExpired)) {
 									GUID group;
 									if (SUCCEEDED(hr = pControl2->GetGroupingParam(&group))) {
 										QUuid quuid(group);
-										if (qmVolumes.contains(quuid) != talking) {
+										if (! seen.contains(quuid)) {
+											seen.insert(quuid);
 											ISimpleAudioVolume *pVolume = NULL;
 											if (SUCCEEDED(hr = pControl2->QueryInterface(__uuidof(ISimpleAudioVolume), (void **) &pVolume))) {
-												if (talking) {
-													BOOL bMute = TRUE;
-													if (SUCCEEDED(hr = pVolume->GetMute(&bMute)) && ! bMute) {
-														float fVolume = 1.0f;
-														if (!qmVolumes.contains(quuid) && SUCCEEDED(hr = pVolume->GetMasterVolume(&fVolume)) && ! qFuzzyCompare(fVolume,0.0f)) {
-															float fSetVolume = fVolume * g.s.fOtherVolume;
-															if (SUCCEEDED(hr = pVolume->SetMasterVolume(fSetVolume, NULL))) {
-																hr = pVolume->GetMasterVolume(&fSetVolume);
-																qmVolumes.insert(quuid, VolumePair(fVolume,fSetVolume));
-															}
+												BOOL bMute = TRUE;
+												bool keep = false;
+												if (SUCCEEDED(hr = pVolume->GetMute(&bMute)) && ! bMute) {
+													float fVolume = 1.0f;
+													if (SUCCEEDED(hr = pVolume->GetMasterVolume(&fVolume)) && ! qFuzzyCompare(fVolume,0.0f)) {
+														float fSetVolume = fVolume * g.s.fOtherVolume;
+														if (SUCCEEDED(hr = pVolume->SetMasterVolume(fSetVolume, NULL))) {
+															hr = pVolume->GetMasterVolume(&fSetVolume);
+															qmVolumes.insert(pVolume, VolumePair(fVolume,fSetVolume));
+															keep = true;
 														}
 													}
-												} else {
-													VolumePair vp = qmVolumes.value(quuid);
-													if (vp.first) {
-														float fVolume = 1.0f;
-														hr = pVolume->GetMasterVolume(&fVolume);
-														if (qFuzzyCompare(fVolume, vp.second)) {
-															pVolume->SetMasterVolume(vp.first, NULL);
-														}
-														qmVolumes.remove(quuid);
-													}
 												}
-												pVolume->Release();
+												if (! keep)
+													pVolume->Release();
 											}
 										}
 									}
@@ -709,15 +716,19 @@ void WASAPIOutput::run() {
 	}
 
 	hr = pAudioClient->GetMixFormat(&pwfx);
+	if (FAILED(hr)) {
+		qWarning("WASAPIOutput: GetMixFormat failed: %llx", hr);
+		goto cleanup;
+	}
 	pwfxe = reinterpret_cast<WAVEFORMATEXTENSIBLE *>(pwfx);
 
-	iMixerFreq = pwfx->nSamplesPerSec;
-
 	if (FAILED(hr) || (pwfx->wBitsPerSample != (sizeof(float) * 8)) || (pwfxe->SubFormat != KSDATAFORMAT_SUBTYPE_IEEE_FLOAT)) {
 		qWarning("WASAPIOutput: Subformat is not IEEE Float");
 		goto cleanup;
 	}
 
+	iMixerFreq = pwfx->nSamplesPerSec;
+
 	hr = pAudioClient->Initialize(AUDCLNT_SHAREMODE_SHARED, AUDCLNT_STREAMFLAGS_EVENTCALLBACK, bufferDuration, 0, pwfx, NULL);
 	if (FAILED(hr)) {
 		qWarning("WASAPIOutput: Initialize failed");
@@ -763,6 +774,7 @@ void WASAPIOutput::run() {
 	iChannels = pwfx->nChannels;
 	initializeMixer(chanmasks);
 
+	bool mixed = false;
 	while (bRunning && ! FAILED(hr)) {
 		hr = pAudioClient->GetCurrentPadding(&numFramesAvailable);
 		if (FAILED(hr))
@@ -770,12 +782,17 @@ void WASAPIOutput::run() {
 
 		packetLength = bufferFrameCount - numFramesAvailable;
 
+		if (lastspoke != (g.bAttenuateOthers || mixed)) {
+			lastspoke = g.bAttenuateOthers || mixed;
+			setVolumes(pDevice, lastspoke);
+		}
+
 		while (packetLength > 0) {
 			hr = pRenderClient->GetBuffer(packetLength, &pData);
 			if (FAILED(hr))
 				goto cleanup;
 
-			bool mixed = mix(reinterpret_cast<float *>(pData), packetLength);
+			mixed = mix(reinterpret_cast<float *>(pData), packetLength);
 			if (mixed)
 				hr = pRenderClient->ReleaseBuffer(packetLength, 0);
 			else
@@ -783,9 +800,13 @@ void WASAPIOutput::run() {
 			if (FAILED(hr))
 				goto cleanup;
 
-			if (lastspoke != mixed) {
-				lastspoke = mixed;
-				setVolumes(pDevice, mixed);
+			if (!g.s.bAttenuateOthers && !g.bAttenuateOthers) {
+				mixed = false;
+			}
+
+			if (lastspoke != (g.bAttenuateOthers || mixed)) {
+				lastspoke = g.bAttenuateOthers || mixed;
+				setVolumes(pDevice, lastspoke);
 			}
 
 			hr = pAudioClient->GetCurrentPadding(&numFramesAvailable);
