@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -46,6 +46,9 @@
 #ifdef USE_DBUS
 #include "DBus.h"
 #endif
+#ifdef USE_VLD
+#include "vld.h"
+#endif
 #include "VersionCheck.h"
 #include "NetworkConfig.h"
 #include "CrashReporter.h"
@@ -64,21 +67,23 @@ extern char *os_lang;
 
 class QAppMumble:public QApplication {
 	public:
-		QAppMumble(int & argc, char ** argv) : QApplication(argc, argv) {}
+		QAppMumble(int &pargc, char **pargv) : QApplication(pargc, pargv) {}
 		void commitData(QSessionManager&);
 };
 
 void QAppMumble::commitData(QSessionManager &) {
 	// Make sure the config is saved and supress the ask on quite message
-	g.s.save();
-	g.mw->bSupressAskOnQuit = true;
+	if (g.mw) {
+		g.s.save();
+		g.mw->bSuppressAskOnQuit = true;
+	}
 }
 
 int main(int argc, char **argv) {
 	int res;
 
 
-	QT_REQUIRE_VERSION(argc, argv, "4.5.0");
+	QT_REQUIRE_VERSION(argc, argv, "4.4.0");
 
 	// Initialize application object.
 	QAppMumble a(argc, argv);
@@ -89,19 +94,29 @@ int main(int argc, char **argv) {
 
 	Global::g_global_struct = new Global();
 
+	qsrand(QDateTime::currentDateTime().toTime_t());
+
 #if defined(Q_OS_WIN) || defined(Q_OS_MAC)
 	os_init();
 #endif
 
+	bool bAllowMultiple = false;
 	QUrl url;
 	if (a.arguments().count() > 1) {
-		QUrl u = QUrl::fromEncoded(a.arguments().last().toUtf8());
-		if (u.isValid() && (u.scheme() == QLatin1String("mumble"))) {
-			url = u;
-		} else {
-			QFile f(a.arguments().last());
-			if (f.exists()) {
-				url = QUrl::fromLocalFile(f.fileName());
+		QStringList args = a.arguments();
+		for (int i = 1; i < args.count(); ++i) {
+			if (args.at(i) == QLatin1String("-m")) {
+				bAllowMultiple = true;
+			} else {
+				QUrl u = QUrl::fromEncoded(args.at(i).toUtf8());
+				if (u.isValid() && (u.scheme() == QLatin1String("mumble"))) {
+					url = u;
+				} else {
+					QFile f(args.at(i));
+					if (f.exists()) {
+						url = QUrl::fromLocalFile(f.fileName());
+					}
+				}
 			}
 		}
 	}
@@ -125,36 +140,61 @@ int main(int argc, char **argv) {
 	}
 #endif
 
-#ifdef QT_NO_DEBUG
-	QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
-	if (url.isValid()) {
-		QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
-		if (reply.type() == QDBusMessage::ReplyMessage) {
-			return 0;
-		}
-	} else {
-		QDBusMessage reply=qdbi.call(QLatin1String("focus"));
-		if (reply.type() == QDBusMessage::ReplyMessage) {
-			return 0;
+	if (! bAllowMultiple) {
+		if (url.isValid()) {
+			int major, minor, patch;
+			major = 1;
+			minor = 1;
+			patch = 0;
+
+			QString version = url.queryItemValue(QLatin1String("version"));
+
+			QRegExp rx(QLatin1String("(\\d+)\\.(\\d+)\\.(\\d+)"));
+			if (rx.exactMatch(version)) {
+				major = rx.cap(1).toInt();
+				minor = rx.cap(2).toInt();
+				patch = rx.cap(3).toInt();
+			}
+
+			if ((major == 1) && (minor == 1)) {
+				QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble11x"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
+
+				QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
+				if (reply.type() == QDBusMessage::ReplyMessage) {
+					return 0;
+				} else {
+					QString executable = a.applicationFilePath();
+					int idx = executable.lastIndexOf(QLatin1String("mumble"));
+					if (idx >= 0) {
+						QStringList args;
+						args << url.toString();
+
+						executable.replace(idx, 6, QLatin1String("mumble11x"));
+						if (QProcess::startDetached(executable, args))
+							return 0;
+					}
+				}
+			} else {
+				QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
+
+				QDBusMessage reply=qdbi.call(QLatin1String("openUrl"), QLatin1String(url.toEncoded()));
+				if (reply.type() == QDBusMessage::ReplyMessage)
+					return 0;
+			}
+		} else {
+			QDBusInterface qdbi(QLatin1String("net.sourceforge.mumble.mumble"), QLatin1String("/"), QLatin1String("net.sourceforge.mumble.Mumble"));
+
+			QDBusMessage reply=qdbi.call(QLatin1String("focus"));
+			if (reply.type() == QDBusMessage::ReplyMessage)
+				return 0;
 		}
 	}
-#endif
 #endif
 	// Load preferences
 	g.s.load();
 
 	DeferInit::run_initializers();
 
-#ifdef QT_NO_DEBUG
-	// Remove the hardcoded dev-path to plugins. We don't want those.
-	QStringList np;
-	foreach(const QString &qs, a.libraryPaths()) {
-		if (! qs.startsWith(QLatin1String("C:/dev/Qt")))
-			np << qs;
-	}
-	a.setLibraryPaths(np);
-#endif
-
 	if (! g.s.qsStyle.isEmpty()) {
 		a.setStyle(g.s.qsStyle);
 	}
@@ -168,6 +208,8 @@ int main(int argc, char **argv) {
 			QDir::addSearchPath(QLatin1String("skin"), fi.path());
 			a.setStyleSheet(styleSheet);
 		}
+	} else {
+		a.setStyleSheet(MainWindow::defaultStyleSheet);
 	}
 
 	QDir::addSearchPath(QLatin1String("skin"),QLatin1String(":/"));
@@ -200,14 +242,19 @@ int main(int argc, char **argv) {
 	else if (qttranslator.load(QLatin1String("translation:qt_") + locale))
 		a.installTranslator(&qttranslator);
 
+	g.qsRegionalHost = qsSystemLocale;
+	g.qsRegionalHost = g.qsRegionalHost.remove(QRegExp(QLatin1String("^.+_"))).toLower() + QLatin1String(".mumble.info");
+
 	// Initialize proxy settings
 	NetworkConfig::SetupProxy();
 
 	g.nam = new QNetworkAccessManager();
 
+#ifndef NO_CRASH_REPORT
 	CrashReporter *cr = new CrashReporter();
 	cr->run();
 	delete cr;
+#endif
 
 	// Initialize logger
 	g.l = new Log();
@@ -272,22 +319,46 @@ int main(int argc, char **argv) {
 	}
 
 	if (! CertWizard::validateCert(g.s.kpCertificate)) {
-		CertWizard *cw = new CertWizard(g.mw);
-		cw->exec();
-		delete cw;
+		QDir qd(QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation));
+		QFile qf(qd.absoluteFilePath(QLatin1String("MumbleAutomaticCertificateBackup.p12")));
+		if (qf.open(QIODevice::ReadOnly | QIODevice::Unbuffered)) {
+			Settings::KeyPair kp = CertWizard::importCert(qf.readAll());
+			qf.close();
+			if (CertWizard::validateCert(kp))
+				g.s.kpCertificate = kp;
+		}
+		if (! CertWizard::validateCert(g.s.kpCertificate)) {
+			CertWizard *cw = new CertWizard(g.mw);
+			cw->exec();
+			delete cw;
+
+			if (! CertWizard::validateCert(g.s.kpCertificate)) {
+				g.s.kpCertificate = CertWizard::generateNewCert();
+				if (qf.open(QIODevice::WriteOnly | QIODevice::Truncate | QIODevice::Unbuffered)) {
+					qf.write(CertWizard::exportCert(g.s.kpCertificate));
+					qf.close();
+				}
+			}
+		}
 	}
 
 	if (g.s.bUpdateCheck)
 		new VersionCheck(true, g.mw);
 
+#ifdef SNAPSHOT_BUILD
+	new VersionCheck(false, g.mw, true);
+#endif
+
 	if (g.s.bPluginCheck)
 		g.p->checkUpdates();
 
 	if (url.isValid()) {
-		g.mw->openUrl(url);
+		OpenURLEvent *oue = new OpenURLEvent(url);
+		qApp->postEvent(g.mw, oue);
 #ifdef Q_OS_MAC
 	} else if (os_url) {
-		g.mw->openUrl(QUrl::fromEncoded(QByteArray(os_url)));
+		OpenURLEvent *oue = new OpenURLEvent(QUrl::fromEncoded(QByteArray(os_url)));
+		qApp->postEvent(g.mw, oue);
 #endif
 	} else {
 		g.mw->on_qaServerConnect_triggered();
@@ -296,10 +367,20 @@ int main(int argc, char **argv) {
 	res=a.exec();
 
 	g.s.save();
+	if (g.sh && g.sh->isRunning())
+		Database::setShortcuts(g.sh->qbaDigest, g.s.qlShortcuts);
+
+	AudioInputPtr ai = g.ai;
+	AudioOutputPtr ao = g.ao;
 
 	g.ao.reset();
 	g.ai.reset();
 
+	while (! ai.unique() || ! ao.unique()) {}
+
+	ai.reset();
+	ao.reset();
+
 	g.sh->disconnect();
 	delete g.sh;
 	delete g.mw;
@@ -321,5 +402,14 @@ int main(int argc, char **argv) {
 
 	delete Global::g_global_struct;
 	Global::g_global_struct = NULL;
+
+#ifndef QT_NO_DEBUG
+	// Hide Qt memory leak.
+	QSslSocket::setDefaultCaCertificates(QList<QSslCertificate>());
+	// Release global protobuf memory allocations.
+#if (GOOGLE_PROTOBUF_VERSION >= 2001000)
+	google::protobuf::ShutdownProtobufLibrary();
+#endif
+#endif
 	return res;
 }
