@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -162,8 +162,27 @@ void os_init() {
 	mumble_speex_init();
 
 #ifdef QT_NO_DEBUG
+#ifdef COMPAT_CLIENT
+	errno_t res = 0;
+	size_t reqSize, bSize;
+	_wgetenv_s(&reqSize, NULL, 0, L"APPDATA");
+	if (reqSize > 0) {
+		reqSize += strlen("/Mumble/Console11x.txt");
+		bSize = reqSize;
+
+		STACKVAR(wchar_t, buff, reqSize+1);
+
+		_wgetenv_s(&reqSize, buff, bSize, L"APPDATA");
+		wcscat_s(buff, bSize, L"/Mumble/Console11x.txt");
+		res = _wfopen_s(&fConsole, buff, L"a+");
+	}
+	if ((res != 0) || (! fConsole)) {
+		res=_wfopen_s(&fConsole, L"Console11x.txt", L"a+");
+	}
+#else
 	QString console = g.qdBasePath.filePath(QLatin1String("Console.txt"));
 	errno_t res = _wfopen_s(&fConsole, console.utf16(), L"a+");
+#endif
 
 	if ((res == 0) && fConsole)
 		qInstallMsgHandler(mumbleMessageOutput);
@@ -188,7 +207,15 @@ void os_init() {
 	musComment.Buffer = wcComment;
 	musComment.BufferSize = wcslen(wcComment) * sizeof(wchar_t);
 
+#ifdef COMPAT_CLIENT
+	QString dump = QDesktopServices::storageLocation(QDesktopServices::DataLocation) + QLatin1String("\\mumble11x.dmp");
+#else
 	QString dump = g.qdBasePath.filePath(QLatin1String("mumble.dmp"));
+#endif
+
+	QFileInfo fi(dump);
+	QDir::root().mkpath(fi.absolutePath());
+
 	if (wcscpy_s(wcCrashDumpPath, PATH_MAX, dump.utf16()) == 0)
 		SetUnhandledExceptionFilter(MumbleUnhandledExceptionFilter);
 
