@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -69,6 +69,15 @@
 		sendMessage(uSource, mppd); \
 	}
 
+#define PERM_DENIED_FALLBACK(type,version,text) \
+	{ \
+		MumbleProto::PermissionDenied mppd; \
+		mppd.set_type(MumbleProto::PermissionDenied_DenyType_##type); \
+		if (uSource->uiVersion < version) \
+			mppd.set_reason(u8(text)); \
+		sendMessage(uSource, mppd); \
+	}
+
 #define PERM_DENIED_HASH(user) \
 	{ \
 		MumbleProto::PermissionDenied mppd; \
@@ -445,6 +454,10 @@ void Server::msgUserState(ServerUser *uSource, MumbleProto::UserState &msg) {
 			PERM_DENIED(pDstServerUser, c, ChanACL::Enter);
 			return;
 		}
+		if (iMaxUsersPerChannel && (c->qlUsers.count() >= iMaxUsersPerChannel)) {
+			PERM_DENIED_FALLBACK(ChannelFull, 0x010201, QLatin1String("Channel is full."));
+			return;
+		}
 	}
 
 	if (msg.has_mute() || msg.has_deaf() || msg.has_suppress()) {
@@ -1270,11 +1283,21 @@ void Server::msgUserList(ServerUser *uSource, MumbleProto::UserList &msg) {
 				unregisterUser(id);
 			} else {
 				const QString &name = u8(u.name());
-				log(uSource, QString::fromLatin1("Renamed user %1 to '%2'").arg(QString::number(id), name));
+				if (validateUserName(name)) {
+					log(uSource, QString::fromLatin1("Renamed user %1 to '%2'").arg(QString::number(id), name));
 
-				QMap<int, QString> info;
-				info.insert(ServerDB::User_Name, name);
-				setInfo(id, info);
+					QMap<int, QString> info;
+					info.insert(ServerDB::User_Name, name);
+					setInfo(id, info);
+				} else {
+					MumbleProto::PermissionDenied mppd;
+					mppd.set_type(MumbleProto::PermissionDenied_DenyType_UserName);
+					if (uSource->uiVersion < 0x010201)
+						mppd.set_reason(u8(QString::fromLatin1("%1 is not a valid username").arg(name)));
+					else
+						mppd.set_name(u8(name));
+					sendMessage(uSource, mppd);
+				}
 			}
 		}
 	}
