@@ -46,6 +46,7 @@ MetaParams::MetaParams() {
 	iTimeout = 30;
 	iMaxBandwidth = 72000;
 	iMaxUsers = 1000;
+	iMaxUsersPerChannel = 0;
 	iMaxTextMessageLength = 5000;
 	bAllowHTML = true;
 	iDefaultChan = 0;
@@ -189,13 +190,25 @@ void MetaParams::read(QString fname) {
 			}
 		}
 
-		if (hasipv6)
+		if (hasipv6) {
 			qlBind << QHostAddress(QHostAddress::AnyIPv6);
 #ifdef Q_OS_UNIX
-		else
+			if (hasipv4) {
+				int s = ::socket(AF_INET6, SOCK_STREAM, 0);
+				if (s != -1) {
+					int ipv6only = 0;
+					socklen_t optlen = sizeof(ipv6only);
+					if (getsockopt(s, IPPROTO_IPV6, IPV6_V6ONLY, &ipv6only, &optlen) == 0) {
+						if (ipv6only == 0)
+							hasipv4 = false;
+					}
+					close(s);
+				}
+			}
 #endif
-			if (hasipv4)
-				qlBind << QHostAddress(QHostAddress::Any);
+		}
+		if (hasipv4)
+			qlBind << QHostAddress(QHostAddress::Any);
 	}
 
 	qsPassword = qsSettings->value("serverpassword", qsPassword).toString();
@@ -206,6 +219,7 @@ void MetaParams::read(QString fname) {
 	iMaxBandwidth = qsSettings->value("bandwidth", iMaxBandwidth).toInt();
 	iDefaultChan = qsSettings->value("defaultchannel", iDefaultChan).toInt();
 	iMaxUsers = qsSettings->value("users", iMaxUsers).toInt();
+	iMaxUsersPerChannel = qsSettings->value("usersperchannel", iMaxUsersPerChannel).toInt();
 	qsWelcomeText = qsSettings->value("welcometext", qsWelcomeText).toString();
 	bCertRequired = qsSettings->value("certrequired", bCertRequired).toBool();
 
@@ -437,6 +451,26 @@ bool Meta::boot(int srvnum) {
 	}
 	qhServers.insert(srvnum, s);
 	emit started(s);
+
+#ifdef Q_OS_UNIX
+	int sockets = 19; // Base
+	foreach(s, qhServers) {
+		sockets += 11; // Listen sockets, signal pipes etc.
+		sockets += s->iMaxUsers; // One per user
+	}
+
+	struct rlimit r;
+	if (getrlimit(RLIMIT_NOFILE, &r) == 0) {
+		if (r.rlim_cur < r.rlim_max) {
+			r.rlim_cur = r.rlim_max;
+			setrlimit(RLIMIT_NOFILE, &r);
+			getrlimit(RLIMIT_NOFILE, &r);
+		}
+		if (r.rlim_cur < sockets)
+			qCritical("Current booted servers require minimum %d file descriptors when all slots are full, but only %d file descriptors are allowed for this process. Your server will crash and burn; read the FAQ for details.", sockets, r.rlim_cur);
+	}
+#endif
+
 	return true;
 }
 
