@@ -44,11 +44,13 @@ struct MetaParams {
 	int iTimeout;
 	int iMaxBandwidth;
 	int iMaxUsers;
+	int iMaxUsersPerChannel;
 	int iDefaultChan;
 	int iMaxTextMessageLength;
 	bool bAllowHTML;
 	QString qsPassword;
 	QString qsWelcomeText;
+	bool bCertRequired;
 
 	int iBanTries;
 	int iBanTimeframe;
