@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -45,6 +45,11 @@ using namespace Murmur;
 
 static MurmurIce *mi = NULL;
 static Ice::ObjectPtr iopServer;
+static Ice::PropertiesPtr ippProperties;
+
+void IceParse(int &argc, char *argv[]) {
+	ippProperties = Ice::createProperties(argc, argv);
+}
 
 void IceStart() {
 	mi = new MurmurIce();
@@ -98,10 +103,9 @@ static void channelToChannel(const ::Channel *c, Murmur::Channel &mc) {
 	mc.id = c->iId;
 	mc.name = u8(c->qsName);
 	mc.parent = c->cParent ? c->cParent->iId : -1;
-	mc.links.clear();
-	if (! c->qsDesc.isEmpty())
-		mc.description = u8(c->qsDesc);
+	mc.description = u8(c->qsDesc);
 	mc.position = c->iPosition;
+	mc.links.clear();
 	foreach(::Channel *chn, c->qsPermLinks)
 		mc.links.push_back(chn->iId);
 	mc.temporary = c->bTemporary;
@@ -181,8 +185,25 @@ MurmurIce::MurmurIce() {
 	if (meta->mp.qsIceEndpoint.isEmpty())
 		return;
 
+	Ice::PropertiesPtr ipp = Ice::createProperties();
+
+	::Meta::mp.qsSettings->beginGroup("Ice");
+	foreach(const QString &v, ::Meta::mp.qsSettings->childKeys()) {
+		ipp->setProperty(u8(v), u8(::Meta::mp.qsSettings->value(v).toString()));
+	}
+	::Meta::mp.qsSettings->endGroup();
+
+	Ice::PropertyDict props = ippProperties->getPropertiesForPrefix("");
+	Ice::PropertyDict::iterator i;
+	for (i=props.begin(); i != props.end(); ++i) {
+		ipp->setProperty((*i).first, (*i).second);
+	}
+
+	Ice::InitializationData idd;
+	idd.properties = ipp;
+
 	try {
-		communicator = Ice::initialize();
+		communicator = Ice::initialize(idd);
 		adapter = communicator->createObjectAdapterWithEndpoints("Murmur", qPrintable(meta->mp.qsIceEndpoint));
 		MetaPtr m = new MetaI;
 		MetaPrx mprx = MetaPrx::uncheckedCast(adapter->add(m, communicator->stringToIdentity("Meta")));
@@ -467,14 +488,27 @@ void MurmurIce::nameToIdSlot(int &id, const QString &name) {
 	}
 }
 
-void MurmurIce::authenticateSlot(int &res, QString &uname, const QString &pw) {
+void MurmurIce::authenticateSlot(int &res, QString &uname, const QList<QSslCertificate> &certlist, const QString &certhash, bool certstrong, const QString &pw) {
 	::Server *server = qobject_cast< ::Server *> (sender());
 
 	ServerAuthenticatorPrx prx = mi->qmServerAuthenticator.value(server->iServerNum);
 	::std::string newname;
 	::Murmur::GroupNameList groups;
+	::Murmur::CertificateList certs;
+
+	certs.resize(certlist.size());
+	for (int i=0;i<certlist.size();++i) {
+		::Murmur::CertificateDer der;
+		QByteArray qba = certlist.at(i).toDer();
+		der.resize(qba.size());
+		const char *ptr = qba.constData();
+		for (int j=0;j<qba.size();++j)
+			der[j] = ptr[j];
+		certs[i] = der;
+	}
+
 	try {
-		res = prx->authenticate(u8(uname), u8(pw), newname, groups);
+		res = prx->authenticate(u8(uname), u8(pw), certs, u8(certhash), certstrong, newname, groups);
 	} catch (...) {
 		badAuthenticator(server);
 	}
@@ -770,11 +804,11 @@ static void impl_Server_setSuperuserPassword(const ::Murmur::AMD_Server_setSuper
 static void impl_Server_getLog(const ::Murmur::AMD_Server_getLogPtr cb, int server_id,  ::Ice::Int min,  ::Ice::Int max) {
 	NEED_SERVER_EXISTS;
 
-	::Murmur::LogEntry le;
 	::Murmur::LogList ll;
 
 	QList<ServerDB::LogRecord> dblog = ServerDB::getLog(server_id, min, max);
 	foreach(const ServerDB::LogRecord &e, dblog) {
+		::Murmur::LogEntry le;
 		logToLog(e, le);
 		ll.push_back(le);
 	}
@@ -784,8 +818,8 @@ static void impl_Server_getLog(const ::Murmur::AMD_Server_getLogPtr cb, int serv
 static void impl_Server_getUsers(const ::Murmur::AMD_Server_getUsersPtr cb, int server_id) {
 	NEED_SERVER;
 	::Murmur::UserMap pm;
-	::Murmur::User mp;
 	foreach(const ::User *p, server->qhUsers) {
+		::Murmur::User mp;
 		if (static_cast<const ServerUser *>(p)->sState == ::ServerUser::Authenticated) {
 			userToUser(p, mp);
 			pm[p->uiSession] = mp;
@@ -797,8 +831,8 @@ static void impl_Server_getUsers(const ::Murmur::AMD_Server_getUsersPtr cb, int
 static void impl_Server_getChannels(const ::Murmur::AMD_Server_getChannelsPtr cb, int server_id) {
 	NEED_SERVER;
 	::Murmur::ChannelMap cm;
-	::Murmur::Channel mc;
 	foreach(const ::Channel *c, server->qhChannels) {
+		::Murmur::Channel mc;
 		channelToChannel(c, mc);
 		cm[c->iId] = mc;
 	}
@@ -840,11 +874,32 @@ static void impl_Server_getTree(const ::Murmur::AMD_Server_getTreePtr cb, int se
 	cb->ice_response(recurseTree(server->qhChannels.value(0)));
 }
 
+static void impl_Server_getCertificateList(const ::Murmur::AMD_Server_getCertificateListPtr cb, int server_id, ::Ice::Int session) {
+	NEED_SERVER;
+	NEED_PLAYER;
+
+	::Murmur::CertificateList certs;
+
+	const QList<QSslCertificate> &certlist = user->peerCertificateChain();
+
+	certs.resize(certlist.size());
+	for (int i=0;i<certlist.size();++i) {
+		::Murmur::CertificateDer der;
+		QByteArray qba = certlist.at(i).toDer();
+		der.resize(qba.size());
+		const char *ptr = qba.constData();
+		for (int j=0;j<qba.size();++j)
+			der[j] = ptr[j];
+		certs[i] = der;
+	}
+	cb->ice_response(certs);
+}
+
 static void impl_Server_getBans(const ::Murmur::AMD_Server_getBansPtr cb, int server_id) {
 	NEED_SERVER;
 	::Murmur::BanList bl;
-	::Murmur::Ban mb;
 	foreach(const ::Ban &ban, server->qlBans) {
+		::Murmur::Ban mb;
 		banToBan(ban, mb);
 		bl.push_back(mb);
 	}
@@ -978,8 +1033,10 @@ static void impl_Server_setChannelState(const ::Murmur::AMD_Server_setChannelSta
 	int channelid = state.id;
 	NEED_SERVER;
 	NEED_CHANNEL;
-	::Channel *np;
-	NEED_CHANNEL_VAR(np, state.parent);
+	::Channel *np = NULL;
+	if (channel->iId != 0) {
+		NEED_CHANNEL_VAR(np, state.parent);
+	}
 
 	QString qsName = u8(state.name);
 
@@ -1261,6 +1318,64 @@ static void impl_Server_setTexture(const ::Murmur::AMD_Server_setTexturePtr cb,
 		cb->ice_response();
 }
 
+static void impl_Server_addUserToGroup(const ::Murmur::AMD_Server_addUserToGroupPtr cb, int server_id, ::Ice::Int channelid,  ::Ice::Int session,  const ::std::string& group) {
+	NEED_SERVER;
+	NEED_PLAYER;
+	NEED_CHANNEL;
+
+	QString qsgroup = u8(group);
+	if (qsgroup.isEmpty()) {
+		cb->ice_exception(InvalidChannelException());
+		return;
+	}
+
+	::Group *g = channel->qhGroups.value(qsgroup);
+	if (! g)
+		g = new ::Group(channel, qsgroup);
+
+	g->qsTemporary.insert(- session);
+	server->clearACLCache(user);
+
+	cb->ice_response();
+}
+
+static void impl_Server_removeUserFromGroup(const ::Murmur::AMD_Server_removeUserFromGroupPtr cb, int server_id,  ::Ice::Int channelid,  ::Ice::Int session,  const ::std::string& group) {
+	NEED_SERVER;
+	NEED_PLAYER;
+	NEED_CHANNEL;
+
+	QString qsgroup = u8(group);
+	if (qsgroup.isEmpty()) {
+		cb->ice_exception(InvalidChannelException());
+		return;
+	}
+
+	::Group *g = channel->qhGroups.value(qsgroup);
+	if (! g)
+		g = new ::Group(channel, qsgroup);
+
+	g->qsTemporary.remove(- session);
+	server->clearACLCache(user);
+
+	cb->ice_response();
+}
+
+static void impl_Server_redirectWhisperGroup(const ::Murmur::AMD_Server_redirectWhisperGroupPtr cb, int server_id,  ::Ice::Int session,  const ::std::string& source,  const ::std::string& target) {
+	NEED_SERVER;
+	NEED_PLAYER;
+
+	QString qssource = u8(source);
+	QString qstarget = u8(target);
+	if (qstarget.isEmpty())
+		user->qmWhisperRedirect.remove(qssource);
+	else
+		user->qmWhisperRedirect.insert(qssource, qstarget);
+
+	server->clearACLCache(user);
+
+	cb->ice_response();
+}
+
 static void impl_Meta_getServer(const ::Murmur::AMD_Meta_getServerPtr cb, const Ice::ObjectAdapterPtr adapter, ::Ice::Int id) {
 	QList<int> server_list = ServerDB::getAllServers();
 	if (! server_list.contains(id))
