@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -303,6 +303,7 @@ void Server::readParams() {
 	iTimeout = Meta::mp.iTimeout;
 	iMaxBandwidth = Meta::mp.iMaxBandwidth;
 	iMaxUsers = Meta::mp.iMaxUsers;
+	iMaxUsersPerChannel = Meta::mp.iMaxUsersPerChannel;
 	iMaxTextMessageLength = Meta::mp.iMaxTextMessageLength;
 	bAllowHTML = Meta::mp.bAllowHTML;
 	iDefaultChan = Meta::mp.iDefaultChan;
@@ -314,6 +315,7 @@ void Server::readParams() {
 	qurlRegWeb = Meta::mp.qurlRegWeb;
 	bBonjour = Meta::mp.bBonjour;
 	bAllowPing = Meta::mp.bAllowPing;
+	bCertRequired = Meta::mp.bCertRequired;
 	qrUserName = Meta::mp.qrUserName;
 	qrChannelName = Meta::mp.qrChannelName;
 
@@ -349,6 +351,7 @@ void Server::readParams() {
 	iTimeout = getConf("timeout", iTimeout).toInt();
 	iMaxBandwidth = getConf("bandwidth", iMaxBandwidth).toInt();
 	iMaxUsers = getConf("users", iMaxUsers).toInt();
+	iMaxUsersPerChannel = getConf("usersperchannel", iMaxUsersPerChannel).toInt();
 	iMaxTextMessageLength = getConf("textmessagelength", iMaxTextMessageLength).toInt();
 	bAllowHTML = getConf("allowhtml", bAllowHTML).toBool();
 	iDefaultChan = getConf("defaultchannel", iDefaultChan).toInt();
@@ -360,6 +363,7 @@ void Server::readParams() {
 	qurlRegWeb = QUrl(getConf("registerurl", qurlRegWeb.toString()).toString());
 	bBonjour = getConf("bonjour", bBonjour).toBool();
 	bAllowPing = getConf("allowping", bAllowPing).toBool();
+	bCertRequired = getConf("certrequired", bCertRequired).toBool();
 
 	qrUserName=QRegExp(getConf("username", qrUserName.pattern()).toString());
 	qrChannelName=QRegExp(getConf("channelname", qrChannelName.pattern()).toString());
@@ -376,6 +380,8 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		iMaxBandwidth = i ? i : Meta::mp.iMaxBandwidth;
 	else if (key == "users")
 		iMaxUsers = i ? i : Meta::mp.iMaxUsers;
+	else if (key == "usersperchannel")
+		iMaxUsersPerChannel = i ? i : Meta::mp.iMaxUsersPerChannel;
 	else if (key == "textmessagelength")
 		iMaxTextMessageLength = i ? i : Meta::mp.iMaxTextMessageLength;
 	else if (key == "allowhtml")
@@ -392,6 +398,8 @@ void Server::setLiveConf(const QString &key, const QString &value) {
 		qsRegHost = !v.isNull() ? v : Meta::mp.qsRegHost;
 	else if (key == "registerurl")
 		qurlRegWeb = !v.isNull() ? v : Meta::mp.qurlRegWeb;
+	else if (key == "certrequired")
+		bCertRequired = !v.isNull() ? QVariant(v).toBool() : Meta::mp.bCertRequired;
 	else if (key == "bonjour") {
 		bBonjour = !v.isNull() ? QVariant(v).toBool() : Meta::mp.bBonjour;
 #ifdef USE_BONJOUR
@@ -434,21 +442,28 @@ BandwidthRecord::BandwidthRecord() {
 		a_iBW[i] = 0;
 }
 
-void BandwidthRecord::addFrame(int size) {
-	iSum -= a_iBW[iRecNum];
-	a_iBW[iRecNum] = static_cast<unsigned char>(size);
-	iSum += a_iBW[iRecNum];
+bool BandwidthRecord::addFrame(int size, int maxpersec) {
+	quint64 elapsed = a_qtWhen[iRecNum].elapsed();
 
+	if (elapsed == 0)
+		return false;
+
+	int nsum = iSum-a_iBW[iRecNum]+size;
+	int bw = static_cast<int>((nsum * 1000000LL) / elapsed);
+
+	if (bw > maxpersec)
+		return false;
+
+	a_iBW[iRecNum] = static_cast<unsigned short>(size);
 	a_qtWhen[iRecNum].restart();
 
+	iSum = nsum;
+
 	iRecNum++;
 	if (iRecNum == N_BANDWIDTH_SLOTS)
 		iRecNum = 0;
-}
 
-int BandwidthRecord::bytesPerSec() const {
-	quint64 elapsed = a_qtWhen[iRecNum].elapsed();
-	return static_cast<int>((iSum * 1000000LL) / elapsed);
+	return true;
 }
 
 int BandwidthRecord::onlineSeconds() const {
@@ -737,8 +752,8 @@ void Server::sendMessage(ServerUser *u, const char *data, int len, QByteArray &c
 }
 
 #define SENDTO \
-		if (! pDst->bDeaf && ! pDst->bSelfDeaf && (pDst != u)) { \
-			if (poslen && pDst->ssContext == u->ssContext) \
+		if ((!pDst->bDeaf) && (!pDst->bSelfDeaf) && (pDst != u)) { \
+			if ((poslen > 0) && (pDst->ssContext == u->ssContext)) \
 				sendMessage(pDst, buffer, len, qba); \
 			else \
 				sendMessage(pDst, buffer, len - poslen, qba_npos); \
@@ -762,9 +777,8 @@ void Server::processMsg(ServerUser *u, const char *data, int len) {
 
 	// IP + UDP + Crypt + Data
 	int packetsize = 20 + 8 + 4 + len;
-	bw->addFrame(packetsize);
 
-	if ((bw->bytesPerSec() * 8)> iMaxBandwidth) {
+	if (! bw->addFrame(packetsize, iMaxBandwidth/8)) {
 		// Suppress packet.
 		return;
 	}
@@ -929,6 +943,16 @@ void Server::newClient() {
 
 		HostAddress ha(adr);
 
+		QList<Ban> tmpBans = qlBans;
+		foreach(const Ban &ban, qlBans) {
+			if (ban.isExpired())
+				tmpBans.removeOne(ban);
+		}
+		if (qlBans.count() != tmpBans.count()) {
+			qlBans = tmpBans;
+			saveBans();
+		}
+
 		foreach(const Ban &ban, qlBans) {
 			if (ban.haAddress.match(ha, ban.iMask)) {
 				log(QString("Ignoring connection: %1 (Server ban)").arg(addressToString(sock->peerAddress(), sock->peerPort())));
@@ -1227,7 +1251,7 @@ void Server::removeChannel(Channel *chan, Channel *dest) {
 		mpus.set_channel_id(dest->iId);
 		sendAll(mpus);
 
-		userEnterChannel(p, dest, true);
+		userEnterChannel(p, dest);
 	}
 
 	MumbleProto::ChannelRemove mpcr;
@@ -1461,38 +1485,44 @@ bool Server::validateChannelName(const QString &name) {
 }
 
 void Server::recheckCodecVersions() {
-	QMap<int, unsigned int> qm;
+	QMap<int, unsigned int> qmCodecUsercount;
 	QMap<int, unsigned int>::const_iterator i;
 	int users = 0;
+	// Count how many users use which codec
 	foreach(ServerUser *u, qhUsers) {
 		if (u->qlCodecs.isEmpty())
 			continue;
 
 		++users;
 		foreach(int version, u->qlCodecs)
-			++ qm[version];
+			++ qmCodecUsercount[version];
 	}
 
 	if (! users)
 		return;
 
+	// Find the best possible codec most users support
 	int version = 0;
-	unsigned int maxu = 0;
-	i = qm.constEnd();
+	int maximum_users = 0;
+	i = qmCodecUsercount.constEnd();
 	do {
 		--i;
-		if (i.value() > maxu) {
+		if (i.value() > maximum_users) {
 			version = i.key();
-			maxu = i.value();
+			maximum_users = i.value();
 		}
-	} while (i != qm.constBegin());
+	} while (i != qmCodecUsercount.constBegin());
 
-	int cversion = bPreferAlpha ? iCodecAlpha : iCodecBeta;
-	if (cversion == version)
+	int current_version = bPreferAlpha ? iCodecAlpha : iCodecBeta;
+	if (current_version == version)
 		return;
 
 	MumbleProto::CodecVersion mpcv;
 
+	// If we don't already use the compat bitstream version set
+	// it as alpha and announce it. If another codec now got the
+	// majority set it as the opposite of the currently valid bPreferAlpha
+	// and announce it.
 	if (version == static_cast<qint32>(0x8000000a))
 		bPreferAlpha = true;
 	else
