@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
    Copyright (C) 2009, Stefan Hacker <dd0t@users.sourceforge.net>
 
    All rights reserved.
@@ -60,12 +60,11 @@ struct BandwidthRecord {
 	int iRecNum;
 	int iSum;
 	Timer qtFirst;
-	unsigned char a_iBW[N_BANDWIDTH_SLOTS];
+	unsigned short a_iBW[N_BANDWIDTH_SLOTS];
 	Timer a_qtWhen[N_BANDWIDTH_SLOTS];
 
 	BandwidthRecord();
-	void addFrame(int size);
-	int bytesPerSec() const;
+	bool addFrame(int size, int maxpersec);
 	int onlineSeconds() const;
 	int idleSeconds() const;
 	int bandwidth() const;
@@ -189,6 +188,7 @@ class Server : public QThread {
 		int iTimeout;
 		int iMaxBandwidth;
 		int iMaxUsers;
+		int iMaxUsersPerChannel;
 		int iDefaultChan;
 		int iMaxTextMessageLength;
 		bool bAllowHTML;
