@@ -1,4 +1,4 @@
-/* Copyright (C) 2005-2009, Thorvald Natvig <thorvald@natvig.com>
+/* Copyright (C) 2005-2010, Thorvald Natvig <thorvald@natvig.com>
 
    All rights reserved.
 
@@ -178,7 +178,12 @@ void UnixMurmur::handleSigHup() {
 	::read(iHupFd[1], &tmp, sizeof(tmp));
 
 	if (! qfLog || ! qfLog->isOpen()) {
-		qWarning("Caught SIGHUP, but logfile not in use");
+		if (qfLog) {
+			qWarning("Caught SIGHUP, but logfile not in use -- interpreting as hint to quit");
+			QCoreApplication::instance()->quit();
+		} else {
+			qWarning("Caught SIGHUP, but logfile not in use");
+		}
 	} else {
 		qWarning("Caught SIGHUP, will reopen %s", qPrintable(Meta::mp.qsLogfile));
 		qfLog->close();
@@ -277,7 +282,7 @@ void UnixMurmur::finalcap() {
 	if (getrlimit(RLIMIT_RTPRIO, &r) != 0) {
 		qCritical("Failed to get priority limits.");
 	} else {
-		qWarning("Was %d %d", r.rlim_cur, r.rlim_max);
+		qWarning("Resource limits were %ld %ld", r.rlim_cur, r.rlim_max);
 		r.rlim_cur = r.rlim_max = 1;
 		if (setrlimit(RLIMIT_RTPRIO, &r) != 0) {
 			qCritical("Failed to set priority limits.");
