@@ -39,6 +39,7 @@
 #include "DBus.h"
 #include "Meta.h"
 #include "Version.h"
+#include "SSL.h"
 
 #ifdef Q_OS_UNIX
 #include "UnixMurmur.h"
@@ -123,6 +124,7 @@ static void murmurMessageOutput(QtMsgType type, const char *msg) {
 }
 
 #ifdef USE_ICE
+void IceParse(int &argc, char *argv[]);
 void IceStart();
 void IceStop();
 #endif
@@ -142,6 +144,10 @@ int main(int argc, char **argv) {
 
 	int res;
 
+#ifdef USE_ICE
+	IceParse(argc, argv);
+#endif
+
 #ifdef Q_OS_WIN
 	QApplication a(argc, argv);
 	a.setQuitOnLastWindowClosed(false);
@@ -186,9 +192,11 @@ int main(int argc, char **argv) {
 
 	QString inifile;
 	QString supw;
-	bool readPw = false;
 	bool wipeSsl = false;
 	int sunum = 1;
+#ifndef Q_OS_WIN
+	bool readPw = false;
+#endif
 
 	qInstallMsgHandler(murmurMessageOutput);
 
@@ -261,7 +269,9 @@ int main(int argc, char **argv) {
 		}
 	}
 
+
 	Meta::mp.read(inifile);
+	MumbleSSL::addSystemCA();
 
 #ifdef Q_OS_UNIX
 	unixhandler.setuid();
@@ -402,11 +412,11 @@ int main(int argc, char **argv) {
 #ifdef USE_ICE
 	IceStart();
 #endif
+
 	meta->getOSInfo();
 
 	int major, minor, patch;
 	QString strver;
-
 	meta->getVersion(major, minor, patch, strver);
 
 	qWarning("Murmur %d.%d.%d (%s) running on %s: %s: Booting servers", major, minor, patch, qPrintable(strver), qPrintable(meta->qsOS), qPrintable(meta->qsOSVersion));
